#ifndef GLOBAL_HPP
#define GLOBAL_HPP

#include "singleton.hpp"
#include "functor.hpp"

#include "extra_menu.hpp"
#include "plugins/input/input.hpp"

// FIXME: ugly as hell
#include "plugins/feature/audio/players/audio_player.hpp"
#include "plugins/input/input.hpp"

#include <vector>
#include <string>

class GlobalCommandBase
{
public:
  std::string command;
  bool check_func_base(const Input& i) const;
};

class GlobalCommand : public GlobalCommandBase
{
public:
  GlobalCommand(TimeFv *f, InputCheckBase *icb)
    : call_back_f(f), check_func(icb)
  {}

  GlobalCommand(TimeFv *f, const std::string& c, const std::string& n)
    : name(n), call_back_f(f), check_func(0)
  {
    command = c;
  }

  std::string name;
  TimeFv *call_back_f;
  InputCheckBase *check_func;
};

class Playback
{
public:
  Playback(const std::string& name);

  // possible commands are: play, pause, stop, ff, fb
  void add_command(const GlobalCommand& c);
  // check input against registered playback commands
  bool check_commands(const Input& i) const;

  void enable();
  void disable();

  void clean_up();

  std::string name() const;
  
  std::vector<GlobalCommand> commands;

private:
  bool enabled;
  std::string playback_name;
};

class Movie;
class LcdPrint;

// Global controls the GlobalCommands and thus it will automaticly delete
// them, when they are not needed anymore

class Global
{
 public:
  void clean_up();

  // add a global command that all modules will check
  void add_command(const GlobalCommand& c);
  // check input against all global commands
  bool check_commands(const Input& i, bool disable_mode_change = false);

  void add_playback(const Playback& p);
  // sets the playback to the player with the specified name
  bool set_playback(const std::string& name);
  bool is_playback(const std::string& name);

  // any plugin can offset the playback representation in the vertical direction
  void set_playback_offset(int offset);
  int get_playback_offset();

  // callback for reconfiguring device on change
  void reconfigure_playback();
  void register_reconfigure(TimeFunctorv<AudioPlayer>* callback);

  // for cancelation
  void reset_stop_bit();
  void stop_bit_input_check(const Input& i);
  bool check_stop_bit();

  // remote controls
  void set_valid_keys(const std::string& keys);
  std::string get_valid_keys();
  
  // lcd output functions
  bool lcd_output_possible();
  void lcd_add_output(const std::string& message);
  void lcd_print();

  void register_lcd_output_func(ConstFunctor<LcdPrint, std::string, void>* lcd_output);
  void register_lcd_print_output_func(TimeFunctorv<LcdPrint>* lcd_output);

  // movie playback functions
  bool movie_playback_possible();
  void play_movie(const std::string& path);

  void register_movie_playback_func(ConstFunctor<Movie, std::string, void>* movie_playback);

  // global menu items
  void add_second_menu_item(const ExtraMenuItem& e);
  std::vector<ExtraMenuItem> menu_items;

  Global();

 private:
  void playlist_commands();

  // cancelation
  bool cancel;

  // remote
  std::string valid_keys;

  std::vector<GlobalCommand> commands;
  std::vector<Playback> playbacks;
  std::string playback_name;

  int offset;

  TimeFunctorv<AudioPlayer>* reconfigure_callback;

  // LCD

  ConstFunctor<LcdPrint, std::string, void>* lcd_output_func;
  TimeFunctorv<LcdPrint>* lcd_print_output_func;
  bool registered_lcd_output;

  // Movie

  ConstFunctor<Movie, std::string, void>* movie_playback_func;
  bool registered_movie_playback;
};

typedef Singleton<Global> S_Global;

#endif
