#include "config.h"

#include "options.hpp"

#include "common.hpp"
#include "graphics.hpp"
#include "global.hpp"

#include "plugins/plugins.hpp"

#include "gettext.hpp"
#include "shutdown.hpp"

#include <fstream>

// mkdir()
#include <sys/stat.h>
#include <sys/types.h>

using std::list;
using std::vector;
using std::string;

Options::Options()
{
  im = S_InputMaster::get_instance();
  themes = S_Themes::get_instance();
  render = S_Render::get_instance();
  conf = S_Config::get_instance();
  go = S_GlobalOptions::get_instance();
}

bool Options::mainloop() 
{
  Global *global = S_Global::get_instance();

  im->save_map();
  im->set_map("default");

  Shutdown *sd = S_Shutdown::get_instance();
  
  Input input;

  vector<Option*>::iterator iter = val.begin();
  int pos = 0; // position in values, used by print()

  // set the element to the first non-type element
  do {
    vector<Option*>::iterator j = val.end(); --j;
    if (iter != j) {
      ++iter; ++pos;
    } else {
      iter = val.begin(); pos = 0;
    }
  } while ((*iter)->type);

  bool reload = false, reload_all = false, reconfigure = false;
  int sort_pos = 0;

  bool update_needed = true;

  while (!(input.command == "back" && input.mode == "general"))
    {
      if (update_needed) {
	print((*iter)->name, pos);
	print_lcd_menu((*iter)->name);
      }

      input = im->device->get_input();
      
      if (sd->is_enabled()) {
      	sd->cancel();
      	continue;
      }

      update_needed = true;

      if (input.command == "prev")
 	{
	  do {
	    if (iter != val.begin()) {
	      --iter; --pos;
	    } else {
	      vector<Option*>::iterator j = val.end(); --j;
	      iter = j; pos = val.size() - 1;
	    }
	  } while ((*iter)->type);
 	}
      else if (input.command == "next")
 	{
	  do {
	    vector<Option*>::iterator j = val.end(); --j;
	    if (iter != j) {
	      ++iter; ++pos;
	    } else {
	      iter = val.begin(); pos = 0;
	    }
	  } while ((*iter)->type);
 	}
      else if (((*iter)->name == gettext("reload")) &&
	       (input.command == "left" || input.command == "right" || input.command == "action"))
	{
	  read_dirs();
	  reload = true;
	}
      else if (input.command == "left" || input.command == "right") 
	{
	  if (((*iter)->name == gettext("sorting")) &&
	      (input.command == "left" || input.command == "right"))
	    if (!reload_all) {
	      sort_pos = (*iter)->pos;
	      reload_all = true;
	    } 
	  if (((*iter)->name == gettext("audio device")) &&
	      (input.command == "left" || input.command == "right"))
	    reconfigure = true;

	  if (input.command == "left") {
	    if ((*iter)->pos != 0) {
	      --(*iter)->pos;
	    } else {
	      (*iter)->pos = (*iter)->values.size() - 1;
	    }
	  }
	  else if (input.command == "right") {
	    if ((*iter)->pos != ((*iter)->values.size() - 1)) {
	      ++(*iter)->pos;
	    } else {
	      (*iter)->pos = 0;
	    }
	  }

	  if (sort_pos == (*iter)->pos)
	    reload_all = false;
	}
      else
	update_needed = !global->check_commands(input);
    }

  im->restore_map();

  if (reconfigure)
    global->reconfigure_playback();

  if (reload_all) {
    // FIXME: run read dirs on all feature modules
    return true;
  } else
    return reload;
}

void Options::print_element(const string& element, const string& value,
			    bool marked, int x, int y, int max_size, int max_element_size)
{
  if (marked) {
    render->current.add(new PObj(themes->options_marked_left, x-50, y, 1, NOSCALING));
    render->current.add(new PObj(themes->options_marked, x-25, y, 90+max_size, 1));
    render->current.add(new PObj(themes->options_marked_right, x-25+90+max_size, y, 1, NOSCALING));
  }
  
  render->current.add(new TObj(element, "Vera/17", x, y, themes->options_font1, 
			       themes->options_font2, themes->options_font3, 2));
  render->current.add(new TObj(value, "Vera/17", x + max_element_size + 50, y,
			       themes->options_font1, themes->options_font2, 
			       themes->options_font3, 2));
}

void Options::print(const string& marking, int pos)
{
  render->prepare_new_image();

  render->current.add(new PObj(themes->options_background, 0, 0, 0, SCALE_FULL));
  
  render->current.add(new PObj(themes->startmenu_options, 25, 10, 2, NOSCALING));
 
  if (themes->show_header) {
    render->current.add(new TObj(header, "Vera/28", 100, 10,
				 themes->options_header_font1, themes->options_header_font2, 
				 themes->options_header_font3, 2));
  }

  int y = (conf->p_v_res()>800)?105:70;

  bool direction_down = true;
  int space = vector_lookup(val, pos)->type ? 50 : 25;
  int max_space = conf->p_h_res()-(2*y);
  int divert_pos_down = 0, divert_pos_up = 0;
  int cur_pos = 0, nr_elements = 0;
  bool direction_up_ok = true, direction_down_ok = true;

  if (header != gettext("Options"))
    y += 50;

  while (true) {
    if (!direction_down) {
      cur_pos = pos + divert_pos_up + 1;
      if (cur_pos <= val.size() - 1) {
	space += vector_lookup(val, cur_pos)->type ? 50 : 25;
	if (space > max_space)
	  break;
	nr_elements += 1;
	divert_pos_up += 1;
      } else
	direction_up_ok = false;

      direction_down = true;
    } else {
      cur_pos = pos - divert_pos_down - 1;
      if (cur_pos >= 0) {
	space += vector_lookup(val, cur_pos)->type ? 50 : 25;
	if (space > max_space)
	  break;
	nr_elements += 1;
	divert_pos_down += 1;
      } else
	direction_down_ok = false;

      direction_down = false;
    }

    if (!direction_up_ok && !direction_down_ok)
      break;
  }

  int max_element = 0, max_value = 0;

  // calculate size
  for (std::vector<Option*>::const_iterator i = val.begin(), 
	 end = val.end(); i != end; ++i) {

    Option *o = *i;

    if (o->name != gettext("reload") && !o->type) {

      int element_size = string_format::calculate_string_width(o->name, "Vera/17");

      for (vector<string>::const_iterator oval = o->values.begin(), 
	     oend = o->values.end(); oval != oend; ++oval) {
	int value_size = string_format::calculate_string_width(*oval, "Vera/17");
	max_value = std::max(value_size, max_value);
      }

      max_element = std::max(element_size, max_element);
    } else if (o->name == gettext("reload")) {
      int element_size = string_format::calculate_string_width(o->name, "Vera/17");
      max_element = std::max(element_size, max_element);
    }
  }

  if (max_element > conf->p_v_res()/2 - 20)
    max_element = conf->p_v_res()/2 - 20;
  
  if (max_value > conf->p_v_res()/2 - 20)
    max_value = conf->p_v_res()/2 - 20;

  int max = max_element + max_value;

  if (max == 0)
    max = 70;

  int x = (conf->p_v_res()-max)/2;

  if (header != gettext("Options"))
    print_element(gettext("reload"), "", (gettext("reload") == marking), x, y-50, max, max_element);
  
  int startpos = pos - divert_pos_down;
  
  for (int i = startpos, end = val.size(); i < end; ++i) {

    if (i - startpos > nr_elements)
      break;

    Option *o = vector_lookup(val, i);

    if (o->type) {

      y += 10;
      render->current.add(new TObj(o->name, "Vera/20", x-25, y,
				   themes->options_font1, themes->options_font2, 
				   themes->options_font3, 1));
      y += 40;

    } else if (o->name != gettext("reload")) {

      string name = o->name;
      string_format::format_to_size(name, "Vera/17", max_element, false);
      string value = vector_lookup(o->values, o->pos);
      string_format::format_to_size(value, "Vera/17", max_value, false);

      print_element(name, value, o->name == marking, x, y, max, max_element);
      y += 25;
    }
  }

  render->draw_and_release("options");
}

void Options::print_lcd_menu(const string& marking)
{
  Global *global = S_Global::get_instance();

  if (!global->lcd_output_possible())
    return;

  std::string lcd_header = "      Options";

  // FIXME: lcd size
  //    LcdConf *lcd_conf = LcdConf::get_instance();

  vector<Option*>::iterator j = val.begin();
  while ((*j)->name != marking) {
    ++j;
  }

  std::string cur_name = "> " + (*j)->name;
  if (!(*j)->type && (*j)->values.size() > 0)
    cur_name += ": " + vector_lookup((*j)->values, (*j)->pos);
  std::string prev_name = "", next_name = "";

  vector<Option*>::iterator jend = val.end();
  --jend;

  if (val.size() == 2) {

    j = val.begin();
    if ((*j)->name != marking) {
      ++j;
    }
    if (j == jend)
      j = val.begin();
    else
      ++j;

    next_name = (*j)->name;
    if (!(*j)->type && (*j)->values.size() > 0)
      next_name += ": " + vector_lookup((*j)->values, (*j)->pos);

  } else {

    j = val.begin();
    while ((*j)->name != marking) {
      ++j;
    }

    // prev
    if (j != val.begin()) {
      --j;
      prev_name = (*j)->name;
      if (!(*j)->type && (*j)->values.size() > 0)
	prev_name += ": " + vector_lookup((*j)->values, (*j)->pos);
      ++j;
    } else {
      j = jend;
      prev_name = (*j)->name;
      if (!(*j)->type && (*j)->values.size() > 0)
	prev_name += ": " + vector_lookup((*j)->values, (*j)->pos);
      j = val.begin();
    }

    // next
    if (j != jend)
      ++j;
    else
      j = val.begin();

    next_name = (*j)->name;
    if (!(*j)->type && (*j)->values.size() > 0)
      next_name += ": " + vector_lookup((*j)->values, (*j)->pos);
  }

  global->lcd_add_output(lcd_header);
  global->lcd_add_output(prev_name.substr(0, 20));
  global->lcd_add_output(cur_name.substr(0, 20));
  global->lcd_add_output(next_name.substr(0, 20));
  global->lcd_print();
}

void Options::save() 
{
  std::ofstream file;

  string options_dir = conf->p_var_data_dir() + "options/";

  if (!file_exists(options_dir)) {
    if (mkdir(options_dir.c_str(), 0777) != 0)
      std::cerr << gettext("Could not create options directory: ") << options_dir << std::endl;
  }

  string path = options_dir + get_english_header();

  file.open(path.c_str());
    
  if (!file) {
    DebugPrint perror(gettext("Could not write options to file ") + path,
		      Print::DEBUGGING, DebugPrint::CRITICAL, "OPTIONS");
  } else {
    for (vector<Option*>::const_iterator iter = val.begin(); iter != val.end(); ++iter)
      if (!(*iter)->type && (*iter)->name != gettext("reload"))
	file << (*iter)->english_name << "," << vector_lookup((*iter)->english_values, (*iter)->pos) << std::endl;
  }

  file.close();
}

void Options::load()
{
  string path = conf->p_var_data_dir() + "options/" + get_english_header();

  std::ifstream In(path.c_str());

  if(!In){
    DebugPrint perror(gettext("Could not open options file ") + path,
		      Print::DEBUGGING, DebugPrint::CRITICAL, "OPTIONS");
  } else {

    string name, value, str;
    int i;
    
    while (getline(In, str)) {

      string::size_type comma = str.find(",");
      
      name = str.substr(0,comma);
      value = str.substr(comma+1);
      
      for (vector<Option*>::const_iterator iter = val.begin(); 
	   iter != val.end(); ++iter)
	if (!(*iter)->type && (*iter)->english_name == name) {
	  i = 0;
	  for (vector<string>::const_iterator iter2 = (*iter)->english_values.begin(); 
	       iter2 != (*iter)->english_values.end(); ++iter2, ++i)
	    if (value == (*iter2)) {
	      (*iter)->pos = i;
	      break;
	    }
	}
    }
  }
}
