#ifndef AUDIO_HPP
#define AUDIO_HPP

#include "config.h"

#include "startmenu.hpp"
#include "module.hpp"
#include "simplefile.hpp"
#include "shuffle_list.hpp"
#include "options.hpp"
#include "singleton.hpp"
#include "common-feature.hpp"
#include "updater.hpp"
#include "rand.hpp"
#include "busy_indicator.hpp"
#include "shutdown.hpp"

#include "audio_s.hpp"
#include "shoutcast.hpp"

#include "audio_config.hpp"

// ostringstream
#include <sstream>

#include <fstream>

#include <stack>
#include <vector>
#include <cassert>

class Audio : public Module 
{
  friend class AudioPlayer;
  friend class Updater;
protected:

  Audio();

  // class definitions
  class AudioOpts : public Options 
  {
  public:
    AudioOpts();
    ~AudioOpts();
  
    std::string repeat() 
    {
      return val[2]->values[val[2]->pos];
    }

    std::string shutdown() 
    {
      return val[3]->values[val[3]->pos];
    }
    
    std::string shuffle() 
    {
      return val[4]->values[val[4]->pos];
    }

    std::string bitrate()
    {
      return val[5]->values[val[5]->pos];
    }
  
    std::string metadata_extract() 
    {
      return val[6]->values[val[6]->pos];
    }

    std::vector<Option*> values() 
    {
      return val;
    }

    void set_audio(Audio *a)
    { audio = a; }
  
  private:
    void read_dirs();

    // fixme extract_meta_p ?

    Option *reload_p, *repeat_p, *shuffle_p, *shutdown_p, *extract_meta_p, *bitrate_p;
    Audio *audio;
  };

  // commands
  void exit();
  // audio add specific
  virtual void enter_dir() = 0;
  void play_now();
  void play_track_now();
  void leave_dir();
  // playlist specific
  void save_playlist_func();
  // playlist
  virtual void play() = 0;
  void pause();
  void stop();
  void next_track();
  void prev_track();
  void ff();
  void fb();
  void vol_plus();
  void vol_minus();
  void mute();
  void toggle_playlist_helper();
  virtual void toggle_playlist() = 0;
  virtual void toggle_load_playlist() = 0;

  int playlist_pos_int;

  virtual void add_radio_track(std::vector<std::pair<std::string, std::string> >& cur_files,
			       std::vector<std::string>& print_files) = 0;

  void action_load_playlist();
  void delete_playlist();
  std::vector<std::string> saved_playlists;
  int saved_playlist_pos;
  bool returned, return_value;

  bool exit_loop;
  bool change_mode;

  bool printing_information;

  // radio
  bool in_genre;
  bool radio_update_needed;

  int radio_pos;
  int old_radio_pos;

  std::string format_time(int cur_time, int total_time);
  std::string format_time(int time);

  const int position_int();
  void set_position_int(int pos);

  void print_string_element(const std::string& r, const std::string& position, int y);

  // used for both radio + choose_playlist
  void print_list(const std::vector<std::string>& files, const std::string& position, 
		  const int int_position, bool playlist);

  // print helper functions
  void print_top();
  void print_buttom(int files_size);

  // print audiotrack helper
  virtual void print_audiotrack_helper(int &x_pos, std::string& buffer, 
				       std::string& album, std::string& artist, std::string& title) = 0;

  virtual void print_audio_screensaver_helper(int &x_pos, int &y, int max_x_pic, int max_y_pic) = 0;

  // helper function for fullscreen
  void print_audio_screensaver();

  int displayed_volume;

  void print_audiotrack();
  void print_lcd_audiotrack();
  int print_audio_time();
  
  int check_audio_time();
  virtual void check_audio_state() = 0;

  int check_audio_fullscreen();
  virtual void print_audio_fullscreen() = 0;

  int check_volume();
  void show_volume();
  void clean_up_volume();

  virtual void add_audio_cd() = 0;
  virtual void add_data_cd() = 0;

  virtual void cd_menu() = 0;

  bool mounted_media;

  // for saving information about how you traverse a filesystem a stack is a
  // much better solution. The first element is the dirs (cur_audio_dir before)
  // and the next is the position (audio_int_pos)
  std::stack<std::pair<std::list<std::string>, int > > folders;

  // used when adding files to the playlist
  int playlist_id;

  ShuffleList shuffle_list;

  // playlist
  bool choose_playlist();
  std::vector<std::string> list_playlists();
  virtual void save_playlist(const std::string& playlistfile, bool overwrite = false) = 0;
  virtual bool load_playlist(const std::string& playlistfile) = 0;
  virtual bool load_last_playlist() = 0;

  std::string mainloop() 
  { return ""; }

  AudioOpts opts;

  struct file_sort
  {
    bool operator()(const Simplefile& lhs, const Simplefile& rhs);
  };

  void set_audio_player();

  void add_playback_keys();

  // helper functions
  bool check_mode(const Input& input, bool disable_mode_change);
  bool check_playlist_mode(const Input& input, bool disable_mode_change);
  bool check_playlist_commands(const Input& input, bool disable_mode_change);
  bool check_update_time_on_keypress(const Input& input, bool disable_mode_change);
  void update_time_on_keypress();

  // external
  bool was_playing;
  bool was_started;
  Simplefile track;
  int pos;

  virtual void load_current_dirs() = 0;

  virtual int get_real_playlist_pos() = 0;

  bool radio_stations_downloaded;

  AudioConfig *audio_conf;

  bool disabled_busy_indicator;

  bool display_message;

public:

  void save_runtime_settings();

  // notify
  virtual void reparse_current_dir() = 0;

  bool play_now_warning;

  Audio_s *audio_state;

  AudioOpts* get_opts()
  {
    return &opts;
  }

#ifdef use_media
  void pos_umount();
  void check_mount_before(const audio_type& track);
  void check_mount_after(const audio_type& track);
#endif

  virtual void prev_helper() = 0;
  virtual void next_helper() = 0;
     
  virtual int playlist_size() = 0;
  virtual int audiolist_size() = 0;
  virtual bool last_element_in_playlist() = 0;

  virtual void reset() = 0;
  virtual void read_dirs() = 0;

  void begin_external();
  void end_external();

  enum AudioMode { ADD, PLAY };

  AudioMode mode;

  // entry points
  virtual std::string mainloop(AudioMode mode) = 0;
  void detect_cdformat();
  void choose_radio();

  void startup_updater();
};

template <typename T>
class AudioTemplate : public Audio
{
protected:
  AudioTemplate()
    : in_playlist(false), files(&audio_list)
  {
    reset();
  }

  ~AudioTemplate()
  {
    opts.save();

    // save the playlist
    save_playlist("last");
  }

  bool in_playlist;

  void mainloop_top(AudioMode new_mode)
  {
    if (new_mode == ADD) {
      input_master->set_map("audio");
      files = &audio_list;
      mode = ADD;
    } else {
      input_master->set_map("playlist");
      files = &playlist;
      mode = PLAY;
    }
  }

  virtual void secondary_menu_in_add(ExtraMenu &em) = 0;

  // commands
  void secondary_menu_helper(ExtraMenu &em)
  {
    for (std::vector<ExtraMenuItem>::const_iterator i = global->menu_items.begin(), 
	   end = global->menu_items.end(); i != end; ++i)
      em.add_persistent_item(*i);

    if (mode == ADD) {

      secondary_menu_in_add(em);

      if (vector_lookup(*files, position_int()).type != "dir" && folders.size() > 1)
	em.add_item(ExtraMenuItem(gettext("Add all tracks to playlist"), 
				  input_master->find_shortcut("add_all"), 
				  new TimeFunctorv<AudioTemplate>(this, &AudioTemplate<T>::add_all)));

      if (audio_conf->p_audio_mode_playnow() && 
	  vector_lookup(*files, position_int()).type == "dir")
	em.add_item(ExtraMenuItem(gettext("Play now"), 
				  input_master->find_shortcut("play_now"),
				  new TimeFunctorv<AudioTemplate>(this, &AudioTemplate<T>::play_now)));
      else if (vector_lookup(*files, position_int()).type != "dir")
	em.add_item(ExtraMenuItem(gettext("Play track now"), 
				  input_master->find_shortcut("play_now"),
				  new TimeFunctorv<AudioTemplate>(this, &AudioTemplate<T>::play_track_now)));

    } else {
      em.add_item(ExtraMenuItem(gettext("Play track"), 
				input_master->find_shortcut("action"), 
				new TimeFunctorv<AudioTemplate>(this, &AudioTemplate<T>::play_track)));
      em.add_item(ExtraMenuItem(gettext("Move track up"), 
				input_master->find_shortcut("move_up"), 
				new TimeFunctorv<AudioTemplate>(this, &AudioTemplate<T>::move_up)));
      em.add_item(ExtraMenuItem(gettext("Move track down"), 
				input_master->find_shortcut("move_down"), 
				new TimeFunctorv<AudioTemplate>(this, &AudioTemplate<T>::move_down)));
      em.add_item(ExtraMenuItem(gettext("Delete track"), 
				input_master->find_shortcut("delete"), 
				new TimeFunctorv<AudioTemplate>(this, &AudioTemplate<T>::delete_track)));
      em.add_item(ExtraMenuItem(gettext("Add track to queue"), 
				input_master->find_shortcut("queue"), 
				new TimeFunctorv<AudioTemplate>(this, &AudioTemplate<T>::queue_track)));
      em.add_item(ExtraMenuItem(gettext("Clear playlist"), 
				input_master->find_shortcut("clear_list"), 
				new TimeFunctorv<AudioTemplate>(this, &AudioTemplate<T>::clear_playlist)));
      em.add_item(ExtraMenuItem(gettext("Save playlist"),
				input_master->find_shortcut("save_playlist"), 
				new TimeFunctorv<AudioTemplate>(this, &AudioTemplate<T>::save_playlist_func)));
    }

    em.add_item(ExtraMenuItem(gettext("Load playlist"),
			      input_master->find_shortcut("playlist_mode"), 
			      new TimeFunctorv<Audio>(this, &Audio::toggle_load_playlist)));

    em.add_item(ExtraMenuItem(gettext("Search"), 
			      input_master->find_shortcut("search"), 
			      new TimeFunctorv<AudioTemplate>(this, &AudioTemplate<T>::search_func)));
    em.add_item(ExtraMenuItem(gettext("Options"), 
			      input_master->find_shortcut("options"), 
			      new TimeFunctorv<AudioTemplate>(this, &AudioTemplate<T>::options)));
    em.add_item(ExtraMenuItem(gettext("Return to startmenu"), 
			      input_master->find_shortcut("back"), 
			      new TimeFunctorv<AudioTemplate>(this, &AudioTemplate<T>::exit)));

    conf->s_sam(true);
  }

  void print_lcd_menu()
  {
    if (!global->lcd_output_possible())
      return;

    std::string header = (mode == ADD) ? "       Audio" : "      Playlist";

    // FIXME: lcd size
    //    LcdConf *lcd_conf = LcdConf::get_instance();

    // current
    std::string cur_name = get_short_name_from_file(vector_lookup(*files, position_int()));

    if (cur_name.size() > 18)
      cur_name = "> " + cur_name.substr(2);
    else
      cur_name = "> " + cur_name;

    std::string prev_name = "";

    if (files->size() > 2) {
      // prev
      int prev_pos = position_int() - 1;
      if (prev_pos == -1)
	prev_pos = files->size()-1;

      prev_name = get_short_name_from_file(vector_lookup(*files, prev_pos));
    }

    std::string next_name = "";

    if (files->size() > 1) {
      // next
      int next_pos = position_int() + 1;
      if (next_pos == files->size())
	next_pos = 0;

      next_name = get_short_name_from_file(vector_lookup(*files, next_pos));
    }

    global->lcd_add_output(header);
    global->lcd_add_output(prev_name);
    global->lcd_add_output(cur_name);
    global->lcd_add_output(next_name);
    global->lcd_print();
  }

  // notify
  void reparse_current_dir()
  {
    // process audio dir 
    load_current_dirs();

    // sanity check
    if (folders.top().second > audio_list.size()-1)
      folders.top().second = 0;
  }

  virtual std::string get_name_from_file(const T& d) = 0;
  virtual std::string get_short_name_from_file(const T& d) = 0;
  virtual void print(const std::vector<T>& files) = 0;

  bool search_compare(const T& t)
  {
    // optimization
    if (search_str.empty())
      return true;

    std::string name = get_name_from_file(t);

    return (string_format::lowercase(name.substr(0, search_str.size())) == lowercase_search_str);
  }

  virtual void search_func() = 0;

  virtual void options_reloaded() = 0;

  void options()
  {
    BusyIndicator::get_instance().idle();

    std::string random_mode_before = opts.shuffle();

    if (opts.mainloop()) {
      //reload

      load_current_dirs();

      input_master->set_map("audio");
      mode = ADD;
      files = &audio_list;

      set_position_int(0);

      options_reloaded();
    }

    if (opts.shuffle() != random_mode_before) {
      shuffle_list.randomize();
      audio_state->empty_played_tracks();
    }
  }

  void add_all()
  {
    std::string path = vector_lookup(*files, position_int()).path;
    std::string::size_type pos_backslash;

    if ((pos_backslash = path.rfind("/")) != std::string::npos) {

      std::string file_dir = path.substr(0, pos_backslash);

      std::vector<T> addfiles = add_dir(file_dir);

      for (typename std::vector<T>::const_iterator iter = addfiles.begin(); iter != addfiles.end(); ++iter)
	{
	  T p = *iter;
	  p.id = ++playlist_id;
	  playlist.push_back(p);
	  shuffle_list.add_track(p);

	  if (playlist.size() == 1)
	    // first element added so we need to set the pos
	    playlist_pos_int = 0;
	}

      if (addfiles.size() > 0)
	DialogWaitPrint pdialog(gettext("Added directory to playlist"), 1000);
    }
  }

  std::vector<T> get_audio_cd_list()
  {
    int tracksize = cd->get_nr_tracks_cd();

    // FIXME?
    //    audio_state->p->update_cdrom(cd->get_device());

    // add elements to the playlist

    std::vector<T> audiolist_new;

    T tmp_track;

    Overlay overlay = Overlay("dialog");

    DialogStepPrint pdialog(gettext("Looking up CD in database"), overlay);

    for (int i = 1; i < tracksize+1; ++i) {

      if (global->check_stop_bit()) {
	audiolist_new.clear();
	break;
      }

      tmp_track.id = ++id;

      std::ostringstream name;
      
#ifdef player_xine
      name << "cdda:/" << i;
#else // alsaplayer
      if (i < 10)
	name << "Track 0" << i << ".cdda";
      else
	name << "Track " << i << ".cdda";
#endif

      audio_state->p->collect_info(name.str());

      if (audio_state->p->p_artist() == "" && audio_state->p->p_album() == "" && audio_state->p->p_title() == "") {
	tmp_track.name = name.str().substr(0, 8);
      } else
	tmp_track.name = audio_state->p->p_artist() + " - " + audio_state->p->p_album() + " - " + audio_state->p->p_title();

      tmp_track.lowercase_name = string_format::lowercase(tmp_track.name);

      tmp_track.path = name.str(); 
      tmp_track.type="media-track"; 
      
      audio_list.push_back(tmp_track);
    }

    pdialog.print_step2();

    return audiolist_new;
  }

  void add_radio_track(std::vector<std::pair<std::string, std::string> >& cur_files, 
		       std::vector<std::string>& print_files)
  {
    Shoutcast *shoutcast = Shoutcast::get_instance();

    if (radio_pos == 0 && in_genre)
      {
	// self-defined radio stations
	
	print_files.clear();
	cur_files.clear();
	      
	std::list<std::pair<std::string, std::string> > radio_stations = audio_conf->p_internet_radio();

	int counter = 0;
	for (std::list<std::pair<std::string, std::string> >::iterator i = radio_stations.begin(),
	       end = radio_stations.end(); i != end; ++i) {
	  i->first = itostr(++counter, 3) + ". " + i->first;
	  print_files.push_back(i->first);
	  cur_files.push_back(*i);
	}

	radio_update_needed = true;
	in_genre = false;
	old_radio_pos = radio_pos;
	radio_pos = 0;
      }
    else if (in_genre)
      {
	std::list<std::pair<std::string, std::string> > shoutcast_toplist = 
	  shoutcast->get_toplist(vector_lookup(cur_files, radio_pos).second);

	print_files.clear();
	cur_files.clear();

	for (std::list<std::pair<std::string, std::string> >::iterator i = shoutcast_toplist.begin(),
	       end = shoutcast_toplist.end(); i != end; ++i) {
	  print_files.push_back(i->first);
	  cur_files.push_back(*i);
	}

	radio_update_needed = true;
	in_genre = false;
	old_radio_pos = radio_pos;
	radio_pos = 0;
      }
    else
      {
	T s;
	std::pair<std::string, std::string> station;

	if(old_radio_pos == 0)
	  station = vector_lookup(cur_files, radio_pos);
	else
	  station = shoutcast->get_playlist(vector_lookup(cur_files, radio_pos).second);

	s.id = ++playlist_id;
	s.name = station.first;
	s.lowercase_name = string_format::lowercase(station.first);
	s.path = station.second;
	s.type = "";

	if (s.path.find("rtsp://") == 0 || s.path.find("rtp://") == 0) {
	  if (!audio_state->p->supports_rtp())
	    DialogWaitPrint pdialog(gettext("Alsaplayer does not support rtsp"), 2000);
	  else
	    s.type = "web";
	} else if (s.path.find("http://") == 0)
	  s.type = "web";
	else if (s.path.find("bttvradio://") == 0)
	  DialogWaitPrint pdialog(gettext("Please recompile mms with bttvradio enabled"), 2000);

	if (!s.type.empty()) {
	  DialogWaitPrint pdialog(gettext("Added track to playlist"), 1000);

	  playlist.push_back(s);
	  shuffle_list.add_track(s);
	}

	if (playlist.size() == 1)
	  // first element added so we need to set the pos
	  playlist_pos_int = 0;

	if (!audio_state->p_playing()) {
	  // setup
	  AudioMode old_mode = mode;
	  mode = PLAY;
	  files = &playlist;
	  int old_playlist_pos = playlist_pos_int;
	  playlist_pos_int = playlist.size() - 1;

	  play_track();

	  // reset
	  playlist_pos_int = old_playlist_pos;
	  files = &audio_list;
	  mode = old_mode;
	}
      }
  }

  // audio add specific
  // helper function
  void add_track_to_playlist(T p)
  {
    p.id = ++playlist_id;
    playlist.push_back(p);
    shuffle_list.add_track(p);

    if (playlist.size() == 1)
      // first element added so we need to set the pos
      playlist_pos_int = 0;
  }

  virtual std::vector<T> add_dir(const std::string& dir, bool from_media = false) = 0;

  void add()
  {
    std::string message;

    // add file or dir to playlist add the buttom of the list
    if (vector_lookup(*files, position_int()).type == "dir") {
	      
      std::vector<T> addfiles = add_dir(vector_lookup(*files, position_int()).path);

      for (typename std::vector<T>::const_iterator iter = addfiles.begin(), 
	     iend = addfiles.end(); iter != iend; ++iter)
	add_track_to_playlist(*iter);

      if (display_message)
	message = gettext("Added directory to playlist");
	      
    } else {
      add_track_to_playlist(vector_lookup(*files, position_int()));
      if (display_message)
	message = gettext("Added track to playlist");
    }
    if (display_message)
      DialogWaitPrint pdialog(message, 1000);
  }

  void play_track()
  {
    // stop movie playback if needed
    if (global->is_playback("Movie")) {
      Input stop;
      stop.command = "stop";
      stop.mode = "playback";
      
      global->check_commands(stop);

      global->set_playback("Audio");
    }

    T cur_track = vector_lookup(*files, playlist_pos_int);

#ifdef use_media
    check_mount_before(cur_track);
#endif
  
    audio_state->p->set_cur_nr(cur_track);

    audio_state->p->addfile(cur_track);

    if (cur_track.type == "web")
      audio_state->p->set_streaming(true);
    else
      audio_state->p->set_streaming(false);

#ifdef use_media
    check_mount_after(cur_track);
#endif

    if (audio_state->played_tracks.size() == 0) {
      // notify the random track generation that we want to start with this track
      shuffle_list.track_played(cur_track);
    }

    audio_state->add_track_to_played(cur_track);

    if (opts.shuffle() == gettext("real random"))
      audio_state->direction = Audio_s::ADDED;
  }

  void prev()
  {
    if (position_int() != 0)
      set_position_int(position_int()-1);
    else
      set_position_int(files->size()-1);
  }

  void next()
  {
    set_position_int((position_int()+1)%files->size());
  }

  virtual void page_up() = 0;
  virtual void page_down() = 0;

  // playlist specific
  void move_up()
  {
    if (playlist.size() != 1) {
      if (playlist_pos_int != 0) {
	T r;
	typename std::vector<T>::iterator j = playlist.begin() + playlist_pos_int;

	r = (*j);
	playlist.erase(j); --j;
	playlist.insert(j++, r);
	--j;

	// move the view
	playlist_pos_int = playlist_pos_int - 1;
      } else {
	typename std::vector<T>::iterator j = playlist.end(); --j;
	T r_end = playlist.back();
	T r_start = playlist.front();

	playlist.erase(j);
	playlist.erase(playlist.begin());
	playlist.push_back(r_start);
	playlist.insert(playlist.begin(), r_end);
	j = playlist.end(); --j;

	// move the view
	playlist_pos_int = playlist.size()-1;
      }
    }
  }

  void move_down()
  {
    if (playlist.size() != 1) {
      if ((playlist_pos_int + 1) != playlist.size()) {
	T r;
	typename std::vector<T>::iterator j = playlist.begin() + playlist_pos_int + 1;

	r = (*j);
	playlist.erase(j);
	playlist.insert(--j, r);
		
	// move the view
	playlist_pos_int = playlist_pos_int + 1;
      } else {
	typename std::vector<T>::iterator j = playlist.end(); --j;
	T r_end = playlist.back();
	T r_start = playlist.front();

	playlist.erase(j);
	playlist.erase(playlist.begin());
	playlist.push_back(r_start);
	playlist.insert((playlist.begin()), r_end);
	j = playlist.begin();

	// move the view
	playlist_pos_int = 0;
      }
    }
  }

  void delete_track()
  {
    assert(playlist_pos_int < playlist.size());
  
    typename std::vector<T>::iterator i = playlist.begin() + playlist_pos_int;

    if (files->size() == 1) {
	    
      if (audio_conf->p_audio_stop_on_clear()) {
	audio_state->p->stop();
	Simplefile empty;
	audio_state->p->set_cur_nr(empty);
      }

      // remove track if queued
      audio_state->remove_track_from_queue(*i);

      audio_state->remove_track_from_played(*i);
    
      playlist.clear();
      shuffle_list.clear();

      change_mode = true;

    } else {

      if (audio_state->p->p_cur_nr() == *i)
	audio_state->p->next();

      // remove track if queued
      audio_state->remove_track_from_queue(*i);

      audio_state->remove_track_from_played(*i);

      shuffle_list.remove_track(*i);
      playlist.erase(i);

      // this is playlist.size(), not -1, because we just deleted a track
      if (playlist_pos_int == playlist.size())
	// the last element of the list
	playlist_pos_int = playlist_pos_int - 1;
    }
  }

  void queue_track()
  {
    Simplefile cur_track = vector_lookup(*files, position_int());
    if (audio_state->queue_pos(cur_track) == 0)
      audio_state->add_track_to_queue(cur_track);
    else
      audio_state->remove_track_from_queue(cur_track);
  }

  void clear_playlist()
  {
    if (audio_conf->p_audio_stop_on_clear()) {
      audio_state->p->stop();
      Simplefile empty;
      audio_state->p->set_cur_nr(empty);
    }

    audio_state->remove_queued_tracks();

    audio_state->empty_played_tracks();

    playlist.clear();
    shuffle_list.clear();

    change_mode = true;
  }

  void play()
  {
    if (playlist.size() > 0) {
      if (opts.shuffle() == gettext("off")) {
	if ((audio_state->p->p_cur_nr()).path.empty()) {
	  audio_state->p->set_cur_nr(vector_lookup(playlist, playlist_pos_int));
	  audio_state->add_track_to_played(audio_state->p->p_cur_nr());
	}
      } else
	next_helper();

      if (audio_state->p->p_cur_nr().type == "web")
	audio_state->p->set_streaming(true);
      else
	audio_state->p->set_streaming(false);

      audio_state->p->play();

      // update the screen with the playing track
      if (in_playlist && !audio_state->fullscreen_info)
	print(*files);
    }
  }

  int get_real_playlist_pos()
  {
    Simplefile empty;
    Simplefile cur_track = audio_state->p->p_cur_nr();
    
    if (cur_track != empty) {
      typename std::vector<T>::iterator j = playlist.begin();

      int p = 0;
      while (j != playlist.end()) {
	if (j->id == cur_track.id)
	  return p;
	else {
	  ++j;
	  ++p;
	}
      }
    
      return playlist_pos_int;
      
    } else {
      return playlist_pos_int;
    }
  }

  void toggle_playlist()
  {
    if (input_master->current_map() == "playlist") {

      //!!!!! restore map?

      in_playlist = false;

      exit_loop = true;

    } else if (playlist.size() != 0) {

      input_master->save_map();

      // FIXME: this could be done a little nicer

      bool pictures_fullscreen_was_running = false;
      bool audio_fullscreen_was_running = false;

      // disable pictures slideshow when entering audio playlist
      ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();
      if (screen_updater->timer.status("pictures")) {
	pictures_fullscreen_was_running = true;
	screen_updater->timer.deactivate("pictures");
	if (!screen_updater->timer.status("audio_fullscreen"))
	  screen_updater->timer.activate("audio_fullscreen");
	else
	  audio_fullscreen_was_running = true;
      }

      bool mode_was_add = false;
      if (mode == ADD)
	mode_was_add = true;

      in_playlist = true;

      mainloop(PLAY);

      if (mode_was_add) {
	files = &audio_list;
	mode = ADD;
      }

      input_master->restore_map();
	
      if (pictures_fullscreen_was_running) {
	if (!audio_fullscreen_was_running)
	  screen_updater->timer.deactivate("audio_fullscreen");
	screen_updater->timer.activate("pictures");
      }

    } else if (playlist.size() == 0)
      DialogWaitPrint pdialog(gettext("No tracks in playlist"), 1000);
  }

  void toggle_load_playlist()
  {
    if (list_playlists().size() >= 1) {

      if (mode != PLAY)
	input_master->save_map();

      if (choose_playlist()) {
	if (mode == PLAY)
	  files = &playlist;

	playlist_pos_int = 0;
      }

      if (mode == PLAY)
	input_master->set_map("playlist");
      else
	input_master->set_map("audio");
	      
    } else
      Print pdialog(gettext("No saved playlists found"), Print::SCREEN);
  }

  virtual void intelligent_random_next() = 0;

  void next_helper()
  {
    if (playlist_size() == 0) {
      Simplefile s;
      audio_state->p->set_cur_nr(s);
      return;
    }

    Rand *rand = S_Rand::get_instance();

    typename std::vector<T>::iterator last_element = playlist.end();
    typename std::vector<T>::iterator j = playlist.begin();

    --last_element;

    Simplefile cur_nr = audio_state->p->p_cur_nr();

    if (audio_state->queue_size() != 0) {
    
      audio_state->p->set_cur_nr(audio_state->next_in_queue());

    } else if (opts.shuffle() == gettext("real random")) {

      if (playlist.size() > 1) {
	int r = rand->number(playlist.size());

	while (cur_nr == vector_lookup(playlist, r))
	  r = rand->number(playlist.size());

	audio_state->p->set_cur_nr(vector_lookup(playlist, r));
	audio_state->direction = Audio_s::ADDED;
      }

    } else if (opts.shuffle() == gettext("winamp random")) {

      Simplefile new_nr;
      while ((new_nr = shuffle_list.next_track()) == cur_nr && playlist.size() != 1);
      audio_state->p->set_cur_nr(new_nr);

    } else if (opts.shuffle() == gettext("intelligent")) {

      intelligent_random_next();

    } else {
      
      if (cur_nr != (*last_element) || conv::stob(opts.repeat()) && !conv::stob(opts.shutdown())) {

	// the currently selected track was deleted
	if (cur_nr.id == 0) {
	  int p = playlist_pos_int;
	  if (playlist_pos_int >= playlist_size()-1)
	    p = 0;
	  audio_state->p->set_cur_nr(vector_lookup(playlist, p));
	  return;
	}

	Simplefile c = cur_nr;

	if (!(c == (*last_element) && conv::stob(opts.repeat())) && !conv::stob(opts.shutdown())) {
      
	  while (j != playlist.end()) {

	    if (j->id == c.id)
	      break;
	    else
	      j++;
	  }
      
	  j++;
	}
      }
      audio_state->p->set_cur_nr(*j);
    }

    if (j->id != 0) // deleted track
      audio_state->add_track_to_played(*j);
  }

  virtual void intelligent_random_prev() = 0;

  void prev_helper()
  {
    if (playlist_size() == 0) {
      Simplefile s;
      audio_state->p->set_cur_nr(s);
      return;
    }

    Rand *rand = S_Rand::get_instance();

    typename std::vector<T>::iterator j = playlist.begin();

    Simplefile cur_nr = audio_state->p->p_cur_nr();

    if (opts.shuffle() == gettext("real random")) {
      
      Simplefile prev_track_played = audio_state->prev_track_played();

      if (audio_state->direction == Audio_s::ADDED && prev_track_played.id != 0) {
	prev_track_played = audio_state->prev_track_played();
	audio_state->direction = Audio_s::TAKEN;
      }

      if (prev_track_played.id != 0) {
	
	audio_state->p->set_cur_nr(prev_track_played);

      } else {

	if (playlist.size() > 1) {
	  // empty
	  int r = rand->number(playlist.size());

	  while (cur_nr == vector_lookup(playlist, r))
	    r = rand->number(playlist.size());

	  audio_state->p->set_cur_nr(vector_lookup(playlist, r));
	  audio_state->direction = Audio_s::ADDED;
	}
      }
      
    } else if (opts.shuffle() == gettext("winamp random")) {

      Simplefile prev_nr;
      while ((prev_nr = shuffle_list.prev_track()) == cur_nr && playlist.size() != 1);
      audio_state->p->set_cur_nr(prev_nr);

    } else if (opts.shuffle() == gettext("intelligent")) {

      Simplefile prev_track_played = audio_state->prev_track_played();

      if (audio_state->direction == Audio_s::ADDED && prev_track_played.id != 0) {
	prev_track_played = audio_state->prev_track_played();
	audio_state->direction = Audio_s::TAKEN;
      }

      if (prev_track_played.id != 0) {
	
	audio_state->p->set_cur_nr(prev_track_played);

      } else {

	intelligent_random_prev();

      }
      
    } else {
      
      Simplefile c = cur_nr;
    
      while (j != playlist.end()) {

	if (j->id == c.id)
	  break;
	else
	  ++j;
      }
    
      if (cur_nr == (*playlist.begin()))
	j = playlist.end();

      --j;

      audio_state->p->set_cur_nr(*j);
    }

    audio_state->add_track_to_played(*j);
  }

  // The following function for parsing audio dirs.
  std::vector<T> parse_dir(const std::list<std::string>& dirs)
  {
    std::vector<T> cur_files;
    
    for (std::list<std::string>::const_iterator iter = dirs.begin(); 
	 iter != dirs.end(); ++iter)
      {
	// Add files
	assert(iter->size() > 0);
	std::vector<T> tempfiles = rdir(*iter);
	cur_files.insert(cur_files.end(), tempfiles.begin(), tempfiles.end());
      }
    
    return cur_files;
  }

  void load_current_dirs()
  {
    audio_list = parse_dir(folders.top().first);

    if (audio_conf->p_audio_dirs().size() > 1 && folders.size() == 1)
      std::sort(audio_list.begin(), audio_list.end(), file_sort());
  }

  void cd_menu()
  {
    input_master->set_map("audio");

    int tracksize = cd->get_nr_tracks_cd();

    if (tracksize == -1) {
      Print pdialog(gettext("The disc does not contain any recognizable files"), 
		    Print::SCREEN);
      return;
    }

    ExtraMenu em("Audio CD playback");

    em.add_item(ExtraMenuItem(gettext("Add CD to playlist and start playback"), "", 
			      new TimeFunctorv<Audio>(this, &Audio::add_audio_cd)));

    em.add_item(ExtraMenuItem(gettext("Examine CD"), "", 
			      new TimeFunctorv<AudioTemplate<T> >(this, &AudioTemplate<T>::examine_audio_cd)));

    em.mainloop();
  }

  void examine_audio_cd()
  {
    std::vector<T> old_audio_list = audio_list;
    std::stack<std::pair<std::list<std::string>, int > > old_folders = folders;

    std::vector<T> audiolist_new = get_audio_cd_list();

    if (audiolist_new.size() == 0) {
      Print pdialogs(gettext("The disc does not contain any recognizable files"), 
		     Print::SCREEN);
    } else {

      audio_list.clear();
      while (folders.size() > 0)
	folders.pop();

      for (typename std::vector<T>::const_iterator iter = audiolist_new.begin(); 
	   iter != audiolist_new.end(); ++iter)
	audio_list.push_back(*iter);
    
      std::list<std::string> dirs;
      dirs.push_back(cd->get_mount_point());
      folders.push(std::make_pair(dirs, 0)); 
     
      mainloop(ADD);
 
      // restore old setting
      audio_list = old_audio_list;
      folders = old_folders;
    }
  }

  void add_audio_cd()
  {
    std::vector<T> audiolist_new = get_audio_cd_list();

    if (audiolist_new.size() == 0) {
      Print pdialogs(gettext("The disc does not contain any recognizable files"), 
		     Print::SCREEN);
    } else {

      int playlist_pos = playlist.size();

      for (typename std::vector<T>::const_iterator iter = audiolist_new.begin(); 
	   iter != audiolist_new.end(); ++iter) {
	T p = *iter;
	p.id = ++playlist_id;
	playlist.push_back(p);
	shuffle_list.add_track(p);
      }

      playlist_pos_int = playlist_pos;

      // play the first track
      mode = PLAY;
      files = &playlist;
      play_track();

      mainloop(PLAY);
    }
  }

  void add_data_cd()
  {
    std::vector<T> old_audio_list = audio_list;
    std::stack<std::pair<std::list<std::string>, int > > old_folders = folders;

    //mount the disc
    run::external_program("mount " + cd->get_mount_point());

    std::vector<T> audiolist_new = add_dir(cd->get_mount_point(), true);

    if (audiolist_new.size() == 0) {
      Print pdialog(gettext("The disc does not contain any recognizable files"), 
		    Print::SCREEN);

      //umount the disc
      run::external_program("umount " + cd->get_mount_point());

    } else {

      audio_list.clear();
      while (folders.size() > 0)
	folders.pop();

      audio_list = audiolist_new;

      std::list<std::string> dirs;
      dirs.push_back(cd->get_mount_point());
      folders.push(std::make_pair(dirs, 0)); 
 
      //umount the disc
      run::external_program("umount " + cd->get_mount_point());

      mainloop(ADD);

      // restore old setting
      audio_list = old_audio_list;
      folders = old_folders;
    }
  }

  void save_playlist(const std::string& playlistfile, bool overwrite = false)
  {
    std::vector<T> cur_files = playlist;

    if (playlistfile == "") {
      DialogWaitPrint pdialog(3000);
      pdialog.add_line(gettext("Can not save playlist with an empty name"));
      pdialog.add_line("");
      pdialog.add_line(gettext("Please use another name"));
      pdialog.print();
      return;
    }
  
    // doesn't make any sense to save nothing
    if (cur_files.size() != 0 || playlistfile == "last") {

      if (!overwrite && playlistfile != "last" && file_exists(conf->p_var_data_dir() + "playlists/" + playlistfile)) {
	DialogWaitPrint pdialog(3000);
	pdialog.add_line(gettext("Playlist with the chosen name already exists"));
	pdialog.add_line("");
	pdialog.add_line(gettext("Please use another name"));
	pdialog.print();
	return;
      }
    
      std::ofstream file;

      std::string plist = conf->p_var_data_dir() + "playlists/" + playlistfile; 
    
      file.open(plist.c_str());
    
      if (!file) {
	DebugPrint perror(gettext("Could not write playlist to file: ") + plist, 
			  Print::DEBUGGING, DebugPrint::CRITICAL, "Audio");
      } else {
      
	for (typename std::vector<T>::const_iterator iter = cur_files.begin(); 
	     iter != cur_files.end(); ++iter) {

	  if (iter->type == "web")
	    file << iter->type << "," << iter->path << ";" << iter->name << std::endl;
	  else
	    file << iter->type << "," << iter->path << std::endl;
	
	}
      }
    
      file.close();
    }
  }

  void load_media_track(const std::string& filename, std::vector<T>& cur_files, 
			bool& cdrom_set, bool& media_checked, bool& media_ok)
  {
    T file;
    file.id = ++playlist_id;

    // init it if it's not already done
    if (!audio_conf->p_audio_started())
      audio_state = S_Audio_s::get_instance();

#if 0  // FIXME
    if (!cdrom_set) {
      audio_state->p->update_cdrom(cd->get_device());
      cdrom_set = true;
    }
#endif

    if (!media_checked) {
      media_checked = true;
      bool opened = false;

      if (cd->cdrom == -1) {
	if (!cd->open()) {
	  media_ok = false;
	  return;
	}
	opened = true;
      }

      Cd::cdstatus cdstatus = cd->check_cddrive();
      if (cdstatus == Cd::OK)
	media_ok = true;
      else
	media_ok = false;

      if (opened)
	cd->close();
    }

    if (media_ok) {
      audio_state->p->collect_info(filename);

      if (audio_state->p->p_artist().empty() && audio_state->p->p_album().empty() && audio_state->p->p_title().empty()) {
	// FIXME gstreamer?
	// determine if alsaplayer or xine track
	if (filename.find("Track") != std::string::npos)
	  file.name = filename.substr(0, 7);
	else {
	  file.name = "Track " + filename.substr(6);
	}
      } else
	file.name = audio_state->p->p_artist() + " - " + audio_state->p->p_album() + " - " + audio_state->p->p_title();

      file.lowercase_name = string_format::lowercase(file.name);

      file.path = filename;
      file.type="media-track"; 

      cur_files.push_back(file);
    }
  }

  void load_media_file(const std::string& filename, std::vector<T>& cur_files, 
		       bool& media_mounted, bool& media_checked, bool& media_ok)
  {
    if (!media_checked) {
      media_checked = true;
      bool opened = false;

      if (cd->cdrom == -1) {
	if (!cd->open()) {
	  media_ok = false;
	  return;
	}
	opened = true;
      }

      Cd::cdstatus cdstatus = cd->check_cddrive();
      if (cdstatus == Cd::OK)
	media_ok = true;
      else
	media_ok = false;

      if (opened)
	cd->close();
    }

    if (media_ok) {
      // mount the drive if isn't mounted yet
      if (!media_mounted) {
	run::external_program("mount " + cd->get_mount_point());
	media_mounted = true;
      }

      if (file_exists(filename)) {
	T file(addsimplefile(filename, check_type(filename, audio_conf->p_filetypes_a()), true));
	file.id = ++playlist_id;

	cur_files.push_back(file);
      }
    }
  }

  void load_radio_file(const std::string& filename, const std::string& name, 
		       const std::string& type, std::vector<T>& cur_files)
  {
    if ((filename.find("rtsp://") == 0 || filename.find("rtp://") == 0) && !audio_state->p->supports_rtp())
      return;

    T file;
    file.id = ++playlist_id;
    file.name = name;
    file.lowercase_name = string_format::lowercase(name);
    file.path = filename;
    file.type = type;
    
    cur_files.push_back(file);
  } 

  bool load_last_playlist()
  {
    std::cout << "loading last saved playlist" << std::endl;
    bool status = load_playlist("last");
    
    std::pair<int, bool> loaded_values = audio_state->p->load_runtime_settings();

    playlist_pos_int = loaded_values.first;
    play_now_warning = loaded_values.second;

    // sanity check
    if (playlist_pos_int >= playlist.size())
      playlist_pos_int = 0;
    
    return status;
  }

  bool load_playlist(const std::string& playlistfile)
  {
    std::vector<T> cur_files;

    std::ostringstream playlist_path; 
    playlist_path << conf->p_var_data_dir() + "playlists/" << playlistfile;

    std::ifstream In(playlist_path.str().c_str());

    std::string str, type = "", filename, name;

    bool media_mounted = false;
    bool cdrom_set = false;

    bool media_checked = false;
    bool media_ok = false;

    if(!In){
      DebugPrint perror(gettext("Could not open playlist: ") + 
			playlist_path.str(), 
			Print::DEBUGGING, DebugPrint::CRITICAL, "Audio");
    } else {

      while (getline(In, str)) {

	std::string::size_type pos_comma = str.find(",");

	if (pos_comma != std::string::npos) {
	  type = str.substr(0, pos_comma);

	  if (type == "web") {
	    int pos_semi = str.rfind(";");
	    if (pos_semi == std::string::npos) {
	      DebugPrint perror(gettext("Playlist malformed, skipping line"), 
				Print::DEBUGGING, DebugPrint::CRITICAL, "Audio");
	      continue;
	    } else {
	      filename = str.substr(pos+1, pos_semi-(pos+1));
	      name = str.substr(pos_semi+1);
	    }
	  } else
	    filename = str.substr(pos_comma+1);
	} else
	  continue;
      
	if (type == "media-track" && conf->p_media()) {

	  load_media_track(filename, cur_files, cdrom_set, media_checked, media_ok);

	} else if (type == "media-file" && conf->p_media()) {

	  load_media_file(filename, cur_files, media_mounted, media_checked, media_ok);

	} else if (type == "web") {

	  load_radio_file(filename, name, type, cur_files);

	} else {
	  if (file_exists(filename)) {
	    T file(addsimplefile(filename, check_type(filename, audio_conf->p_filetypes_a())));
	    file.id = ++playlist_id;
      
	    cur_files.push_back(file);
	  }
	}
      }
    
      if (media_mounted)
	run::external_program("umount " + cd->get_mount_point());

      if (cur_files.size() != 0) {

	shuffle_list.clear();
	for (typename std::vector<T>::const_iterator i = cur_files.begin(), 
	       end = cur_files.end(); i != end; ++i)
	  shuffle_list.add_track(*i);
	audio_state->remove_queued_tracks();
	playlist.clear();
	playlist = cur_files;
	playlist_pos_int = 0;

	return true;

      } else {

	return false;

      }

    }
    return false;
  }

  void check_audio_state()
  {
    if (audio_conf->p_audio_started()) {
      
      audio_state->p->gather_info();
      
      // should we play the next track?
      if (!audio_state->p->streaming() && 
	  (audio_state->p->play_next() ||
	   !audio_state->p->is_playing() && audio_state->p_playing() && 
	   !audio_state->p_pause() && !audio_state->p->is_buffering())) {

	// random modes takes care of stopping when the last track has been played
	if ((opts.shuffle() == gettext("off") && audio_state->p->p_cur_nr() != playlist.back()) || 
	    opts.shuffle() != gettext("off") || conv::stob(opts.repeat()) || 
	    audio_state->queue_size() > 0) {
	  // next takes care of checking random modes
	  audio_state->p->next();
	  if (in_playlist && !audio_state->fullscreen_info && !printing_information)
	    print(*files);
	} else {
	  // we reached the end stop playing 
	  audio_state->set_playing(false);

	  if (playlist_size() == 0) {
	    // make sure we can't press play after the playback has ended
	    Simplefile s;
	    audio_state->p->set_cur_nr(s);
	  } else {
	    if (in_playlist && !audio_state->fullscreen_info)
	      print(*files);
	  }
	
	  if (conv::stob(opts.shutdown())) {
	    Shutdown *sd = S_Shutdown::get_instance();
	    sd->enable();
	  }
	}
      }
    }
  }

  void print_audio_fullscreen()
  {
    if (audio_conf->p_audio_started()) {

      if ((time(0)-conf->p_last_key()) > 60*conf->p_idle_time() && audio_state->p_playing()) {
	audio_state->fullscreen_info = true;
	audio_state->playback_fullscreen_shown = true;
	print_audio_screensaver();
      } else {
	if (audio_state->playback_fullscreen_shown) {

	  render->wait_and_aquire();

	  audio_state->playback_fullscreen_shown = false;
	  audio_state->fullscreen_info = false;
	  audio_state->playback_fullscreen.cleanup();

	  render->complete_redraw();
	  render->image_mut.leaveMutex();

	  if (in_playlist)
	    print(*files);
	}
      }
    }
  }

  virtual std::vector<T> rdir(const std::string& argv) = 0;

  std::vector<T> *files;

  // for changing mode
  std::vector<T> audio_list;

  std::vector<T> playlist;

public:

  void reset()
  {
    for (int i = 0, size = folders.size(); i < size; ++i)
      folders.pop();

    std::list<std::string> dirs = audio_conf->p_audio_dirs();

    folders.push(std::make_pair(dirs, 0)); 

    files = &audio_list;
    mode = ADD;
  }

  std::string get_playlist_element(int index)
  {
    return vector_lookup(playlist, index).path;
  }

  int audiolist_size() { return audio_list.size(); }
  int playlist_size() { return playlist.size(); }

  bool last_element_in_playlist()
  {
    return (audio_state->p->p_cur_nr() == playlist.back());
  }
};

#endif
