#include "plugin.hpp"
#include "audio.hpp"
#include "themes.hpp"
#include "config.hpp"
#include "print.hpp"
#include "plugins.hpp"
#include "cd.hpp"

#include "graphical_audio.hpp"
#include "simple_audio.hpp"

// FIXME: check that at least one audio player is available

void audio_hd()
{
  Audio *audio = get_class<Audio>("Audio");

  if (!audio->loaded_correctly) {
    audio->read_dirs();
    audio->loaded_correctly = true;
  } 

  if (audio->reload_needed) {
    DialogWaitPrint pdialog(gettext("Reloading directories"), 1000);

    audio->read_dirs();
  }

  if (audio->audiolist_size() == 0) {
	      
    Print pdialog(Print::SCREEN);
    pdialog.add_line(gettext("Could not find any music"));
    pdialog.add_line("");
    pdialog.add_line(gettext("Please specify a correct path "));
    pdialog.add_line(gettext("in the configuration file"));
    pdialog.print();

    audio->reload_needed = true;
	    
  } else {

    audio->reload_needed = false;
    audio->loaded_correctly = true;

    audio->mainloop(Audio::ADD);
  }
}

void audio_cd()
{
  Cd *cd = S_Cd::get_instance();

  if (cd->open()) {

    Audio *audio = get_class<Audio>("Audio");

    Cd::cdstatus cdstatus = cd->check_cddrive();

    if (cdstatus == Cd::OK)
      audio->detect_cdformat();

    cd->close();
  }
}

void audio_radio()
{
  AudioConfig *audio_conf = S_AudioConfig::get_instance();

  if (audio_conf->p_internet_radio().size() == 0) {

    // FIXME: error is not correct if we can just download some instead!
	      
    Print pdialog(Print::SCREEN);
    pdialog.add_line(gettext("Could not find any internet radio stations"));
    pdialog.add_line(gettext("Please define some in the configuration file"));
    pdialog.print();

  } else {
    get_class<Audio>("Audio")->choose_radio();
  }
}

class AudioPlugin : public FeaturePlugin
{
public:

  AudioPlugin() {
    Config *conf = S_Config::get_instance();
    AudioConfig *audio_conf = S_AudioConfig::get_instance();

    audio_conf->parse_configuration_file(conf->p_homedir());

    if (audio_conf->p_graphical_audio_mode())
      module = new GraphicalAudio();
    else
      module = new SimpleAudio();

    Themes *themes = S_Themes::get_instance();

    std::string text;

    if (audio_conf->p_audio_hd()) {
      // add audio_hd button
      text = gettext("Play audio from harddrive");
#if !defined(music_radio)
      if (!conf->p_media())
	text = gettext("Play audio");
#endif
      features.push_back(startmenu_item(text, "audio_hd", themes->startmenu_music_dir, 0, VoidFunctor(&audio_hd)));
    }

    if (conf->p_media()) {
      // add cd button
      text = gettext("Play an audio cd");
#if !defined(music_radio)
      if (!audio_conf->p_audio_hd())
	text = gettext("Play audio");
#endif
      features.push_back(startmenu_item(text, "audio_disc", themes->startmenu_music_cd, 1, VoidFunctor(&audio_cd)));
    }

#ifdef music_radio
    // add cd button
    features.push_back(startmenu_item(gettext("Play Internet Radio"), "audio_radio", themes->startmenu_radio, 2, VoidFunctor(&audio_radio)));
#endif
  }

  ~AudioPlugin()
  {
    (static_cast<Audio *>(module))->save_runtime_settings();
    delete module;
    module = 0;
  }

  std::string plugin_name() const { return gettext("Audio"); }

  int priority() const { return 1; }
  unsigned long capability() const { return 0; }
};

MMS_PLUGIN(AudioPlugin)
