#include "dbaudiofile.hpp"

#include "graphical_audio.hpp"

#include "common.hpp"

#include <fileref.h>
#include <tag.h>

using namespace TagLib;

using std::string;

string Dbaudiofile::to_string() const
{
  string result;

  if (artist.empty() || title.empty())
    result = name;
  else {

    string title_trimmed = string_format::convert(string_format::trim(title), true);
    string artist_trimmed = string_format::convert(string_format::trim(artist), true);
    string album_trimmed = string_format::convert(string_format::trim(album), true);

    if (!artist.empty())
      result += artist_trimmed;
    if (!album.empty())
      result += " - " + album_trimmed;
    if (!title.empty())
      result += " - " + title_trimmed;
  }

  return result;
}

string Dbaudiofile::short_to_string() const
{
  string result;

  if (artist.empty() || title.empty())
    result = name;
  else {

    string title_trimmed = string_format::convert(string_format::trim(title), true);
    string artist_trimmed = string_format::convert(string_format::trim(artist), true);
    string album_trimmed = string_format::convert(string_format::trim(album), true);

    // FIXME: lcd size

    if (title_trimmed.size() > 20)
      result = title_trimmed.substr(0, 20);
    else if (title_trimmed.size() + artist_trimmed.size() > 20) {
      result = (artist_trimmed + " - " + title_trimmed);
      result = result.substr(result.size()-20);
    } else {
      result = (artist_trimmed + " - " + album_trimmed + " - " + title_trimmed);
      result = result.substr(result.size()-20);
    }
  }

  return result;
}

string Dbaudiofile::get_name_from_file_in_picture_dir() const
{
  if (title.empty())
    return name;
  else
    return string_format::convert(string_format::trim(title), true);
}

void Dbaudiofile::set_values(const Dbaudiofile& s)
{
  id = s.id;
  db_id = s.db_id;
  name = s.name;
  lowercase_name = s.lowercase_name;
  path = s.path;
  type = s.type;
}

Dbaudiofile::Dbaudiofile() 
  : artist(""), album(""), title(""), year(0), bitrate(0), length(0), 
    rating(0), track(0), fetched(false)
{}

Dbaudiofile::Dbaudiofile(const GSimplefile& s) 
  : GSimplefile(s), artist(""), album(""), title(""), year(0), bitrate(0), 
    length(0), rating(0), track(0), fetched(false)
{}

Dbaudiofile::Dbaudiofile(const string& p)
  : artist(""), album(""), title(""),
    year(0), bitrate(0), length(0), rating(0), track(0)
{
  GraphicalAudio *ga = get_class<GraphicalAudio>("Audio");

  ga->imms_mut.enterMutex();

  SQLQuery *q = ga->db.query("Audio", ("SELECT * FROM %t WHERE filename='" + string_format::escape_string(p) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {

    SQLRow &row = (*q)[0];

    title = row["Title"];
    bitrate = conv::atoi(row["Bitrate"]);
    length = conv::atoi(row["Length"]);
    track= conv::atoi(row["Track"]);

    SQLQuery *q2 = ga->db.query("Artist", ("SELECT name FROM %t WHERE id='" + row["Artist"] + "'").c_str());
    if (q2 && q2->numberOfTuples() > 0) {
      SQLRow &c_row = (*q2)[0];
      artist = c_row["name"];
    }
    delete q2;
      
    q2 = ga->db.query("Album", ("SELECT name FROM %t WHERE id='" + row["Album"] + "'").c_str());
    if (q2 && q2->numberOfTuples() > 0) {
      SQLRow &c_row = (*q2)[0];
      album = c_row["name"];
    }
    delete q2;

  } else {
    
    FileRef fileref(p.c_str(), true, AudioProperties::Accurate);
    
    if (!fileref.isNull() && fileref.tag()) {
      artist = fileref.tag()->artist().toCString();
      title = fileref.tag()->title().toCString();
      album = fileref.tag()->album().toCString();
      year = fileref.tag()->year();
      track = fileref.tag()->track();
    }
    
    if (!fileref.isNull() && fileref.audioProperties()) {
      length = fileref.audioProperties()->length();
      bitrate = fileref.audioProperties()->bitrate();
    }

    int artist_id = 0;

    SQLQuery *q2 = ga->db.query("Artist", ("SELECT id FROM %t WHERE name='" + string_format::escape_string(artist) + "'").c_str());
    if (q2 && q2->numberOfTuples() > 0) {
      SQLRow &row = (*q2)[0];
      artist_id = conv::atoi(row["id"]);
    }
    delete q2;

    if (artist_id == 0) {
      char *tmp = sqlite3_mprintf("INSERT INTO Artist VALUES(NULL, '%q')", artist.c_str());
      ga->db.execute(tmp);
      sqlite3_free(tmp);

      q2 = ga->db.query("Artist", ("SELECT id FROM %t WHERE name='" + string_format::escape_string(artist) + "'").c_str());
      if (q2 && q2->numberOfTuples() > 0) {
	SQLRow &row = (*q2)[0];
	artist_id = conv::atoi(row["id"]);
      }
      delete q2;
    }

    int album_id = 0;

    q2 = ga->db.query("Album", ("SELECT id FROM %t WHERE name='" + string_format::escape_string(album) + "'").c_str());
    if (q2 && q2->numberOfTuples() > 0) {
      SQLRow &row = (*q2)[0];
      album_id = conv::atoi(row["id"]);
    }
    delete q2;

    if (album_id == 0) {
      char *tmp = sqlite3_mprintf("INSERT INTO Album VALUES(NULL, '%q')", album.c_str());
      ga->db.execute(tmp);
      sqlite3_free(tmp);

      q2 = ga->db.query("Album", ("SELECT id FROM %t WHERE name='" + string_format::escape_string(album) + "'").c_str());
      if (q2 && q2->numberOfTuples() > 0) {
	SQLRow &row = (*q2)[0];
	album_id = conv::atoi(row["id"]);
      }
      delete q2;
    }

    char *tmp = sqlite3_mprintf("INSERT INTO Audio VALUES(NULL, '%q', '%q', '%q', '%q', '%q', '%q', '%q', '%q')",
				conv::itos(artist_id).c_str(), conv::itos(album_id).c_str(), 
				title.c_str(), p.c_str(), 
				conv::ftos(bitrate).c_str(), conv::itos(length).c_str(), 
				conv::itos(track).c_str(), "");
    ga->db.execute(tmp);
    sqlite3_free(tmp);
  }
  
  delete q;

  rating = ga->imms->get_rating(p);

  ga->imms_mut.leaveMutex();

  path = p;
}

