#include "config.h"

#include "libfspp.hpp"

#include "graphical_audio.hpp"

#include "search.hpp"
#include "graphics.hpp"

#ifdef use_inotify
#include "notify.hpp"
#endif

using std::string;
using std::vector;
using std::list;

GraphicalAudio::GraphicalAudio()
  : db((conf->p_var_data_dir() + "audio.db").c_str()), imms_cur_path(""), extracted_all_files(false)
{
  imms = new IMMSC();
  //  imms->start();
  imms->check_connection();
  imms->setup(true);

  check_db();

  extracted_all_files = false;

  if (conf->p_v_res() == 720)
    y_start = 85;
  else if (conf->p_v_res() == 800)
    y_start = 90;
  else if (conf->p_v_res() == 1024)
    y_start = 110;
  else if (conf->p_v_res() == 1280)
    y_start = 110;
  else
    y_start = 70; // default value

  if (conf->p_h_res() == 405)
    y_start = 70;

  image_width = 115;
  image_height = 120;

  if (conf->p_h_res() == 405) {
    image_width = 65;
    image_height = 67;
  }

  image_width_all = image_width+25;
  image_height_all = image_height+55;
  image_height_all_eks_text = image_height + 6;

  images_per_row = (conf->p_v_res()-2*25)/image_width_all;

  // lets make some more room for text 
  image_width_all += ((conf->p_v_res()-2*25)%image_width_all)/images_per_row;

  // + 20 is for the last row, since we don't need to add extra space
  rows = (conf->p_h_res()-2*y_start+20)/image_height_all;
  rows_search = (conf->p_h_res()-2*y_start+20-40)/image_height_all;

  image_height_all = (conf->p_h_res()-2*y_start+20)/rows;
  image_height_all_search = (conf->p_h_res()-2*y_start+20-40)/rows_search;
}

GraphicalAudio::~GraphicalAudio()
{
  delete imms;
}

void GraphicalAudio::create_db()
{
  vector<string> fields;
  fields.push_back("version INTEGER NOT NULL");
  db.createTable("Schema", fields);

  db.execute("INSERT INTO Schema VALUES('1')");

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("parent INTEGER");
  fields.push_back("filename TEXT");
  fields.push_back("name TEXT");
  fields.push_back("is_folder INTEGER"); // 1 = true
  db.createTable("Folders", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("name TEXT");
  db.createTable("Artist", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("name TEXT");
  db.createTable("Album", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("name INTEGER");
  db.createTable("Year", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("name TEXT");
  db.createTable("Genre", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("aid INTEGER NOT NULL");
  fields.push_back("gid INTEGER NOT NULL");
  db.createTable("GAudio", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("Artist INTEGER");
  fields.push_back("Album INTEGER");
  fields.push_back("Title TEXT");
  fields.push_back("filename TEXT");
  fields.push_back("Bitrate FLOAT");
  fields.push_back("Length INTEGER");
  fields.push_back("Track INTEGER");
  fields.push_back("Cover TEXT");
  db.createTable("Audio", fields);
}

void GraphicalAudio::check_db()
{
  if (db.hasTable("Schema")) {
    // check for correct version
    SQLQuery *q = db.query("Schema", "SELECT version FROM %t");
    if (q && q->numberOfTuples() > 0) {
      SQLRow &row = (*q)[0];
      if (row["version"] != "1")
	// FIXME: upgrade db
	;
    }
    delete q;
  } else {
    // create the whole db
    create_db();
  }
}

void GraphicalAudio::reenter(int db_id)
{
  SQLQuery *q = db.query("Folders", ("SELECT filename FROM %t WHERE id='" + conv::itos(db_id) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {
    SQLRow &row = (*q)[0];
    string filename = row["filename"];
    string name_in_folder, path, current_path;

    // FIXME: restore?
    for (int i = 0, size = folders.size(); i < size; ++i)
      folders.pop();

    string::size_type pos_backslash = filename.rfind('/');
    if (pos_backslash != string::npos) {
      name_in_folder = filename.substr(pos_backslash+1);
      path = filename.substr(0, pos_backslash);
    }

    // FIXME: restore?
    audio_list = rdir(path);

    int pos_in_folder = 0;
    for (vector<Dbaudiofile>::iterator i = audio_list.begin(), iend = audio_list.end(); 
	 i != iend; ++i, ++pos_in_folder)
      if (i->path == filename)
	break;

    list<string> dirs = audio_conf->p_audio_dirs();

    for (list<string>::iterator i = dirs.begin(), iend = dirs.end(); i != iend; ++i)
      if (filename.find(*i) != string::npos) {
	current_path = path.substr(0, i->size());
	path = path.substr(i->size());
	//	folders.push(std::make_pair(dirs, 0));
	while ((pos = path.find('/')) != string::npos) {
	  string folder = path.substr(0, pos);
	  current_path += folder;
	  list<string> templs; templs.push_back(current_path);
	  folders.push(std::make_pair(templs, 0));
	  path = path.substr(pos+1);
	}
	if (current_path[current_path.size()-1] != '/')
	  current_path += '/';
	list<string> templs; templs.push_back(current_path + path);
	folders.push(std::make_pair(templs, pos_in_folder));
	break;
      }

#ifdef use_inotify
    S_Notify::get_instance()->register_plugin("audio", folders.top().first,
					      new TimeFunctorv<Audio>(this, &Audio::reparse_current_dir));
#endif

    loaded_correctly = true;

    mainloop(ADD);
  }
  delete q;
}

vector<std::pair<string, int> > GraphicalAudio::gen_search_list(const string& search_word)
{
  string local_dir = "";

  if (mode == ADD) {
    list<string> current_folders = folders.top().first;
    bool first = true;
    for (list<string>::iterator i = current_folders.begin(), iend = current_folders.end();
	 i != iend; ++i) {
      if (first) {
	local_dir += " AND ";
	first = false;
      } else
	local_dir += " OR ";

      local_dir += "filename LIKE '" + *i + "%%'";
    }
  }

  vector<std::pair<string, int> > result;

  vector<int> ids_already_found;

  // folders
  SQLQuery *q = db.query("Folders", ("SELECT DISTINCT id, filename, name, is_folder FROM %t WHERE fuzzycmp('" + search_word + "', name, 1)").c_str());

  if (q) {

    for (int i = 0; i < q->numberOfTuples(); ++i) {

      SQLRow &row = (*q)[i];

      string out;

      if (row["is_folder"] == "1") {

	bool result_ok = false;
	// FIXME: search all top folders
	list<string> current_folders = folders.top().first;
	for (list<string>::iterator i = current_folders.begin(), iend = current_folders.end();
	     i != iend; ++i) {
	  if (row["filename"].find(*i) != string::npos) {
	    result_ok = true;
	    break;
	  }
	}

	if (result_ok) {
	  out = row["name"];
	  result.push_back(std::make_pair(string_format::convert(out), conv::atoi(row["id"])));
	}
	
      } else {

	SQLQuery *q3 = db.query("Audio", ("SELECT * FROM %t WHERE filename='" + string_format::escape_string(row["filename"]) + "'").c_str());
	if (q3 && q3->numberOfTuples() > 0) {
	  assert(q3->numberOfTuples() == 1);
	  SQLRow &c_row = (*q3)[0];
	  SQLQuery *q2 = db.query("Artist", ("SELECT name FROM %t WHERE id='" + c_row["Artist"] + "'").c_str());
	  if (q2 && q2->numberOfTuples() > 0) {
	    SQLRow &c_row2 = (*q2)[0];
	    out += c_row2["name"];
	  }
	  delete q2;
	
	  q2 = db.query("Album", ("SELECT name FROM %t WHERE id='" + row["Album"] + "'").c_str());
	  if (q2 && q2->numberOfTuples() > 0) {
	    SQLRow &c_row2 = (*q2)[0];
	    out += " - " + c_row2["name"];
	  }
	  delete q2;
	
	  out += " - " + c_row["Title"];
	  result.push_back(std::make_pair(string_format::convert(out), conv::atoi(row["id"])));
	  ids_already_found.push_back(conv::atoi(row["id"]));
	}
	delete q3;
      }
    }
  }
  delete q;

  // title
  q = db.query("Audio", ("SELECT DISTINCT * FROM %t WHERE fuzzycmp('" + search_word + "', title, 1)").c_str());

  if (q) {
    for (int i = 0; i < q->numberOfTuples(); ++i) {

      SQLRow &row = (*q)[i];

      string out;

      SQLQuery *q2 = db.query("Artist", ("SELECT name FROM %t WHERE id='" + row["Artist"] + "'").c_str());
      if (q2 && q2->numberOfTuples() > 0) {
	SQLRow &c_row = (*q2)[0];
	out += c_row["name"];
      }
      delete q2;

      q2 = db.query("Album", ("SELECT name FROM %t WHERE id='" + row["Album"] + "'").c_str());
      if (q2 && q2->numberOfTuples() > 0) {
	SQLRow &c_row = (*q2)[0];
	out += " - " + c_row["name"];
      }
      delete q2;

      out += " - " + row["Title"];

      int id;

      SQLQuery *q3 = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_string(row["filename"]) + "'").c_str());
      if (q3 && q3->numberOfTuples() > 0) {
	assert(q3->numberOfTuples() == 1);
	SQLRow &c_row = (*q3)[0];
	id = conv::atoi(c_row["id"]);
      } else
	assert(false);

      delete q3;

      result.push_back(std::make_pair(string_format::convert(out), id));
      ids_already_found.push_back(id);
    }
  }
  delete q;

  // artist
  q = db.query("Artist", ("SELECT * FROM %t WHERE fuzzycmp('" + search_word + "', name, 1)").c_str());
  if (q) {
    for (int i = 0; i < q->numberOfTuples(); ++i) {
      SQLRow &row = (*q)[i];

      string artist_id = row["id"];
      string artist = row["name"];

      SQLQuery *q2 = db.query("Audio", ("SELECT * FROM %t WHERE Artist='" + artist_id + "'" + local_dir).c_str());
      if (q2) {
	for (int j = 0; j < q2->numberOfTuples(); ++j) {
	  SQLRow &row2 = (*q2)[j];

	  int id;
	  
	  SQLQuery *q4 = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_string(row2["filename"]) + "'").c_str());
	  if (q4 && q4->numberOfTuples() > 0) {
	    assert(q4->numberOfTuples() == 1);
	    SQLRow &c_row = (*q4)[0];
	    id = conv::atoi(c_row["id"]);
	  } else
	    assert(false);

	  delete q4;

	  // check if already found
	  bool found_match = false;
	  for (vector<int>::const_iterator iter = ids_already_found.begin(), iend = ids_already_found.end();
	       iter != iend; ++iter)
	    if (*iter == id) {
	      found_match = true;
	      break;
	    }

	  if (found_match)
	    continue;

	  string out = artist;

	  SQLQuery *q3 = db.query("Album", ("SELECT name FROM %t WHERE id='" + row2["Album"] + "'").c_str());
	  if (q3 && q3->numberOfTuples() > 0) {
	    SQLRow &c_row = (*q3)[0];
	    out += " - " + c_row["name"];
	  }
	  delete q3;

	  out += " - " + row2["Title"];

	  result.push_back(std::make_pair(string_format::convert(out), id));
	  ids_already_found.push_back(id);
	}
      }
      delete q2;
    }
  }
  delete q;

  // album
  q = db.query("Album", ("SELECT * FROM %t WHERE fuzzycmp('" + search_word + "', name, 1)").c_str());
  if (q) {
    for (int i = 0; i < q->numberOfTuples(); ++i) {
      SQLRow &row = (*q)[i];

      string album_id = row["id"];
      string album = row["name"];

      SQLQuery *q2 = db.query("Audio", ("SELECT * FROM %t WHERE Album='" + album_id + "'" + local_dir).c_str());
      if (q2) {
	for (int j = 0; j < q2->numberOfTuples(); ++j) {
	  SQLRow &row2 = (*q2)[j];

	  int id;
	  
	  SQLQuery *q4 = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_string(row2["filename"]) + "'").c_str());
	  if (q4 && q4->numberOfTuples() > 0) {
	    assert(q4->numberOfTuples() == 1);
	    SQLRow &c_row = (*q4)[0];
	    id = conv::atoi(c_row["id"]);
	  } else
	    assert(false);

	  delete q4;

	  // check if already found
	  bool found_match = false;
	  for (vector<int>::const_iterator iter = ids_already_found.begin(), iend = ids_already_found.end();
	       iter != iend; ++iter)
	    if (*iter == conv::atoi(row2["id"])) {
	      found_match = true;
	      break;
	    }

	  if (found_match)
	    continue;

	  string out;

	  SQLQuery *q3 = db.query("Artist", ("SELECT name FROM %t WHERE id='" + row2["Artist"] + "'").c_str());
	  if (q3 && q3->numberOfTuples() > 0) {
	    SQLRow &c_row = (*q3)[0];
	    out += c_row["name"];
	  }
	  delete q3;

	  out += " - " + album + " - " + row2["Title"];

	  result.push_back(std::make_pair(string_format::convert(out), id));
	  ids_already_found.push_back(id);
	}
      }
      delete q2;
    }
  }
  delete q;

  return result;
}

int GraphicalAudio::check_search()
{
  return 1000; // ms
}

void GraphicalAudio::search_func()
{
  BusyIndicator::get_instance().idle();

  ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();

  search_mode = true;
  search_help_offset = -1;

  print_marked_search_letter();

  TimeFunctori<GraphicalAudio> time_func(this, &GraphicalAudio::check_metadata);

  TimeFunctorv<GraphicalAudio> p(this, &GraphicalAudio::print_marked_search_letter);
  TimeFunctorv<GraphicalAudio> c(this, &GraphicalAudio::clean_up_search_print);

  list<string> empty;
  empty.push_back("");

  screen_updater->trigger.add(TriggerElement("search marker", empty,  &p, &c));
  screen_updater->timer.add(TimeElement("search marker", &time_func, &p));

  ConstFunctor<GraphicalAudio, string, vector<std::pair<string, int> > > gen_list_func(this, &GraphicalAudio::gen_search_list);
  NonConstFunctor<GraphicalAudio, vector<Dbaudiofile>, void> print_func(this, &GraphicalAudio::print);

  int new_pos = input_master->device->graphical_search(*files, &print_func, &gen_list_func,
						       search_mode, search_str, lowercase_search_str, 
						       offset, search_help_offset);

  screen_updater->trigger.del("search marker");
  screen_updater->timer.del("search marker");

  if (new_pos != -1) {
    if (mode == ADD)
      folders.top().second = new_pos;
    else
      playlist_pos_int = new_pos;
  }
}

void GraphicalAudio::page_up()
{
  int jump = conf->p_jump();
  
  if (mode == ADD) {
    if (search_mode)
      jump = images_per_row*rows_search;
    else
      jump = images_per_row*rows;
  }

  if (files->size() > jump) {
    int new_pos = position_int()-jump;
    if (position_int() == 0)
      set_position_int(files->size()-1+new_pos);
    else if (new_pos < 0)
      set_position_int(0);
    else
      set_position_int(new_pos);
  }
}

void GraphicalAudio::page_down()
{
  int jump = conf->p_jump();

  if (mode == ADD) {
    if (search_mode)
      jump = images_per_row*rows_search;
    else
      jump = images_per_row*rows;
  }

  if (files->size() > jump) {
    if (position_int() > (files->size() - jump) && position_int() != (files->size()-1))
      set_position_int(files->size()-1);
    else
      set_position_int((position_int()+jump)%files->size());
  }
}

string GraphicalAudio::mainloop(AudioMode new_mode)
{
  BusyIndicator &busy_indicator = BusyIndicator::get_instance();
  Shutdown *sd = S_Shutdown::get_instance();

  mainloop_top(new_mode);

  Input input;
  
  bool update_needed = true;

  while (!exit_loop)
    {
      if (update_needed) {
	print(*files);
	print_lcd_menu();
       }

      busy_indicator.idle();

      input = input_master->device->get_input();

      busy_indicator.busy();

      if (sd->is_enabled()) {
	sd->cancel();
	continue;
      }

      update_needed = true;

      // general
      if (input.command == "page_up")
 	{
	  page_up();
 	}
      else if (input.command == "page_down")
 	{
	  page_down();
 	}
      else if (input.command == "second_action")
	{
	  secondary_menu();
	}
      else if (input.command == "search")
	{
	  search_func();
	}
      else if (input.command == "options")
	{
	  options();
	}
      else if (input.command == "cover" && vector_lookup(*files, position_int()).type == "dir")
	{
	  choose_cover();
	}
      else if (input.command == "info" && input.mode == "graphical")
	{
	  print_information();
	}
      // playlist
      else if (mode == PLAY && input.command == "move_up")
	{
	  move_up();
	}
      else if (mode == PLAY && input.command == "move_down")
	{
	  move_down();
	}
      else if (mode == PLAY && input.command == "delete")
	{
	  delete_track();
	}
      else if (mode == PLAY && input.command == "queue")
	{
	  queue_track();
	}
      else if (mode == PLAY && input.command == "clear_list")
	{
	  clear_playlist();
	}
      else if (mode == PLAY && input.command == "save_playlist")
	{
	  save_playlist_func();
	}
      else if (mode == PLAY && input.command == "action")
	{
	  play_track();
	}
      // audio-add
      else if (mode == ADD and !no_folders_in_dir(*files)) {
	if (input.command == "prev")
	  {
	    if (files->size() > images_per_row) {
	      for (int i = 0; i < images_per_row; ++i) {
		if (folders.top().second == 0) {
		  folders.top().second = files->size()-1;
		  i += images_per_row-files->size()%images_per_row;
		}
		else 
		  folders.top().second -= 1;
	      }
	    }
	  }
	else if (input.command == "next")
	  {
	    if (files->size() > images_per_row) {
	      for (int i = 0; i < images_per_row; ++i) {
		if (folders.top().second == files->size()-1 && i == 0) {
		  folders.top().second = 0;
		  break;
		} else if (folders.top().second == files->size()-1) {
		  break;
		} else {
		  folders.top().second += 1;
		}
	      }
	    }
	  }
	else if (input.command == "left")
	  {
	    if (folders.top().second != 0)
	      folders.top().second -= 1;
	    else
	      folders.top().second = files->size() - 1;
	  }
	else if (input.command == "right")
	  {
	    folders.top().second = (folders.top().second+1)%files->size();
	  }
	else if (input.command == "back" && folders.size() > 1)
	  {
	    leave_dir();
	  }
	else if (input.command == "back" && folders.size() == 1)
	  {
	    exit();
	  }
	else if (input.command == "add_dir") 
	  {
	    if (vector_lookup(*files, position_int()).type == "dir")
	      add();
	  }
	else if (input.command == "action")
	  {
	    if (vector_lookup(*files, position_int()).type == "dir")
	      enter_dir();
	    else
	      add();
	  }
	else if (input.command == "add_all")
	  {
	    add_all();
	  }
  	else if (input.command == "play_now")
	  {
	    if (audio_conf->p_audio_mode_playnow() && 
		vector_lookup(*files, position_int()).type == "dir")
	      play_now();
	  }
      } else {
	// general and no folders
	if (input.command == "prev")
	  {
	    if (position_int() != 0)
	      set_position_int(position_int()-1);
	    else
	      set_position_int(files->size()-1);
	  }
	else if (input.command == "next")
	  {
	    set_position_int((position_int()+1)%files->size());
	  }
	else if (input.command == "back" && input.mode == "general")
	  {
	    if (mode == ADD && folders.size() > 1)
	      leave_dir();
	    else
	      exit();
	  }
	// audio-add and no folders
	else if (mode == ADD && input.command == "action")
	  {
	    add();
	  }
	else if (mode == ADD && input.command == "add_all")
	  {
	    add_all();
	  }
	else if (mode == ADD && input.command == "play_now")
	  {
	    if (vector_lookup(*files, position_int()).type != "dir")
		play_track_now();
	  }
      }
      update_needed = !global->check_commands(input);
    }

  exit_loop = false;

  if (in_playlist)
    in_playlist = false;

  return "";
}

void GraphicalAudio::secondary_menu_in_add(ExtraMenu &em)
{
  bool is_dir = (vector_lookup(*files, position_int()).type == "dir");

  if (is_dir) {
    em.add_item(ExtraMenuItem(gettext("Enter directory"), 
			      input_master->find_shortcut("action"), 
			      new TimeFunctorv<GraphicalAudio>(this, &GraphicalAudio::enter_dir)));

    em.add_item(ExtraMenuItem(gettext("Add directory to playlist"), 
			      input_master->find_shortcut("add_dir"), 
			      new TimeFunctorv<GraphicalAudio>(this, &GraphicalAudio::add)));

    em.add_item(ExtraMenuItem(gettext("Choose cover for folder"), 
			      input_master->find_shortcut("cover"), 
			      new TimeFunctorv<GraphicalAudio>(this, &GraphicalAudio::choose_cover)));

  } else {
    em.add_item(ExtraMenuItem(gettext("Add track to playlist"), 
			      input_master->find_shortcut("action"), 
			      new TimeFunctorv<GraphicalAudio>(this, &GraphicalAudio::add)));
  }

  if (folders.size() > 1)
    em.add_item(ExtraMenuItem(gettext("Go up one directory"),
			      input_master->find_shortcut("back"),
			      new TimeFunctorv<GraphicalAudio>(this, &GraphicalAudio::leave_dir)));
  else
    em.add_item(ExtraMenuItem(gettext("Return to startmenu"),
			      input_master->find_shortcut("back"),
			      new TimeFunctorv<GraphicalAudio>(this, &GraphicalAudio::exit)));
}

void GraphicalAudio::secondary_menu()
{
  ExtraMenu em;

  secondary_menu_helper(em);

  if (vector_lookup(*files, position_int()).type != "dir")
    em.add_item(ExtraMenuItem(gettext("print Information"), input_master->find_shortcut("info"), 
			      new TimeFunctorv<GraphicalAudio>(this, &GraphicalAudio::print_information)));

  em.mainloop();
}

void GraphicalAudio::enter_dir()
{
  if (vector_lookup(*files, position_int()).type == "dir") {
    string temp_cur_dir = vector_lookup(*files, position_int()).path;

    int nr_files = 0;
    for (file_iterator<file_t, default_order> i (temp_cur_dir); i != i.end (); i.advance(false))
      if (!isDirectory(i->getName()) && check_type(i->getName(), audio_conf->p_filetypes_a()) != emptyMyPair)
	++nr_files;

    ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();

    screen_updater->status_progressbar = 0;
    screen_updater->total_progressbar = nr_files;
    screen_updater->header = gettext("Extracting metadata");

    TimeFunctoriNC cp(&check_progressbar);
    TimeFunctorvNC pg(&print_progressbar);

    screen_updater->timer.add(TimeElement("progressbar", &cp, &pg));

    vector<Dbaudiofile> templist = rdir(temp_cur_dir);

    screen_updater->timer.del("progressbar");
    cleanup_progressbar();

    if (templist.size() != 0) {

      list<string> templs; 
      if (temp_cur_dir[temp_cur_dir.size()-1] != '/')
	templs.push_back(temp_cur_dir + '/');
      else
	templs.push_back(temp_cur_dir);
      folders.push(std::make_pair(templs, 0));
      audio_list = templist;
      files = &audio_list;

#ifdef use_inotify
      S_Notify::get_instance()->register_plugin("audio", folders.top().first,
						new TimeFunctorv<Audio>(this, &Audio::reparse_current_dir));
#endif

    } else
      DialogWaitPrint pdialog(gettext("Folder is empty"), 1000);
  }
}

void GraphicalAudio::print_audiotrack_helper(int &x_pos, string& buffer, 
					     string& album, string& artist, string& title)
{
  string pic = GSimplefile(audio_state->p->p_cur_nr()).find_cover_in_current_dir();

  if (!pic.empty()) {
    x_pos = 70;

    x_pos += global->get_playback_offset();

    render->create_scaled_image_wrapper(pic, 57, 63);
    int pic_y_pos = conf->p_h_res()-60;
    if (conf->p_v_res() == 720)
      pic_y_pos += 2;
    audio_state->playback.add(new PFObj(pic, 7 + global->get_playback_offset(), pic_y_pos, 57, 63, false, 0));
  }

  // the extracted information is more reliant for VBR mp3 than the audio
  // player, same for artist, album and title
  if (!audio_state->p->p_cur_nr().path.empty() && audio_state->p->p_cur_nr().type != "web") {

    Dbaudiofile dbaf(audio_state->p->p_cur_nr().path);
    buffer = format_time(audio_state->p->p_cur_time(), dbaf.length != 0 ? dbaf.length : audio_state->p->p_total_time());
    artist = string_format::convert(dbaf.artist, true);
    album = string_format::convert(dbaf.album, true);
    title = string_format::convert(dbaf.title, true);

  } else {
    if (audio_state->p->p_cur_nr().type == "web")
      buffer = format_time(audio_state->p->p_cur_time());
    else
      buffer = format_time(audio_state->p->p_cur_time(), audio_state->p->p_total_time());
    artist = audio_state->p->p_artist();
    album = audio_state->p->p_album();
    title = audio_state->p->p_title();
  }
}

void GraphicalAudio::print_audio_screensaver_helper(int &x_pos, int &y, int max_x_pic, int max_y_pic)
{
  string pic = GSimplefile(audio_state->p->p_cur_nr()).find_cover_in_current_dir();

  if (!pic.empty()) {
    render->create_scaled_image_wrapper(pic, max_x_pic, max_y_pic);
    PFObj *p = new PFObj(pic, 60, 60, max_x_pic, max_y_pic, true, 2);
    y = (conf->p_h_res()-p->real_h)/2;
    p->real_y = p->y = y;
    audio_state->playback_fullscreen.add(new RObj(55, y-5, p->w+10, p->h+10, 255, 255, 255, 255, 1));
    audio_state->playback_fullscreen.add(p);
  } else 
    x_pos = 100;
}

void GraphicalAudio::print_choose_cover(const vector<string>& covers, int cover_pos, Overlay& o)
{
  render->wait_and_aquire();

  if (o.elements.size() > 0)
    o.partial_cleanup();

  int x, y, max_x = conf->p_v_res()-250, max_y = 90;

  int cover_width = image_width+25;
  int cover_height = image_height+15;

  int covers_per_row = (max_x-30)/cover_width;
  int image_rows = (conf->p_h_res()-(2*max_y))/cover_height;

  int max = std::min(covers_per_row*image_rows, int(covers.size()));

  int box_height = cover_height*(((max % covers_per_row == 0) ? 0 : 1) + max/covers_per_row)+30+50;

  x = (conf->p_v_res()-(max_x+30))/2;
  y = (conf->p_h_res()-(box_height))/2;

  o.add(new RObj(x-15+2, y-15+2, max_x+30+1, box_height+1, 0, 0, 0, 200, 0));
  o.add(new RObj(x-15, y-15, max_x+30, box_height,  
		 themes->dialog_background1, themes->dialog_background2, 
		 themes->dialog_background3, themes->dialog_background4, 1));

  string header = gettext("choose a cover");

  int x_size = string_format::calculate_string_width(header, "Vera/20");

  o.add(new TObj(header, "Vera/20", (conf->p_v_res()-x_size)/2, y-5, 
		 themes->dialog_font1, themes->dialog_font2, themes->dialog_font3, 3));
  y += 50;
  
  // print the range
  int pos_in_list = cover_pos;
  int screen_pos, start;

  if (covers.size() > image_rows*covers_per_row) {
    if (pos_in_list < std::ceil(double(image_rows)/2.0)*covers_per_row) {
      screen_pos = pos_in_list;
      start = 0;
    } else {
      screen_pos = pos_in_list%covers_per_row+covers_per_row;
      if (screen_pos == covers_per_row-1)
	screen_pos = 2*covers_per_row-1;
      start = pos_in_list-screen_pos;
    }
  } else {
    screen_pos = pos_in_list;
    start = 0;
  }

  x = (conf->p_v_res()-covers_per_row*cover_width)/2;
  int x_start = x;

  for (int j = 0; j < image_rows*covers_per_row && j < covers.size(); ++j) {

    string pic = vector_lookup(covers, start);
    
    bool border = false;
	
    if (screen_pos == j)
      border = true;
	
    if (!file_exists(render->image_get_fullpath(pic, image_width, image_height))) {
      render->create_scaled_image_wrapper(pic, image_width, image_height);
      // to insert the image into the transmap so that it won't fail when
      // we draw the screen again
      bool tmp;
      render->image_dimensions(render->image_get_fullpath(pic, image_width, image_height), &tmp);
    }

    PFObj *p = new PFObj(pic, x, y, image_width, image_height, border, 3);
    
    p->x = p->real_x = x + (cover_width - (p->w + 10))/2;
    p->y = p->real_y = y + (cover_height - (p->h + 10))/2;

    if (p->border) {
      p->real_x -= 5;
      p->real_y -= 5;
    } else {
      p->real_x -= 1;
      p->real_y -= 1;
    }

    o.add(p);

    x += cover_width;

    if (start == covers.size()-1) {
      if ((j+1)%covers_per_row != 0)
	j += (covers_per_row-(j+1)%covers_per_row);
      
      x = x_start;
      y += cover_height;

      start = 0;
      continue;
      
    } else
      ++start;
    
    if ((j+1)%covers_per_row == 0) {
      y += cover_height;
      x = x_start;
    }
  }

  render->draw_and_release("choose cover screen", true);
}

void GraphicalAudio::choose_cover()
{
  // find all covers available
  vector<string> covers;
  int cover_pos = 0;

  MyPair filetype;
  
  for (file_iterator<file_t, default_order> i (vector_lookup(*files, position_int()).path); 
       i != i.end (); ++i)
    {
      filetype = check_type(i->getName(), conf->p_filetypes_p());
      
      if (filetype != emptyMyPair && i->getName().rfind(".front-cover.jpg") == string::npos)
	covers.push_back(i->getName());
    }

  if (covers.size() == 0) {
    DialogWaitPrint pdialog(gettext("No covers found"), 2000);
    return;
  }
  
  // display a list and let the user select one
  BusyIndicator::get_instance().idle();

  ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();
  screen_updater->timer.deactivate("audio_fullscreen");

  Overlay overlay = Overlay("dialog");

  Input input;

  while (!(input.command == "back" && input.mode == "general")) {

    print_choose_cover(covers, cover_pos, overlay);

    input = input_master->device->get_input();

    if (input.command == "left")
      {
	if (cover_pos != 0)
	  --cover_pos;
	else
	  cover_pos = covers.size() - 1;
      }
    else if (input.command == "right")
      {
	if (cover_pos != covers.size()-1)
	  ++cover_pos;
	else
	  cover_pos = 0;
      }
    else if (input.command == "prev")
      {
	if (covers.size() > 5) {
	  cover_pos = cover_pos - 5;
	  if (cover_pos < 0)
	    cover_pos = covers.size() - 1 + cover_pos;
	}
      }
    else if (input.command == "next")
      {
	cover_pos = (cover_pos + 5) % covers.size();
      }
    else if (input.command == "action")
      {
	string cover = vector_lookup(*files, position_int()).path + "/.front-cover.jpg";

	cover_map[vector_lookup(*files, position_int()).path] = cover;

	// create symlink based on selection
	if (file_exists(cover))
	  run::external_program("rm \"" + vector_lookup(*files, position_int()).path + "/.front-cover.jpg\"");
	run::external_program("ln -s \"" + vector_lookup(covers, cover_pos) + "\" \"" + 
			      vector_lookup(*files, position_int()).path + "/.front-cover.jpg\"");

	if (!file_exists(render->image_get_fullpath(cover, image_width, image_height))) {
	  render->create_scaled_image_wrapper(cover, image_width, image_height);
	  // to insert the image into the transmap so that it won't fail when
	  // we draw the screen again
	  bool tmp;
	  render->image_dimensions(render->image_get_fullpath(cover, image_width, image_height), &tmp);
	}

	break;
      }
  }

  render->wait_and_aquire();
  overlay.cleanup();
  render->image_mut.leaveMutex();

  screen_updater->timer.activate("audio_fullscreen");
}

void GraphicalAudio::print_information()
{
  BusyIndicator::get_instance().idle();

  if (isDirectory(vector_lookup(*files, position_int()).path))
    return;

  printing_information = true;

  render->prepare_new_image();

  render->current.add(new PObj(themes->audio_background, 0, 0, 0, SCALE_FULL));

  render->current.add(new PObj(themes->startmenu_music_dir, 25, 10, 2, NOSCALING));

  if (themes->show_header) {
    render->current.add(new TObj(gettext("Audio - Information"), "Vera/28", 100, 10, 
				 themes->audio_header_font1, themes->audio_header_font2, 
				 themes->audio_header_font3, 2));
  }

  int y = 100;

  Dbaudiofile dbaf = vector_lookup(*files, position_int());
  // lookup information each time see this information as the rating of the
  // song might have changed
  dbaf = Dbaudiofile(dbaf.path);

  string pic = vector_lookup(*files, position_int()).find_cover_in_current_dir();

  int max_x_pic = 240, max_y_pic = 266;

  if (conf->p_v_res() == 1024) {
    max_x_pic = 300;
    max_y_pic = 333;
  }

  int x = max_x_pic + 35 + 35;
  if (pic.empty())
    x = 50;
  
  if (!pic.empty()) {
    render->create_scaled_image_wrapper(pic, max_x_pic, max_y_pic);
    render->current.add(new PFObj(pic, 35, y+5, max_x_pic, max_y_pic, true, 3));
  }

  int w, first_column_width = 0;
  w = string_format::calculate_string_width(gettext("Artist:"), "Vera/18");
  first_column_width = std::max(w, first_column_width);
  w = string_format::calculate_string_width(gettext("Album:"), "Vera/18");
  first_column_width = std::max(w, first_column_width);
  w = string_format::calculate_string_width(gettext("Title:"), "Vera/18");
  first_column_width = std::max(w, first_column_width);
  w = string_format::calculate_string_width(gettext("Year:"), "Vera/18");
  first_column_width = std::max(w, first_column_width);
  w = string_format::calculate_string_width(gettext("Bitrate:"), "Vera/18");
  first_column_width = std::max(w, first_column_width);
  w = string_format::calculate_string_width(gettext("Length:"), "Vera/18");
  first_column_width = std::max(w, first_column_width);
  w = string_format::calculate_string_width(gettext("Rating:"), "Vera/18");
  first_column_width = std::max(w, first_column_width);
  w = string_format::calculate_string_width(gettext("Track:"), "Vera/18");
  first_column_width = std::max(w, first_column_width);

  first_column_width += 10;

  string artist = string_format::convert(dbaf.artist, true);
  string_format::format_to_size(artist, "Vera/18", conf->p_v_res()-(x+first_column_width)-25, true);
  render->current.add(new TObj(gettext("Artist:"), "Vera/18", x, y, themes->audio_font1,
			       themes->audio_font2, themes->audio_font3, 3));
  render->current.add(new TObj(artist, "Vera/18", 
			       x+first_column_width, y, 
			       themes->audio_font1, themes->audio_font2, 
			       themes->audio_font3, 3));
  y+=28;

  string album = string_format::convert(dbaf.album, true);
  string_format::format_to_size(album, "Vera/18", conf->p_v_res()-(x+first_column_width)-25, true);
  render->current.add(new TObj(gettext("Album:"), "Vera/18", x, y, 
			       themes->audio_font1, themes->audio_font2, 
			       themes->audio_font3, 3));
  render->current.add(new TObj(album, "Vera/18", 
			       x+first_column_width, y,  
			       themes->audio_font1, themes->audio_font2, 
			       themes->audio_font3, 3));
  y+=28;

  string title = string_format::convert(dbaf.title, true);
  string_format::format_to_size(title, "Vera/18", conf->p_v_res()-(x+first_column_width)-25, true);
  render->current.add(new TObj(gettext("Title:"), "Vera/18", x, y, 
			       themes->audio_font1, themes->audio_font2, 
			       themes->audio_font3, 3));
  render->current.add(new TObj(title, "Vera/18", 
			       x+first_column_width, y,  
			       themes->audio_font1, themes->audio_font2, 
			       themes->audio_font3, 3));
  y+=28;

  render->current.add(new TObj(gettext("Year:"), "Vera/18", x, y, 
			       themes->audio_font1, themes->audio_font2, 
			       themes->audio_font3, 3));
  render->current.add(new TObj(conv::itos(dbaf.year), "Vera/18", 
			       x+first_column_width, y, 
			       themes->audio_font1, themes->audio_font2, 
			       themes->audio_font3, 3));
  y+=28;

  render->current.add(new TObj(gettext("Bitrate:"), "Vera/18", x, y, 
			       themes->audio_font1, themes->audio_font2, 
			       themes->audio_font3, 3));
  render->current.add(new TObj(conv::itos(dbaf.bitrate), "Vera/18", 
			       x+first_column_width, y,
			       themes->audio_font1, themes->audio_font2, 
			       themes->audio_font3, 3));
  y+=28;

  std::ostringstream time;

  if (remaining_seconds(dbaf.length) < 10)
    time << seconds_to_minutes(dbaf.length) << ":0" << remaining_seconds(dbaf.length);
  else
    time << seconds_to_minutes(dbaf.length) << ":" << remaining_seconds(dbaf.length);
	  
  render->current.add(new TObj(gettext("Length:"), "Vera/18", x, y, 
			       themes->audio_font1, themes->audio_font2, 
			       themes->audio_font3, 3));
  render->current.add(new TObj(time.str(), "Vera/18", 
			       x+first_column_width, y,  
			       themes->audio_font1, themes->audio_font2, 
			       themes->audio_font3, 3));
  y+=28;

  std::ostringstream rating;
  if (dbaf.rating != -1)
    rating << dbaf.rating;
  else
    rating << gettext("not rated yet");

  render->current.add(new TObj(gettext("Rating:"), "Vera/18", x, y, 
			       themes->audio_font1, themes->audio_font2, 
			       themes->audio_font3, 3));
  render->current.add(new TObj(rating.str(), "Vera/18", x+first_column_width, y,  
			       themes->audio_font1, themes->audio_font2, 
			       themes->audio_font3, 3));
  y+=28;

  render->current.add(new TObj(gettext("Track:"), "Vera/18", x, y, 
			       themes->audio_font1, themes->audio_font2, 
			       themes->audio_font3, 3));
  render->current.add(new TObj(conv::itos(dbaf.track), "Vera/18", 
			       x+first_column_width, y,
			       themes->audio_font1, themes->audio_font2, 
			       themes->audio_font3, 3));
  y+=28;

  render->draw_and_release("Audio information");

  input_master->device->get_input();

  printing_information = false;
}

bool GraphicalAudio::file_exists_in_db(const string& file)
{
  return (db_id(file, false) != -1);
}

int GraphicalAudio::db_id(const string& file, bool is_dir)
{
  int i = -1;

  string good_file;

  if (is_dir)
    good_file = string_format::unique_folder_name(file);
  else
    good_file = file;

  imms_mut.enterMutex();
  SQLQuery *q = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_string(good_file) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {
    SQLRow &row = (*q)[0];
    i = conv::atoi(row["id"]);
  }
  delete q;
  imms_mut.leaveMutex();

  return i;
}

string GraphicalAudio::get_name_from_file(const Dbaudiofile& d)
{
  return d.to_string();
}

string GraphicalAudio::get_short_name_from_file(const Dbaudiofile& d)
{
  return d.short_to_string();
}

void GraphicalAudio::print_element(Dbaudiofile& dbaf, const Dbaudiofile& position, int y, bool picture_dir)
{
  if (!dbaf.fetched) {
    Dbaudiofile tmp_dbaf(dbaf.path);
    tmp_dbaf.fetched = true;
    tmp_dbaf.set_values(dbaf);
    dbaf = tmp_dbaf;
  }

  string name;

  if (mode == ADD && picture_dir)
    name = dbaf.get_name_from_file_in_picture_dir();
  else
    name = get_name_from_file(dbaf);

  if (picture_dir && mode == ADD) {

    int x_start = 358;

    if (conf->p_v_res() == 1024)
      x_start = 418;

    if (conf->p_h_res() == 405)
      x_start = 250;

    string_format::format_to_size(name, "Vera/16", conf->p_v_res()-x_start-85, true);
    
    // shorter
    if (dbaf == position)
      render->current.add(new PObj(themes->audio_marked_small, x_start, y-1, conf->p_v_res()-x_start-75, 2));

    if (isDirectory(dbaf.path))
      name.append("/");

    render->current.add(new TObj(name, "Vera/16", x_start+7, y, 
				 themes->audio_font1, themes->audio_font2, 
				 themes->audio_font3, 3));

  } else {

    int width = conf->p_v_res()-250;
    if (mode == ADD)
      width += 15;

    string_format::format_to_size(name, "Vera/16", width, true);

    if (dbaf == position) 
      render->current.add(new PObj(themes->audio_marked, 70, y-1, conf->p_v_res()-75-70, 2));

    bool printed = false;
    if (mode == PLAY)
      if (dbaf.id == audio_state->p->p_cur_nr().id && audio_state->p_playing()) {
	render->current.add(new TObj(name, "Vera/16", 75, y, 
				     themes->audio_marked_font1, themes->audio_marked_font2, 
				     themes->audio_marked_font3, 3));
	printed = true;
      }
    
    if (!printed) {
      
      if (isDirectory(dbaf.path))
	name.append("/");

      render->current.add(new TObj(name, "Vera/16", 75, y, 
				   themes->audio_font1, themes->audio_font2, 
				   themes->audio_font3, 3));
    }

    if (dbaf.length != 0) {
      char temp[512];
      snprintf(temp, 512, "%d:%02d", dbaf.length/60, dbaf.length%60);
      int x_size = string_format::calculate_string_width(temp, "Vera/16");
      render->current.add(new TObj(temp, "Vera/16", conf->p_v_res()-75-x_size, y, 
				   themes->audio_font1, themes->audio_font2, 
				   themes->audio_font3, 3));
    }

    // FIXME: inefficient
    int qpos = audio_state->queue_pos(dbaf);

    if (qpos != 0)
      render->current.add(new TObj(conv::itos(qpos), "Vera/16", conf->p_v_res()-165, y, 
				   themes->audio_font1, themes->audio_font2, 
				   themes->audio_font3, 3));
  }
}

void GraphicalAudio::print(vector<Dbaudiofile>& cur_files)
{
  if (mode == PLAY or no_folders_in_dir(cur_files))
    print_tracklist(cur_files);
  else
    print_graphical(cur_files);
}

bool GraphicalAudio::no_folders_in_dir(const vector<Dbaudiofile>& cur_files)
{
  for (vector<Dbaudiofile>::const_iterator i = cur_files.begin(), iend = cur_files.end(); i !=
	 iend; ++i)
    if (i->type == "dir")
      return false;

  return true;
}

void GraphicalAudio::options_reloaded()
{
  cover_map.clear();
  extracted_all_files = false;
}

string GraphicalAudio::get_cover_from_dir(const string& dir)
{
  // check if we have already looked in folder
  std::map<string, string>::iterator cm_iter = cover_map.find(dir);
  if (cm_iter != cover_map.end())
    return cm_iter->second;

  string pic = "";

  vector<string> pictures_found;

  MyPair filetype;

  // check for special cover first
  if (file_exists(dir + "/.front-cover.jpg")) {
    pic = dir + "/.front-cover.jpg";
  } else {
    // determine if there is a picture in this dir...
    for (file_iterator<file_t, default_order> i (dir); i != i.end (); i.advance(false))
      {
	filetype = check_type(i->getName(), conf->p_filetypes_p());
      
	if (filetype != emptyMyPair)
	  pictures_found.push_back(i->getName());
      }

    if (!pictures_found.empty()) {
      pic = pictures_found.front();
      if (pictures_found.size() > 1) {
	for (vector<string>::iterator iter = pictures_found.begin(); 
	     iter != pictures_found.end(); ++iter)
	  if (string_format::lowercase(*iter).rfind("front") != string::npos) {
	    pic = *iter;
	    break;
	  }
      }
    }
  }

  cover_map.insert(std::make_pair(dir, pic));

  return pic;
}

string GraphicalAudio::get_first_cover_in_dir(const string& dir)
{
  string cover = "";
  bool cover_found = false;

  string cur_dir = dir;

  list<string> dirs_to_try;

  while (!cover_found) {
    cover = get_cover_from_dir(cur_dir);

    if (!cover.empty()) {
      cover_found = true;
    }
    else {
      bool new_dir_found = false;
      for (file_iterator<file_t, default_order> i (cur_dir); i != i.end (); i.advance(false)) {
       if (isDirectory(i->getName())) {
         if (!new_dir_found) {
           new_dir_found = true;
           cur_dir = i->getName();
         } else {
           dirs_to_try.push_back(i->getName());
         }
       }
      }
      if (!new_dir_found) {
       if (!dirs_to_try.empty()) {
         cur_dir = dirs_to_try.front();
         dirs_to_try.pop_front();
       } else
         break;
      }
    }
  }

  return cover;
}

string GraphicalAudio::get_first_file_in_dir(const string& dir)
{
  for (file_iterator<file_t, default_order> i (dir); i != i.end (); ++i)
    {
      string filename = filesystem::FExpand(i->getName());

      const MyPair filetype = check_type(filename, audio_conf->p_filetypes_a());

      if (filetype != emptyMyPair)
       return filename;
    }

  return "";
}

void GraphicalAudio::print_dir(int cur_pos, int j, int x, int y)
{
  PObj *p;
      
  if (cur_pos == j)
    p = new PObj(themes->audio_dir_marked, x, y, 3, NOSCALING);
  else
    p = new PObj(themes->audio_dir, x, y, 3, NOSCALING);
  
  p->x = p->real_x = x + (image_width_all - p->w)/2;
  p->y = p->real_y = y + (image_height_all_eks_text - p->h)/2;
  
  render->current.add(p);
}

void GraphicalAudio::print_graphical(vector<Dbaudiofile>& cur_files)
{
  render->prepare_new_image();

  render->current.add(new PObj(themes->audio_background, 0, 0, 0, SCALE_FULL));
  
  render->current.add(new PObj(themes->startmenu_music_dir, 25, 10, 2, NOSCALING));

  if (themes->show_header) {
    render->current.add(new TObj(gettext("Audio - Add files"), "Vera/28", 100, 10, 
				 themes->audio_header_font1, themes->audio_header_font2, 
				 themes->audio_header_font3, 2));
  }
  
  if (search_mode) {

    int y = 60;
    if (conf->p_v_res() == 720) {
      y -= 3;
    }

    render->current.add(new RObj(0, y, conf->p_v_res(), 45, 
				 themes->general_search_rectangle_color1, 
				 themes->general_search_rectangle_color2, 
				 themes->general_search_rectangle_color3, 
				 themes->general_search_rectangle_color4, 2));

    render->current.add(new PObj(themes->general_search, 47, y, 3, NOSCALING));

    render->current.add(new TObj(gettext("Search: ") + search_str, "Vera/22", 100, y+5, 
				 themes->audio_font1, themes->audio_font2, 
				 themes->audio_font3, 3));
  }

  // print the range
  int screen_pos, start, pos_in_list, cur_rows, cur_image_height_all;

  if (search_mode) {
    pos_in_list = offset % cur_files.size();
    cur_rows = rows_search;
    cur_image_height_all = image_height_all_search;
  } else {
    pos_in_list = folders.top().second;
    cur_rows = rows;
    cur_image_height_all = image_height_all;
  }

  int nr_in_last_row = cur_files.size()%images_per_row;
  if (nr_in_last_row == 0)
    nr_in_last_row = images_per_row;

  if (cur_files.size() > cur_rows*images_per_row) {
    if (pos_in_list < std::ceil(double(cur_rows)/2.0)*images_per_row) {
      screen_pos = pos_in_list;
      start = 0;
    } else if (pos_in_list >= cur_files.size() - (images_per_row+nr_in_last_row)) { // last two rows
      start = cur_files.size()-((cur_rows-1)*images_per_row+nr_in_last_row);
      screen_pos = pos_in_list-start;
    } else {
      screen_pos = pos_in_list%images_per_row+images_per_row;
      if (screen_pos == images_per_row-1)
	screen_pos = 2*images_per_row-1;
      start = pos_in_list-screen_pos;
    }
  } else {
    screen_pos = pos_in_list;
    start = 0;
  }

  int x_start, x = (conf->p_v_res()-images_per_row*image_width_all)/2, y = y_start;
  x_start = x;
  
  if (search_mode) {
    y = y_start;
    if (conf->p_input() != "keyboard")
      y += 60;
  }
  
  for (int j = 0; j < cur_rows*images_per_row && j < cur_files.size(); ++j) {

    Dbaudiofile s = vector_lookup(cur_files, start);

    if (s.type == "dir") {

      string pic = get_cover_from_dir(s.path);

      if (file_exists(pic)) {

	bool border = false;
	
	if (screen_pos == j)
	  border = true;

	if (!file_exists(render->image_get_fullpath(pic, image_width, image_height))) {
	  render->create_scaled_image_wrapper(pic, image_width, image_height);
	  // to insert the image into the transmap so that it won't fail when
	  // we draw the screen again
	  bool tmp;
	  render->image_dimensions(render->image_get_fullpath(pic, image_width, image_height), &tmp);
	}

	PFObj *p = new PFObj(pic, x, y, image_width, image_height, border, 3);
	
	p->x = p->real_x = x + (image_width_all - (p->w + 10))/2;
	p->y = p->real_y = y + (image_height_all_eks_text - (p->h + 10))/2;

	if (p->border) {
	  p->real_x -= 5;
	  p->real_y -= 5;
	} else {
	  p->real_x -= 1;
	  p->real_y -= 1;
	}

	render->current.add(p);
      } else
	print_dir(screen_pos, j, x, y);

    } else {

      PObj *p = new PObj(themes->audio_file, x-5, y-5, 3, NOSCALING);
      p->x = p->real_x = x + (image_width_all - (p->w + 10))/2;
      p->y = p->real_y = y + (image_height_all_eks_text - (p->h + 10))/2;
      render->current.add(p);
      
      if (!s.fetched) {
	Dbaudiofile tmp_dbaf(s.path);
	tmp_dbaf.fetched = true;
	tmp_dbaf.set_values(s);
	vector_lookup(cur_files, start) = tmp_dbaf;
	s = tmp_dbaf;
      }
    }

    string name = get_name_from_file(s); 

    string old_name = name;
    
    string_format::format_to_size(name, "Vera/13", image_width_all-10, false);

    string name_second_part = old_name.substr(name.size());

    if (name_second_part.size() > 0 && name_second_part[0] != ' ') {
      // splitting words
      string::size_type last_whole_word = name.rfind(' ');
      if (last_whole_word != string::npos) {
	name = name.substr(0, last_whole_word);
	name_second_part = old_name.substr(name.size());
      }
    }

    int x_size = string_format::calculate_string_width(name, "Vera/13");

    string_format::format_to_size(name_second_part, "Vera/13", image_width_all-10, false);

    int x_size2 = string_format::calculate_string_width(name_second_part, "Vera/13");

    if (screen_pos == j) {
      render->current.add(new TObj(name, "Vera/13", x + (image_width_all-x_size)/2, y+image_height_all_eks_text, 
				   themes->audio_marked_font1, themes->audio_marked_font2, 
				   themes->audio_marked_font3, 3));
      render->current.add(new TObj(name_second_part, "Vera/13", x + (image_width_all-x_size2)/2, y+image_height_all_eks_text+20, 
				   themes->audio_marked_font1, themes->audio_marked_font2, 
				   themes->audio_marked_font3, 3));
    } else {
      render->current.add(new TObj(name, "Vera/13", x + (image_width_all-x_size)/2, y+image_height_all_eks_text, 
				   themes->audio_font1, themes->audio_font2, 
				   themes->audio_font3, 3));
      render->current.add(new TObj(name_second_part, "Vera/13", x + (image_width_all-x_size2)/2, y+image_height_all_eks_text+20, 
				   themes->audio_font1, themes->audio_font2, 
				   themes->audio_font3, 3));
    }
    
    x += image_width_all;

    if (start == cur_files.size()-1) {
      if ((j+1)%images_per_row != 0)
	j += (images_per_row-(j+1)%images_per_row);
      
      x = x_start;
      y += cur_image_height_all;

      start = 0;
      continue;
      
    } else
      ++start;
    
    if ((j+1)%images_per_row == 0) {
      y += image_height_all;
      x = x_start;
    }
  }

  std::ostringstream position;

  if (search_mode)
    if (cur_files.size() > 0)
      position << (offset % cur_files.size()) + 1 << "/" << cur_files.size();
    else
      position << "";
  else
    position << folders.top().second+1 << "/" << cur_files.size();
  
  int x_size = string_format::calculate_string_width(position.str(), "Vera/18");
    
  render->current.add(new TObj(position.str(), "Vera/18", conf->p_v_res()-(60+x_size), 20, 
			       themes->audio_font1, themes->audio_font2, 
			       themes->audio_font3, 3));

  render->draw_and_release("audio-graphical");
}

void GraphicalAudio::print_tracklist(vector<Dbaudiofile>& cur_files)
{
  print_top();

  if (search_mode && conf->p_input() != "keyboard") 
    search_top_size += 35;

  string pic = "";
  if (position_int() < cur_files.size())
    pic = vector_lookup(*files, position_int()).find_cover_in_current_dir();

  if (cur_files.size() > 0) {
    BoolPrintFunctorNonConst<GraphicalAudio, Dbaudiofile> print_func(this, &GraphicalAudio::print_element, 
								     !pic.empty());

    if (search_mode && cur_files.size() > 0)
      print_range_modifying(cur_files, vector_lookup(cur_files, offset), 
			    offset, &print_func);
    else
      print_range_modifying(cur_files, vector_lookup(cur_files, position_int()), position_int(), 
			    &print_func);

    if (mode == ADD) {

      if (!pic.empty()) {

	if (cur_files.size() > 0) {

	  int cur_pos = position_int();

	  if (search_mode && cur_files.size() > 0)
	    cur_pos = offset;

	  Dbaudiofile dbaf = vector_lookup(cur_files, cur_pos);

	  int max_x_pic = 240, max_y_pic = 266;
	  
	  if (conf->p_v_res() == 1024) {
	    max_x_pic = 300;
	    max_y_pic = 333;
	  } else if (conf->p_h_res() == 405) {
	    max_x_pic = 150;
	    max_y_pic = 167;
	  }

	  string artist = string_format::convert(string_format::trim(dbaf.artist), true);
	  string album = string_format::convert(string_format::trim(dbaf.album), true);
 
	  string_format::format_to_size(artist, "Vera/20", max_x_pic, false);
	  string_format::format_to_size(album, "Vera/18", max_x_pic, false);
	
	  int y_artist = 100;
	  int y_album = 135;
	  int y_image = 175;

	  if (search_mode) {
	    y_artist += search_top_size;
	    y_album += search_top_size;
	    y_image += search_top_size;
	  }

	  render->current.add(new TObj(artist, "Vera/20", 75, y_artist, 
				       themes->audio_font1, themes->audio_font2, 
				       themes->audio_font3, 3));

	  render->current.add(new TObj(album, "Vera/18", 75, y_album, 
				       themes->audio_font1, themes->audio_font2, 
				       themes->audio_font3, 3));

	  // scale it
	  render->create_scaled_image_wrapper(pic, max_x_pic, max_y_pic);
	  render->current.add(new PFObj(pic, 75, y_image, max_x_pic, max_y_pic, true, 3));
	}
      }
    }
  }

  print_buttom(cur_files.size());

  if (search_mode && conf->p_input() != "keyboard")
    search_top_size -= 35;
}

vector<Dbaudiofile> GraphicalAudio::add_dir(const string& dir, bool from_media)
{
  int nr_files = 0;
  for (file_iterator<file_t, default_order> i (dir); i != i.end (); ++i)
    if (!isDirectory(i->getName()) && check_type(i->getName(), audio_conf->p_filetypes_a()) != emptyMyPair)
      ++nr_files;

  ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();

  screen_updater->status_progressbar = 0;
  screen_updater->total_progressbar = nr_files;
  screen_updater->header = gettext("Extracting metadata");

  TimeFunctoriNC cp(&check_progressbar);
  TimeFunctorvNC pg(&print_progressbar);

  screen_updater->timer.add(TimeElement("progressbar", &cp, &pg));

  vector<Dbaudiofile> cur_files;

  string cur_directory = "";
  vector<Dbaudiofile> files_in_dir;

  for (file_iterator<file_t, default_order> i (dir); i != i.end (); ++i)
    {
      if (global->check_stop_bit()) {
	cur_files.clear();
	break;
      }

      string filename = filesystem::FExpand(i->getName());

      const MyPair filetype = check_type(filename, audio_conf->p_filetypes_a());

      if (filetype != emptyMyPair) {
	if (cur_directory != i.currentDirectory()) {
	  cur_directory = i.currentDirectory();
	  add_tracks_from_dir(files_in_dir, cur_files);
	}

	Dbaudiofile d(addsimplefile(filename, filetype, from_media));
	files_in_dir.push_back(d);

	++screen_updater->status_progressbar;
      }
    }

  // add tracks from the last directory
  add_tracks_from_dir(files_in_dir, cur_files);

  screen_updater->timer.del("progressbar");
  cleanup_progressbar();

  return cur_files;
}

void GraphicalAudio::add_tracks_from_dir(vector<Dbaudiofile> &files_in_dir, vector<Dbaudiofile> &cur_files)
{
  // check if all the files are missing the track tag
  bool missing_track = true;
  for (vector<Dbaudiofile>::iterator fid = files_in_dir.begin(), 
	 fid_end = files_in_dir.end(); fid != fid_end; ++fid)
    if (fid->track != 0) {
      missing_track = false;
      break;
    }

  if (missing_track) {

    // just add all the tracks, we don't care about the order since its
    // probably ok already

    for (vector<Dbaudiofile>::iterator fid = files_in_dir.begin(), 
	   fid_end = files_in_dir.end(); fid != fid_end; ++fid)
      cur_files.push_back(*fid);
    
    files_in_dir.clear();

  } else {

    while (!files_in_dir.empty()) {
      // find the track with the smallest id
      int smallest_track = 1000000;
      vector<Dbaudiofile>::iterator element;

      for (vector<Dbaudiofile>::iterator fid = files_in_dir.begin(), 
	     fid_end = files_in_dir.end(); fid != fid_end; ++fid)
	if (fid->track < smallest_track) {
	  smallest_track = fid->track;
	  element = fid;
	}
      assert(smallest_track != 1000000);
      cur_files.push_back(*element);
      files_in_dir.erase(element);
    }
  }
}

bool GraphicalAudio::rdir_internal(const string& filename, const string& argv, 
				   vector<Dbaudiofile> &cur_files)
{
  if (global->check_stop_bit()) {
    cur_files.clear();
    return false;
  }

  if (isDirectory (filename))
    {
      // dir
      Simplefile r;
      r.id = ++id;
      r.name = filename.substr(argv[argv.size()-1] == '/' ? argv.size() : argv.size() + 1);
      if (conf->p_convert())
	r.name = string_format::convert(r.name);
      r.lowercase_name = string_format::lowercase(r.name);
      if (filename[filename.size()-1] != '/')
	r.path = filename + '/';
      else
	r.path = filename;
      r.type = "dir";
      Dbaudiofile d(r);
      d.db_id = db_id(filename, true);
      assert(d.db_id != -1);
      cur_files.push_back(d);
    } else {
      // file
      const MyPair filetype = check_type(filename, audio_conf->p_filetypes_a());
      if (filetype != emptyMyPair) {
	Dbaudiofile d(addsimplefile(filename, filetype));
	d.db_id = db_id(filename, false);
	assert(d.db_id != -1);
	cur_files.push_back(d);

	++(S_ScreenUpdater::get_instance())->status_progressbar;
      }
    }

  return true;
}

vector<string> GraphicalAudio::check_db_for_folders(const string& parent)
{
  vector<string> names;

  string good_parent = string_format::unique_folder_name(parent);

  SQLQuery *q = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_string(good_parent) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {
    assert(q->numberOfTuples() == 1);
    SQLRow &row = (*q)[0];
    string parent_id = row["id"];
    SQLQuery *q2 = db.query("Folders", ("SELECT filename FROM %t WHERE parent='" + parent_id + "'").c_str());
    if (q2) {
      for (int i = 0; i < q2->numberOfTuples(); ++i) {
	SQLRow &row = (*q2)[i];
	names.push_back(row["filename"]);
      }
    }
    delete q2;
  }
  delete q;

  return names;
}

void GraphicalAudio::insert_file_into_db(const string& filename, const string& parent)
{
  string parent_id = "";
  
  string good_parent = string_format::unique_folder_name(parent);

  SQLQuery *q = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_string(good_parent) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {
    assert(q->numberOfTuples() == 1);
    SQLRow &row = (*q)[0];
    parent_id = row["id"];
  }
  delete q;

  if (parent_id.empty()) {
    char *tmp = sqlite3_mprintf("INSERT INTO Folders VALUES(NULL, '0', '%q', '', '1')", good_parent.c_str());
    db.execute(tmp);
    sqlite3_free(tmp);

    q = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_string(good_parent) + "'").c_str());
    if (q && q->numberOfTuples() > 0) {
      assert(q->numberOfTuples() == 1);
      SQLRow &row = (*q)[0];
      parent_id = row["id"];
    }
    delete q;
  }

  string::size_type i;
  if ((i = filename.rfind('/')) == string::npos)
    i = 0;
  string name = filename.substr((i != 0) ? i+1 : i);
  
  bool is_dir = isDirectory(filename);

  if (!is_dir && (i = name.rfind('.')) != string::npos)
    name = name.substr(0, i);

  string good_filename;

  if (is_dir)
    good_filename = string_format::unique_folder_name(filename);
  else
    good_filename = filename;

  char *tmp_insert = sqlite3_mprintf("INSERT INTO Folders VALUES(NULL, '%q', '%q', '%q', '%q')", 
				     parent_id.c_str(), good_filename.c_str(), name.c_str(),
				     conv::itos(is_dir).c_str());
  db.execute(tmp_insert);
  sqlite3_free(tmp_insert);
}

vector<Dbaudiofile> GraphicalAudio::rdir(const string& argv)
{
  vector<Dbaudiofile> cur_files;

  vector<string> db_files = check_db_for_folders(argv);
  
  if (db_files.size() > 0) {
    for (vector<string>::const_iterator i = db_files.begin(), iend = db_files.end();
	 i != iend; ++i)
      if (!rdir_internal(*i, argv, cur_files))
	break;
  } else {
    for (file_iterator<file_t, default_order> i (argv); i != i.end (); i.advance(false)) {
      string filename = filesystem::FExpand(i->getName());
      insert_file_into_db(filename, argv);
      if (!rdir_internal(filename, argv, cur_files))
	break;
    }
  }
  
  vector<Dbaudiofile> sorted_by_id_files;

  add_tracks_from_dir(cur_files, sorted_by_id_files);

  return sorted_by_id_files;
}

void GraphicalAudio::read_dirs() 
{
  while (!all_metadata_files.empty())
    all_metadata_files.pop();

  vector<Dbaudiofile> playlist_new;
  
  int nr_files = 0;
  list<string> dirs = audio_conf->p_audio_dirs();
  for (list<string>::const_iterator iter = dirs.begin(); iter != dirs.end(); ++iter)
    for (file_iterator<file_t, default_order> i (*iter); i != i.end (); i.advance(false))
      if (!isDirectory (i->getName()) && 
	  check_type(i->getName(), audio_conf->p_filetypes_a()) != emptyMyPair)
	++nr_files;

  ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();

  screen_updater->status_progressbar = 0;
  screen_updater->total_progressbar = nr_files;
  screen_updater->header = gettext("Extracting metadata");

  TimeFunctoriNC cp(&check_progressbar);
  TimeFunctorvNC pg(&print_progressbar);

  screen_updater->timer.add(TimeElement("progressbar", &cp, &pg));

  // process audio dir 
  audio_list = parse_dir(audio_conf->p_audio_dirs());

  screen_updater->timer.del("progressbar");
  cleanup_progressbar();

  if (audio_conf->p_audio_dirs().size() > 1)
    std::sort(audio_list.begin(), audio_list.end(), file_sort());

  if (audio_list.size() != 0)
    // set position to the first file
    folders.top().second = 0;

  if (mode == ADD)
    files = &audio_list;
  else
    files = &playlist;

#ifdef use_inotify
  S_Notify::get_instance()->register_plugin("audio", audio_conf->p_audio_dirs(),
					    new TimeFunctorv<Audio>(this, &Audio::reparse_current_dir));
#endif
}

void GraphicalAudio::intelligent_random_next()
{
  imms_mut.enterMutex();

  imms->playlist_changed(playlist.size());
  next_track_pos = -1;
  imms->select_next();

  int time_left =  audio_state->p->p_total_time() - audio_state->p->p_cur_time();

  if (time_left < 3)
    time_left = 0;

  bool forced = time_left;

  Simplefile cur_nr = audio_state->p->p_cur_nr();

  if (imms_cur_path != cur_nr.path) {
    // FIXME: it should probably end the old song if it is playing
    for (int i = 0, end = playlist.size(); i < end; ++i)
      if (vector_lookup(playlist, i).id == cur_nr.id) {
	imms->start_song(i, cur_nr.path);
	break;
      }
  }

  imms->end_song(!time_left, forced, false);

  while (next_track_pos == -1)
    usleep(50*1000); // wait for next track

  audio_state->p->set_cur_nr(vector_lookup(playlist, next_track_pos));

  imms_cur_path = audio_state->p->p_cur_nr().path;

  imms->start_song(next_track_pos, imms_cur_path);

  audio_state->direction = Audio_s::ADDED;

  imms_mut.leaveMutex();
}

void GraphicalAudio::intelligent_random_prev()
{
  intelligent_random_next();
}

void GraphicalAudio::extract_metadata()
{
  if (!extracted_all_files && (conf->p_idle_time() > 0) && 
      (opts.metadata_extract() == gettext("yes"))) {

    int nr_extracts = 3;

    // check if we are idle
    int idle_time = time(0)-conf->p_last_key();
    if (idle_time < 3)
      return;
    else if (idle_time > 60 * conf->p_idle_time())
      nr_extracts = 4;

    if (all_metadata_files.empty()) {
      list<string> dirs = audio_conf->p_audio_dirs();
      for (list<string>::const_iterator iter = dirs.begin(); iter != dirs.end(); ++iter)
	for (file_iterator<file_t, default_order> i (*iter); i != i.end (); ++i)
	  all_metadata_files.push(i->getName());
      // optimization, we don't want to both make the list and start extracting
      return;
    }

    while (!all_metadata_files.empty()) {
      if (!file_exists_in_db(all_metadata_files.top())) {
	Dbaudiofile(all_metadata_files.top());
	if (nr_extracts == 0)
	  return;
	else
	  --nr_extracts;
      }
      all_metadata_files.pop();
    }

    extracted_all_files = true;
  } 
}

int GraphicalAudio::check_metadata()
{
  return 3000; // ms
}

void GraphicalAudio::startup_updater()
{
  Audio::startup_updater();
  TimeFunctori<GraphicalAudio> *time_func = new TimeFunctori<GraphicalAudio>(this, &GraphicalAudio::check_metadata);
  TimeFunctorv<GraphicalAudio> *call_func = new TimeFunctorv<GraphicalAudio>(this, &GraphicalAudio::extract_metadata);
  BackgroundUpdater *background_updater = S_BackgroundUpdater::get_instance();
  background_updater->timer.add(TimeElement("extra_metadata", time_func, call_func));

  EnterFunctor<GraphicalAudio> *e = new EnterFunctor<GraphicalAudio>(this, &GraphicalAudio::reenter);
  SearchGenList<GraphicalAudio> *s = new SearchGenList<GraphicalAudio>(this, &GraphicalAudio::gen_search_list);
  S_Search::get_instance()->register_module(SearchModule("Audio", s, e));
}
