#include "config.h"

#include "audio_player.hpp"

#include "audio.hpp"
#include "audio_s.hpp"

#include "dbaudiofile.hpp"

#include "common.hpp"
#include "rand.hpp"
#include "gettext.hpp"

#include <vector>
#include <string>
#include <fstream>

using std::vector;
using std::string;

AudioPlayer::AudioPlayer(const string& t, const string& art, 
			 const string& al, int ct, int tt, int bs) :
  buffering_state(bs), is_streaming(false), old_volume(0), 
  title(t), artist(art), album(al), cur_time(ct), total_time(tt), 
  play_next_track(false)
{
  audio_conf = S_AudioConfig::get_instance();
}

void AudioPlayer::stop()
{
  Audio_s *audio_state = S_Audio_s::get_instance();
  audio_state->set_pause(false);

  stop_player();

  // FIXME
#ifdef use_media
  sleep(1);
  Audio *audio = Audio::get_instance();
  audio->pos_umount();
#endif
}

void AudioPlayer::next()
{
  Audio_s *audio_state = S_Audio_s::get_instance();

  if (!audio_state->p_playing())
    return;
    
  if (!audio_state->check_repeat()) 
    audio_state->set_playing(false);
  
  if (audio_state->p_playing()) {

    audio_state->get_audio()->next_helper();

#ifdef use_media
    Audio *audio = Audio::get_instance();
    audio->check_mount_before(cur_nr);
#endif

    addfile(cur_nr);

#ifdef use_media
    audio->check_mount_after(cur_nr);
#endif

  } else
    stop();
}

void AudioPlayer::prev()
{
  Audio_s *audio_state = S_Audio_s::get_instance();
  Audio *audio = audio_state->get_audio();

  if (!audio_state->p_playing())
    return;

  if (audio_state->p_playing()) {

    audio->prev_helper();
  
#ifdef use_media
    audio->check_mount_before(cur_nr);
#endif

    addfile(cur_nr);

#ifdef use_media
    audio->check_mount_after(cur_nr);
#endif

  } else
    stop();
}

std::pair<int, bool> AudioPlayer::load_runtime_settings()
{
  Config *conf = S_Config::get_instance();

  // sane defaults
  int volume = 52, playlist_pos = 0;
  bool play_now_warning = true;

  std::string path = conf->p_var_data_dir() + "options/Audio Runtime";

  std::ifstream in;
  in.open(path.c_str(), std::ios::in);
  
  if (!in.is_open()) {
    DebugPrint perror(gettext("Could not open options file ") + path,
                      Print::DEBUGGING, DebugPrint::CRITICAL, "AUDIO");
  } else {
   
    string name, value, str;
    
    while (getline(in, str)) {

      string::size_type comma = str.find(",");

      if (comma == string::npos)
        continue;
      
      name = str.substr(0,comma);
      value = str.substr(comma+1);

      if (name == "volume")
        volume = conv::atoi(value);
      else if (name == "playlist_pos")
	playlist_pos = conv::atoi(value);
      else if (name == "play_now_warning")
	play_now_warning = conv::stob(value);
    }
    in.close();
  }

  setvol(volume);

  return std::make_pair(playlist_pos, play_now_warning);
}
