#include "config.h"

#include "libfspp.hpp"

#include "simple_audio.hpp"

#include "graphics.hpp"

#ifdef use_inotify
#include "notify.hpp"
#endif

using std::string;
using std::vector;
using std::list;

void SimpleAudio::page_up()
{
  int jump = conf->p_jump();
  
  if (files->size() > jump) {
    int pos = position_int()-jump;
    if (position_int() == 0)
      set_position_int(files->size()-1+pos);
    else if (pos < 0)
      set_position_int(0);
    else
      set_position_int(pos);
  }
}

void SimpleAudio::page_down()
{
  int jump = conf->p_jump();

  if (files->size() > jump) {
    if (position_int() > (files->size() - jump) && position_int() != (files->size()-1))
      set_position_int(files->size()-1);
    else
      set_position_int((position_int()+jump)%files->size());
  }
}

string SimpleAudio::mainloop(AudioMode new_mode)
{
  BusyIndicator &busy_indicator = BusyIndicator::get_instance();
  Shutdown *sd = S_Shutdown::get_instance();

  mainloop_top(new_mode);

  Input input;

  bool update_needed = true;
  
  while (!exit_loop)
    {
      if (update_needed) {
	print(*files);
	print_lcd_menu();
      }

      busy_indicator.idle();

      input = input_master->device->get_input();

      busy_indicator.busy();

      if (sd->is_enabled()) {
	sd->cancel();
	continue;
      }

      update_needed = true;

      // general
      if (input.command == "prev")
	{
	  if (position_int() != 0)
	    set_position_int(position_int()-1);
	  else
	    set_position_int(files->size()-1);
	}
      else if (input.command == "next")
	{
	  set_position_int((position_int()+1)%files->size());
	}
      else if (input.command == "back" && input.mode == "general")
	{
	  exit();
	}
      else if (input.command == "page_up")
 	{
	  page_up();
 	}
      else if (input.command == "page_down")
 	{
	  page_down();
 	}
      else if (input.command == "second_action")
	{
	  secondary_menu();
	}
      else if (input.command == "search")
	{
	  search_func();
	}
      else if (input.command == "options")
	{
	  options();
	}
      // playlist
      else if (mode == PLAY && input.command == "move_up")
	{
	  move_up();
	}
      else if (mode == PLAY && input.command == "move_down")
	{
	  move_down();
	}
      else if (mode == PLAY && input.command == "delete")
	{
	  delete_track();
	}
      else if (mode == PLAY && input.command == "queue")
	{
	  queue_track();
	}
      else if (mode == PLAY && input.command == "clear_list")
	{
	  clear_playlist();
	}
      else if (mode == PLAY && input.command == "save_playlist")
	{
	  save_playlist_func();
	}
      else if (mode == PLAY && input.command == "action")
	{
	  play_track();
	}
      // audio-add
      else if (mode == ADD) {

	if (input.command == "action")
	  {
	    add();
	  }
	else if (input.command == "right")
	  {
	    enter_dir();
	  }
	else if (input.command == "left")
	  {
	    leave_dir();
	  }
	else if (input.command == "add_all")
	  {
	    add_all();
	  }
	else if (input.command == "play_now")
	  {
	    if (vector_lookup(*files, position_int()).type != "dir")
		play_track_now();
	  }
      }

      update_needed = !global->check_commands(input);
    }

  exit_loop = false;

  if (in_playlist)
    in_playlist = false;

  return "";
}

void SimpleAudio::print_audiotrack_helper(int &x_pos, string& buffer, 
					  string& album, string& artist, string& title)
{
  if (audio_state->p->p_cur_nr().type == "web")
    buffer = format_time(audio_state->p->p_cur_time());
  else
    buffer = format_time(audio_state->p->p_cur_time(), audio_state->p->p_total_time());
  artist = audio_state->p->p_artist();
  album = audio_state->p->p_album();
  title = audio_state->p->p_title();
}

void SimpleAudio::secondary_menu_in_add(ExtraMenu &em)
{
  bool is_dir = (vector_lookup(*files, position_int()).type == "dir");

  std::string t = gettext("Add track to playlist");

  if (is_dir)
    t = gettext("Add directory to playlist");

  em.add_item(ExtraMenuItem(t, input_master->find_shortcut("action"), 
			    new TimeFunctorv<SimpleAudio>(this, &SimpleAudio::add)));
	    
  if (is_dir)
    em.add_item(ExtraMenuItem(gettext("Enter directory"), 
			      input_master->find_shortcut("right"), 
			      new TimeFunctorv<SimpleAudio>(this, &SimpleAudio::enter_dir)));

  if (folders.size() > 1)
    em.add_item(ExtraMenuItem(gettext("Go up one directory"), 
			      input_master->find_shortcut("left"), 
			      new TimeFunctorv<SimpleAudio>(this, &SimpleAudio::leave_dir)));
}

void SimpleAudio::secondary_menu()
{
  ExtraMenu em;
  secondary_menu_helper(em);
  em.mainloop();
}

void SimpleAudio::enter_dir()
{
  if (vector_lookup(*files, position_int()).type == "dir") {
    string temp_cur_dir = vector_lookup(*files, position_int()).path;

    vector<Simplefile> templist = rdir(temp_cur_dir);

    if (templist.size() != 0) {
      list<string> templs; templs.push_back(temp_cur_dir + "/");
      folders.push(std::make_pair(templs, 0));
      audio_list = templist;
      files = &audio_list;

#ifdef use_inotify
      S_Notify::get_instance()->register_plugin("audio", folders.top().first,
						new TimeFunctorv<Audio>(this, &Audio::reparse_current_dir));
#endif

    } else
      DialogWaitPrint pdialog(gettext("Folder is empty"), 1000);
  }
}

string SimpleAudio::get_name_from_file(const Simplefile& s)
{
  return s.to_string();
}

string SimpleAudio::get_short_name_from_file(const Simplefile& s)
{
  return get_name_from_file(s);
}

void SimpleAudio::print_element(const Simplefile& r, const Simplefile& position, int y)
{
  string name = r.name;
  string_format::format_to_size(name, "Vera/16", conf->p_v_res()-(75+85+10), true);

  if (r == position)
    render->current.add(new PObj("audio/marked.png", 67, y-1, conf->p_v_res()-135, 2));

  if (r == audio_state->p->p_cur_nr() && mode == PLAY && audio_state->p_playing()) 
    render->current.add(new TObj(name, "Vera/16", 75, y, 
				 themes->audio_marked_font1, themes->audio_marked_font2, 
				 themes->audio_marked_font3, 3));
  else {
    
    if (isDirectory(r.path))
      name.append("/");

    render->current.add(new TObj(name, "Vera/16", 75, y, 
				 themes->audio_font1, themes->audio_font2, 
				 themes->audio_font3, 3));
  }


  // FIXME: inefficient
  int qpos = audio_state->queue_pos(r);

  if (qpos != 0)
    render->current.add(new TObj(conv::itos(qpos), "Vera/16", conf->p_v_res()-85, y, 
				 themes->audio_font1, themes->audio_font2, 
				 themes->audio_font3, 3));
}

void SimpleAudio::print(const vector<Simplefile>& cur_files)
{
  print_top();

  if (cur_files.size() > 0) {
    PrintFunctor<SimpleAudio, Simplefile> print_func(this, &SimpleAudio::print_element);
    if (search_mode && cur_files.size() > 0)
      print_range(cur_files, vector_lookup(cur_files, offset % cur_files.size()), 
		  offset % cur_files.size(), &print_func);
    else
      print_range(cur_files, vector_lookup(cur_files, position_int()), 
		  position_int(), &print_func);
  }

  print_buttom(cur_files.size());
}

vector<Simplefile> SimpleAudio::add_dir(const string& dir, bool from_media)
{
  vector<Simplefile> cur_files;

  for (file_iterator<file_t, default_order> i (dir); i != i.end (); ++i)
    {
      if (global->check_stop_bit()) {
	cur_files.clear();
	break;
      }

      string filename = filesystem::FExpand(i->getName());

      const MyPair filetype = check_type(filename, audio_conf->p_filetypes_a());

      if (filetype != emptyMyPair)
        cur_files.push_back(addsimplefile(filename, filetype, from_media));
    }

  return cur_files;
}

vector<Simplefile> SimpleAudio::rdir(const string& argv)
{
  vector<Simplefile> cur_files;

  for (file_iterator<file_t, default_order> i (argv); i != i.end (); i.advance(false))
    {
      if (global->check_stop_bit()) {
	cur_files.clear();
	break;
      }

      string filename = filesystem::FExpand(i->getName());

      if (isDirectory (filename))
	{
	  // dir
	  Simplefile r;
	  r.id = ++id;
	  r.name = filename.substr(argv[argv.size()-1] == '/' ? argv.size() : argv.size() + 1);
	  if (conf->p_convert())
	    r.name = string_format::convert(r.name);
	  r.lowercase_name = string_format::lowercase(r.name);
	  r.path = filename;
	  r.type = "dir";
	  cur_files.push_back(r);
	} else {
	  // file
	  const MyPair filetype = check_type(filename, audio_conf->p_filetypes_a());
	  if (filetype != emptyMyPair)
	    cur_files.push_back(addsimplefile(filename, filetype));
	}
    }

  return cur_files;
}

void SimpleAudio::read_dirs() 
{
  vector<Simplefile> playlist_new;
  
  // process audio dir 
  audio_list = parse_dir(audio_conf->p_audio_dirs());
  
  if (audio_conf->p_audio_dirs().size() > 1)
    std::sort(audio_list.begin(), audio_list.end(), file_sort());

  if (audio_list.size() != 0)
    // set position to the first file
      folders.top().second = 0;

  if (mode == ADD)
    files = &audio_list;
  else
    files = &playlist;
}
