#include "epg_pic.hpp" 

#include "graphics.hpp"

using std::list;
using std::string;

EpgPic::EpgPic()
  : Epg(78)
{}

void EpgPic::print_timelines(double diff)
{
  if (diff > 0 && diff < timeslots)
    render->current.add(new RObj(int(x_start + (diff*30*60)/scale_factor), y_start, 2, rows*40+10,
				 themes->epg_time_bar1, themes->epg_time_bar2, 
				 themes->epg_time_bar3, themes->epg_time_bar4, 1));
  
  render->current.add(new RObj(x_start, y_start, 2, rows*40+10, 
			       themes->epg_bar1, themes->epg_bar2, 
			       themes->epg_bar3, themes->epg_bar4, 1));

  for (int x = 0; x < timeslots; ++x) {
    time_t tmp_time = cur_time + x*30*60;
    char time_str[30];
    strftime(time_str, 30, "%H:%M", localtime(&tmp_time));
    render->current.add(new TObj(time_str, "Vera/17", 
				 static_cast<int>(x_start-28 + (x*30*60)/scale_factor), y_start-30, 
				 themes->epg_font1, themes->epg_font2, 
				 themes->epg_font3, 2));
    render->current.add(new RObj(static_cast<int>(x_start + ((x+1)*30*60)/scale_factor), y_start, 2, rows*40+10,
				 themes->epg_bar1, themes->epg_bar2, themes->epg_bar3, 
				 themes->epg_bar4, 1));
  }
}

void EpgPic::print_channel(int y, std::list<Channel>::iterator cur_chan)
{
  render->current.add(new PObj(cur_chan->logo, 15, y*40+y_start+10, 2, NOSCALING, false));
}

void EpgPic::print_desc_channel(const std::string& orig_title)
{
  render->current.add(new PObj(sel_channel->logo, 75, 100, 2, NOSCALING, false));

  string title = orig_title;
  string_format::format_to_size(title, "Vera/22", conf->p_v_res()-(145 + 70), true);
  
  render->current.add(new TObj(title, "Vera/22", 140, 95,
			       themes->epg_font1, themes->epg_font2, themes->epg_font3, 2));
}

bool EpgPic::check_tv_file()
{
  // check if the icons can be found
  bool icons = true;

  for (list<Channel>::iterator chan = channels.begin(), end = channels.end(); 
       chan != end; ++chan) {

    string::size_type dot_pos = chan->id.find(".");

    if (dot_pos == string::npos) {
      dot_pos = chan->id.size();
      DebugPrint perror(gettext("Error in channel id, please report this problem"), 
			Print::DEBUGGING, DebugPrint::CRITICAL, "EPG");
    }
      
    chan->logo = conf->p_homedir() + "epg_icons/" 
      + chan->id.substr(0, chan->id.find(".")) + ".gif";
    if (!file_exists(chan->logo)) {
      DebugPrint perror(gettext("Failed to find icon ") + chan->logo, 
			Print::DEBUGGING, DebugPrint::CRITICAL, "EPG");
      icons = false;
      break;
    }
  }

  if (!icons) {
    Print pdialog(Print::SCREEN);
    pdialog.add_line(gettext("Could not find icons for epg"));
    pdialog.add_line("");
    pdialog.add_line(gettext("Please install the icons and try again"));
    pdialog.print();

    return false;
  } else
    return Epg::check_tv_file();
}
