#include "config.h"

#include "plugin.hpp"

#include "game.hpp"

#include "common-feature.hpp"
#include "themes.hpp"
#include "print.hpp"
#include "cd.hpp"
#include "config.hpp"

void no_games_error()
{
  Print pdialog(Print::SCREEN);
  pdialog.add_line(gettext("Could not find any games"));
  pdialog.add_line("");
  pdialog.add_line(gettext("Please specify a correct path "));
  pdialog.add_line(gettext("in the configuration file"));
  pdialog.print();
}

void game_fromhd()
{
  Game *game = get_class<Game>("Game");

  if (!game->loaded_correctly) {
    game->read_dirs();
    game->loaded_correctly = true;
  } 

  if (game->reload_needed) {
    DialogWaitPrint pdialog(gettext("Reloading directories"), 1000);
    game->read_dirs();
  }
	    
  if (game->files_size() == 0) {
    no_games_error();
    game->reload_needed = true;
  } else {
    game->reload_needed = false;
    game->mainloop();
  }
}

#ifdef use_media
void game_cd()
{
  Cd *cd = S_Cd::get_instance();

  if (cd->open()) {

    Cd::cdstatus cdstatus = cd->check_cddrive();

    if (cdstatus == Cd::OK)
      get_class<Game>("Game")->determine_media();
  }
}
#endif

class GamePlugin : public FeaturePlugin
{
public:
  GamePlugin() {
    Config *conf = S_Config::get_instance();
    GameConfig *game_conf = S_GameConfig::get_instance();
    
    game_conf->parse_configuration_file(conf->p_homedir());

    module = new Game();

    Themes *themes = S_Themes::get_instance();

    std::string text;

    // FIXME
    //    if (game_conf->p_game_hd()) {
      text = gettext("Play games from harddrive");

      // add game button
      if (!conf->p_media())
	text = gettext("Play games");

      features.push_back(startmenu_item(text, "game_hd", themes->startmenu_game_dir, 
					0, VoidFunctor(&game_fromhd)));
      //    }

#ifdef use_media
      if (conf->p_media()) {

      text = gettext("Play a dvd/vcd/svcd");
    
      // add dvd/vcd/svcd button
      // FIXME
      //      if (!game_conf->p_game_hd())
	text = gettext("Play games");

      features.push_back(startmenu_item(text, "game_disc", themes->startmenu_game_cd, 
					1, VoidFunctor(&game_cd)));
    }
#endif
  }
  
  std::string plugin_name() const { return gettext("Game"); }
  int priority() const { return 2; }
  unsigned long capability() const { return 0; }
};

MMS_PLUGIN(GamePlugin)
