#ifndef MOVIE_DB_HPP
#define MOVIE_DB_HPP

#include "movie.hpp"
#include "imdb.hpp"

#include "cpp_sqlitepp.hpp"

#include "singleton.hpp"

template <typename TClass>
class MovieIMDbFunctor
{
 private:
  void (TClass::*fpt)(std::string, IMDB_MOVIELIST, int, Overlay&);
  TClass* pt2Object;
  IMDB_MOVIELIST list;
  int p;
  Overlay &o;

 public:
  
  MovieIMDbFunctor(TClass* _pt2Object, 
		   void(TClass::*_fpt)(std::string, IMDB_MOVIELIST, int, Overlay&), 
		   IMDB_MOVIELIST i, int pos, Overlay &overlay)
    : p(pos), o(overlay)
  { pt2Object = _pt2Object; fpt=_fpt; list = i; };

  void call(const std::string& r)
  { return (*pt2Object.*fpt)(r, list, p, o); };
};

class MovieDB : public MovieTemplate<CIMDBMovie>
{
protected:
  SQLDatabase db;

  // commands
  virtual void secondary_menu() = 0;
  void reget_movie_information();
  void print_info();
  void search_func();
  void search_imdb();

  void prev();
  void next();
  void left();
  void right();

  void page_up();
  void page_down();

  // helper
  void imdb_download_movie(CIMDBMovie &m, const CIMDBUrl& url);

  bool imdb_search_mainloop(const std::string& search_word, CIMDBMovie &m, bool search);
  void imdb_search_screen(std::string search_word, IMDB_MOVIELIST results, int position, Overlay& o);

  bool create_thumbnail(const std::string& file, const std::string& thumbnail);

  void print_information(const CIMDBMovie& m);
  void print(const std::vector<CIMDBMovie>& files);

  std::string get_name_from_file(const CIMDBMovie& m);

  virtual std::vector<std::pair<std::string, int> > gen_search_list(const std::string& search_word) = 0;

  void check_db();
  void create_db();

  int image_width;
  int image_height;
  int image_height_all_eks_text;
  int image_height_all_search;

  // including text and space around image
  int image_width_all;
  int image_height_all;

  int images_per_row;
  int rows;
  int rows_search;

  int y_start;

  std::string imdb_dir;

  CIMDB imdb;

  // true means that the movies reside on hd
  bool physical;

public:

  bool is_physical()
  {
    return physical;
  }

  SQLDatabase* get_db() { return &db; }

  // physical means that it exists on the hd
  MovieDB(const std::string& filename, bool physical);
};

typedef Singleton<MovieDB> S_MovieDB;

#endif
