#ifndef MPLAYER_HPP
#define MPLAYER_HPP

#include "movie_player.hpp"

#include "common-feature.hpp"

#include <cc++/thread.h>

class PlaybackChecker;

class Mplayer : public MoviePlayer
{
public:
  Mplayer();

  bool wants_exclusivity();

  void play_movie(const std::string& paths, bool window = false);
  void play_disc(const std::string& play_cmd);
  void play_vcd(const std::string& device);
  void play_dvd(const std::string& device);

  void play();
  void pause();
  void stop();
  void ff();
  void fb();
  void switch_screen_size();
  bool fullscreen();
  bool playing();

  void send_command(const std::string& command);

  Pipe p;
  bool paused, running, in_fullscreen;
  float pos;
  bool stop_playback_ok;

private:

  std::string get_audio_mode();

  void setup_playback(const std::pair<bool, Pipe>& result, bool window);
  void playback_ended();

  void stop_and_switch();

  std::pair<bool, Pipe> play_movie_in_slave_mode(char *buffer, const std::string& sub_lang, 
						 const std::string& audio_mode, const std::string& paths);

  bool broken_pipe;

  enum PlaybackType { MOVIE, MOVIE_DISC, VCD, DVD };

  // for switching between fullscreen and window
  PlaybackType type;
  std::string path;

  PlaybackChecker *checker;
};

class PlaybackChecker : public ost::Thread
{
public:
  PlaybackChecker(Mplayer *mplayer);

  bool stop;

private:
  void run();
  Mplayer *player;
};

#endif
