#include "simple_movie.hpp"

#include "common-feature.hpp"

#include "startmenu.hpp"
#include "graphics.hpp"
#include "extra_menu.hpp"
#include "busy_indicator.hpp"
#include "shutdown.hpp"

#include "libfspp.hpp"

// ostringstream
#include <sstream>

using std::string;
using std::vector;
using std::list;

// commands
void SimpleMovie::secondary_menu()
{
  ExtraMenu em;

  em.add_item(ExtraMenuItem(gettext("Play movie"), input_master->find_shortcut("action"), 
			    new TimeFunctorv<SimpleMovie>(this, &SimpleMovie::action_play)));

  if (vector_lookup(files, folders.top().second).type != "file")
    em.add_item(ExtraMenuItem(gettext("Enter directory"), input_master->find_shortcut("right"), 
			      new TimeFunctorv<SimpleMovie>(this, &SimpleMovie::enter_dir)));
	  
  if (folders.size() > 1)
    em.add_item(ExtraMenuItem(gettext("Go up one directory"), input_master->find_shortcut("left"), 
			      new TimeFunctorv<SimpleMovie>(this, &SimpleMovie::go_back)));

  em.add_item(ExtraMenuItem(gettext("Return to startmenu"), input_master->find_shortcut("back"), 
			    new TimeFunctorv<SimpleMovie>(this, &SimpleMovie::exit)));

  add_standard(em);

  for (vector<ExtraMenuItem>::const_iterator i = global->menu_items.begin(), 
	 end = global->menu_items.end(); i != end; ++i)
    em.add_persistent_item(*i);

  conf->s_sam(true);

  em.mainloop();
}

void SimpleMovie::go_back()
{
  if (!(folders.size() == 1)) {
    folders.pop();
    load_current_dirs();

#ifdef use_inotify
    S_Notify::get_instance()->register_plugin("movie", folders.top().first,
					      new TimeFunctorv<Movie>(this, &Movie::reparse_current_dir));
#endif
  }
}

void SimpleMovie::enter_dir()
{
  if (vector_lookup(files, folders.top().second).type != "file") {
    string temp_cur_dir = vector_lookup(files ,folders.top().second).path;
    vector<Multifile> templist = rdir(temp_cur_dir);

    if (templist.size() != 0) {
      list<string> templs; templs.push_back(temp_cur_dir + "/");
      folders.push(std::make_pair(templs, 0));
      files = templist;

#ifdef use_inotify
      S_Notify::get_instance()->register_plugin("movie", folders.top().first,
						new TimeFunctorv<Movie>(this, &Movie::reparse_current_dir));
#endif
    } else {
      DialogWaitPrint pdialog(gettext("Folder is empty"), 1000);
    }
  }
}

void SimpleMovie::search_func()
{
  BusyIndicator::get_instance().idle();

  ConstFunctor<SimpleMovie, std::vector<Multifile>, void> print_func(this, &SimpleMovie::print);
  ConstFunctor<SimpleMovie, Multifile, bool> compare_func(this, &SimpleMovie::search_compare);
  ConstFunctor<SimpleMovie, Multifile, string> rep_func(this, &SimpleMovie::get_name_from_file);

  input_master->device->search(files, &folders.top().second, 
			       &print_func, &compare_func, &rep_func,
			       search_mode, search_str, lowercase_search_str, offset);
}

void SimpleMovie::page_up()
{
  int jump = conf->p_jump();

  if (files.size() > jump) {
    int diff = folders.top().second-jump;
    if (folders.top().second == 0)
      folders.top().second = (files.size()-1)+diff;
    else if (diff < 0)
      folders.top().second = 0;
    else
      folders.top().second = diff;
  }
}

void SimpleMovie::page_down()
{
  int jump = conf->p_jump();

  if (files.size() > jump) {
    if (folders.top().second > (files.size() - jump) && folders.top().second != (files.size()-1))
      folders.top().second = files.size()-1;
    else
      folders.top().second = (folders.top().second+jump)%files.size();
  }
}

string SimpleMovie::mainloop()
{
  BusyIndicator &busy_indicator = BusyIndicator::get_instance();
  Shutdown *sd = S_Shutdown::get_instance();

  Input input;

  bool update_needed = true;

  while (!exit_loop)
    {
      if (update_needed) {
	print(files);
	print_lcd_menu();
      }

      busy_indicator.idle();

      input = input_master->device->get_input();

      busy_indicator.busy();

      if (sd->is_enabled()) {
	sd->cancel();
	continue;
      }

      update_needed = true;

      if (input.command == "prev")
 	{
	  if (folders.top().second != 0)
	    folders.top().second -= 1;
	  else
	    folders.top().second = files.size() - 1;
 	}
      else if (input.command == "next")
 	{
	  folders.top().second = (folders.top().second+1)%files.size();
 	}
      else if (input.command == "left")
	{
	  go_back();
	}
      else if (input.command == "right")
	{
	  enter_dir();
	}
      else if (input.command == "search")
	{
	  search_func();
	} 
      else if (input.command == "back")
	{
	  exit();
	}
      else if (input.command == "action")
	{
	  action_play();
	}
      else if (input.command == "second_action")
	{
	  secondary_menu();
        } 
      else
	MovieTemplate<Multifile>::movie_mainloop_common(input);

      update_needed != global->check_commands(input);
    }

  exit_loop = false;

  return "";
}

// print a video element
void SimpleMovie::print_movie_element(const Multifile& r, const Multifile& position, int y)
{
  string name = r.name;

  if (r.type != "file")
    name += "/";

  string_format::format_to_size(name, "Vera/16", conf->p_v_res()-75-155, true);
  
  if (r == position)
    render->current.add(new PObj(themes->movie_marked, 70, y-1, conf->p_v_res()-(2*70), 2));

  std::ostringstream out; out << r.filenames.size();

  render->current.add(new TObj(name, "Vera/16", 75, y, 
			       themes->movie_font1, themes->movie_font2, 
			       themes->movie_font3, 3));
  render->current.add(new TObj(r.filetype, "Vera/16", conf->p_v_res()-145, y, 
			       themes->movie_font1, themes->movie_font2, 
			       themes->movie_font3, 3));
  render->current.add(new TObj(out.str(), "Vera/16", conf->p_v_res()-85, y, 
			       themes->movie_font1, themes->movie_font2, 
			       themes->movie_font3, 3));
}

void SimpleMovie::print(const vector<Multifile>& cur_files)
{
  render->prepare_new_image();

  render->current.add(new PObj(themes->movie_background, 0, 0, 0, SCALE_FULL));
  
  render->current.add(new PObj(themes->startmenu_movie_dir, 25, 10, 2, NOSCALING));
  
  if (themes->show_header) {
    render->current.add(new TObj(gettext("Movies"), "Vera/28", 100, 10, 
				 themes->movie_header_font1, themes->movie_header_font2, 
				 themes->movie_header_font3, 2));
  }
  
  if (search_mode) {
    render->current.add(new RObj(0, 60, conf->p_v_res(), 45, 
				 themes->general_search_rectangle_color1, 
				 themes->general_search_rectangle_color2, 
				 themes->general_search_rectangle_color3, 
				 themes->general_search_rectangle_color4, 2));

    render->current.add(new PObj(themes->general_search, 47, 60, 3, NOSCALING));

    render->current.add(new TObj(gettext("Search: ") + search_str, "Vera/22", 100, 65, 
				 themes->movie_font1, themes->movie_font2, 
				 themes->movie_font3, 3));
  }

  if (cur_files.size() > 0) {
    PrintFunctor<SimpleMovie, Multifile> print_func(this, &SimpleMovie::print_movie_element);
    if (search_mode && cur_files.size() > 0)
      print_range(cur_files, vector_lookup(cur_files, offset), 
		  offset, &print_func);
    else
      print_range(cur_files, vector_lookup(cur_files, folders.top().second), 
		  folders.top().second, &print_func);
  }

  std::ostringstream pos;

  if (search_mode)
    if (cur_files.size() > 0)
      pos << offset + 1 << "/" << cur_files.size();
    else
      pos << "";
  else
    pos << folders.top().second+1 << "/" << cur_files.size();
  
  int x = string_format::calculate_string_width(pos.str(), "Vera/18");
    
  render->current.add(new TObj(pos.str(), "Vera/18", conf->p_v_res()-(60+x), 20, 
			       themes->movie_font1, themes->movie_font2, 
			       themes->movie_font3, 3));

  render->draw_and_release("movie");
}

vector<Multifile> SimpleMovie::parse_dir(const list<string>& dirs)
{
  vector<Multifile> vec_files;

  for (list<string>::const_iterator iter = dirs.begin(); iter != dirs.end(); ++iter)
    {
      vector<Multifile> tempfiles = rdir(*iter);

      for (vector<Multifile>::const_iterator iter2 = tempfiles.begin(); iter2 != tempfiles.end(); ++iter2)
	vec_files.push_back(*iter2);
    }

  return vec_files;
}

// read a dir and spit out a list of files and dirs. For movies.
vector<Multifile> SimpleMovie::rdir(const string& argv)
{
  vector<Multifile> cur_files;
  
  for (file_iterator<file_t, default_order> i (argv); i != i.end (); i.advance(false))
    {
      if (global->check_stop_bit()) {
	cur_files.clear();
	break;
      }

      string filename = filesystem::FExpand(i->getName());

      if (isDirectory (filename)) { // dir
	Multifile dir = add_dir(filename);
	if ((dir.filenames).size() != 0 || dir.filetype == "dir")
	  cur_files.push_back(dir);
      } else { // file
	MyPair filetype = check_type(filename, movie_conf->p_filetypes_m());
	if (filetype != emptyMyPair)
	  cur_files.push_back(addfile(filename, filetype));
      }
    }
  return cur_files;
}
