#include "gui/guicontrol.hpp"

pymms::gui::GUIControl::GUIControl(int x, int y, int w, int h, int l) : guicontrol_mutex("guicontrol")
{
  this->x = (x>-1)?x:0;
  this->y = (y>-1)?y:0;
  width = (w>-1)?w:0;
  height = (h>-1)?h:0;
  layer = (l>-1)?l:0;
  focus = false;
  visible = true;
  right = left = prev = next = this;
  id = 0;
  parent_id = 0;
}

void pymms::gui::GUIControl::reset()
{
  ost::MutexLock lock(guicontrol_mutex);
  focus = false;
  visible = true;
  right = left = prev = next = this;
  id = 0;
}

void pymms::gui::GUIControl::control_left(GUIControl* l)
{
  ost::MutexLock lock(guicontrol_mutex);
  left = l;
}

void pymms::gui::GUIControl::control_right(GUIControl* r)
{
  ost::MutexLock lock(guicontrol_mutex);
  right = r;
}

void pymms::gui::GUIControl::control_prev(GUIControl* p)
{
  ost::MutexLock lock(guicontrol_mutex);
  prev = p;
}

void pymms::gui::GUIControl::control_next(GUIControl* n)
{
  ost::MutexLock lock(guicontrol_mutex);
  next = n;
}

void pymms::gui::GUIControl::set_focus(bool f)
{
  ost::MutexLock lock(guicontrol_mutex);
  focus = f;
}

bool pymms::gui::GUIControl::get_focus()
{
  ost::MutexLock lock(guicontrol_mutex);
  return focus;
}

void pymms::gui::GUIControl::set_visible(bool v)
{
  ost::MutexLock lock(guicontrol_mutex);
  visible = v;
}

bool pymms::gui::GUIControl::get_visible()
{
  ost::MutexLock lock(guicontrol_mutex);
  return visible;
}

void pymms::gui::GUIControl::set_height(int h)
{
  ost::MutexLock lock(guicontrol_mutex);
  height = (h>-1)?h:height;
}

int pymms::gui::GUIControl::get_height()
{
  ost::MutexLock lock(guicontrol_mutex);
  return height;
}

void pymms::gui::GUIControl::set_width(int w)
{
  ost::MutexLock lock(guicontrol_mutex);
  width = (w>-1)?w:width;
}

int pymms::gui::GUIControl::get_width()
{
  ost::MutexLock lock(guicontrol_mutex);
  return width;
}

void pymms::gui::GUIControl::set_layer(int l)
{
  ost::MutexLock lock(guicontrol_mutex);
  layer = (l>-1)?l:layer;
}

int pymms::gui::GUIControl::get_layer()
{
  ost::MutexLock lock(guicontrol_mutex);
  return layer;
}

void pymms::gui::GUIControl::set_id(int id)
{
  ost::MutexLock lock(guicontrol_mutex);
  this->id = id;
}

int pymms::gui::GUIControl::get_id()
{
  ost::MutexLock lock(guicontrol_mutex);
  return id;
}

void pymms::gui::GUIControl::set_parent_id(int id)
{
  ost::MutexLock lock(guicontrol_mutex);
  parent_id = id;
}

int pymms::gui::GUIControl::get_parent_id()
{
  ost::MutexLock lock(guicontrol_mutex);
  return parent_id;
}

void pymms::gui::GUIControl::set_position(int x, int y)
{
  ost::MutexLock lock(guicontrol_mutex);
  this->x = (x>-1)?x:this->x;
  this->y = (y>-1)?y:this->y;
}

void pymms::gui::GUIControl::on_right()
{
  if(focus && right->get_id() && right->get_visible())
  {
    focus = false;
    right->set_focus(true);
  }
}

void pymms::gui::GUIControl::on_left()
{
  if(focus && left->get_id() && left->get_visible())
  {
    focus = false;
    left->set_focus(true);
  }
}

void pymms::gui::GUIControl::on_prev()
{
  if(focus && prev->get_id() && prev->get_visible())
  {
    focus = false;
    prev->set_focus(true);
  }
}

void pymms::gui::GUIControl::on_next()
{
  if(focus && next->get_id() && next->get_visible())
  {
    focus = false;
    next->set_focus(true);
  }
}

bool pymms::gui::GUIControl::on_action(std::string action)
{
  ost::MutexLock lock(guicontrol_mutex);
  bool retval = false;

  if(action == "right")
  {
    on_right();
    retval = true;
  }

  if(action == "left")
  {
    on_left();
    retval = true;
  }

  if(action == "prev")
  {
    on_prev();
    retval = true;
  }

  if(action == "next")
  {
    on_next();
    retval = true;
  }

  return retval;
}

void pymms::gui::GUIControl::free_resources()
{}
