#include "gui/guilabelcontrol.hpp"
#include "common.hpp"
#include "text_object.hpp"

pymms::gui::GUILabelControl::GUILabelControl(int x, int y, int w, int h, std::string t, int l, 
			         std::string f, std::string rgb, int tos, 
				 std::string alig) : GUIControl(x, y, w, h, l)
{
  font_size = r = g = b = 0;
  textoffset = tos;
  text = t;
  font = f;
  this->rgb = rgb;
  alignment = alig;
  renderer = S_Render::get_instance();
}

void pymms::gui::GUILabelControl::hex_to_rgb()
{
  int hexrgb;

  sscanf(rgb.c_str(), "%x", &hexrgb);
  r = (hexrgb & 0xff0000) >> 16;
  g = (hexrgb & 0x00ff00) >> 8;
  b = hexrgb & 0x0000ff;
}

void pymms::gui::GUILabelControl::calc_font_size()
{
  font_size = height;
  while(font_size > 0)
  {
    real_size = string_format::calculate_string_size(text, font + "/" + conv::itos(font_size));
    if(real_size.second <= height)
      break;
    else
      font_size--;
  }
}

std::string pymms::gui::GUILabelControl::get_label()
{
  ost::MutexLock lock(guicontrol_mutex);
  return text;
}

void pymms::gui::GUILabelControl::set_label(std::string t)
{
  ost::MutexLock lock(guicontrol_mutex);
  text = t;
}

void pymms::gui::GUILabelControl::set_labelfont(std::string f)
{
  ost::MutexLock lock(guicontrol_mutex);
  font = f;
}

void pymms::gui::GUILabelControl::set_labelrgb(std::string rgb)
{
  ost::MutexLock lock(guicontrol_mutex);
  this->rgb = rgb;
}

int pymms::gui::GUILabelControl::get_realwidth()
{
  ost::MutexLock lock(guicontrol_mutex);
  calc_font_size();

  return real_size.first;
}

int pymms::gui::GUILabelControl::get_realheight()
{
  ost::MutexLock lock(guicontrol_mutex);
  calc_font_size();

  return real_size.second;
}
  

void pymms::gui::GUILabelControl::render()
{
  ost::MutexLock lock(guicontrol_mutex);
  hex_to_rgb();
  calc_font_size();

  std::string font_to_use = font + "/" + conv::itos(font_size);
  std::string temp_text = text;

  string_format::format_to_size(temp_text, font_to_use, (width-textoffset)>0?(width-textoffset):0, false);
  std::pair<int, int> size = string_format::calculate_string_size(temp_text, font_to_use);

  int temp_textoffset = textoffset;
  if(alignment == "right")
    temp_textoffset = width - size.first;
  else if(alignment == "center")
    temp_textoffset = (width - size.first)/2;
  else
    temp_textoffset = textoffset;

  renderer->current.add(new TObj(temp_text, font_to_use, x+temp_textoffset, y, r, g, b, layer + 1));
}  
