#include "gui/guilistcontrol.hpp"
#include "gui/guibuttoncontrol.hpp"

pymms::gui::GUIListControl::GUIListControl(int x, int y, int w, int h, int l, std::string foc, std::string nofoc, 
                               std::string f, std::string rgb, std::string rgbfocus, int tos, std::string alig, 
			       int itemh, int itemsp) : GUIControl(x, y, w, h, l)
{
  font = f;
  this->rgb = rgb;
  this->rgbfocus = rgbfocus;
  image_focus = foc;
  image_nofocus = nofoc;
  textoffset = tos;
  alignment = alig;
  itemheight = (itemh>0)?itemh:30;
  itemspace = (itemsp>0)?itemsp:1;
  pos = 0;
}

pymms::gui::GUIListControl::~GUIListControl()
{
}

void pymms::gui::GUIListControl::set_labelfont(std::string f)
{
  ost::MutexLock lock(guicontrol_mutex);
  font = f;
}

void pymms::gui::GUIListControl::set_labelrgb(std::string rgb)
{
  ost::MutexLock lock(guicontrol_mutex);
  this->rgb = rgb;
}

void pymms::gui::GUIListControl::set_labelrgbfocus(std::string rgbfocus)
{
  ost::MutexLock lock(guicontrol_mutex);
  this->rgbfocus = rgbfocus;
}

bool pymms::gui::GUIListControl::add_item(GUIListItem *item)
{
  ost::MutexLock lock(guicontrol_mutex);
  bool retval = true;

  for (std::vector<GUIListItem*>::iterator i = items.begin(),
      end = items.end(); i != end; i++) {
    if((*i) == item)
    {
      retval =  false;
      break;
    }
  }

  if(retval)
    items.push_back(item);

  return retval;
}

void pymms::gui::GUIListControl::clear()
{
  ost::MutexLock lock(guicontrol_mutex);
  items.clear();
  pos = 0;
}

void pymms::gui::GUIListControl::free_resources()
{
  ost::MutexLock lock(guicontrol_mutex);
  for (std::vector<GUIListItem*>::iterator i = items.begin(),
      end = items.end(); i != end; i++) {
    delete (*i);
  }

  items.clear();
  pos = 0;
}

pymms::gui::GUIListItem *pymms::gui::GUIListControl::get_selected_item()
{
  ost::MutexLock lock(guicontrol_mutex);
  GUIListItem * retval;

  if(items.size())
    retval =  items[pos];
  else
    retval =  0;

  return retval;
}

int pymms::gui::GUIListControl::get_selected_position()
{
  ost::MutexLock lock(guicontrol_mutex);
  return pos;
}

bool pymms::gui::GUIListControl::on_action(std::string action)
{
  ost::MutexLock lock(guicontrol_mutex);
  bool retval = false;

  if(action == "right")
  {
    on_right();
    retval = true;
  }

  if(action == "left")
  {
    on_left();
    retval = true;
  }

  if(action == "prev" && focus)
  {
    if(pos > 0)
      pos--;
    else
      pos = items.size() - 1;

    retval = true;
  }

  if(action == "next" && focus)
  {
    if(pos < (items.size() - 1))
      pos++;
    else
      pos = 0;

    retval = true;
  }

  return retval;
}

void pymms::gui::GUIListControl::set_itemheight(int itemh)
{
  ost::MutexLock lock(guicontrol_mutex);
  itemheight = (itemh>0)?itemh:itemheight;
}

void pymms::gui::GUIListControl::set_itemspace(int itemsp)
{
  ost::MutexLock lock(guicontrol_mutex);
  itemspace = (itemsp>0)?itemsp:itemspace;
}

void pymms::gui::GUIListControl::render()
{
  ost::MutexLock lock(guicontrol_mutex);
  if(items.size())
  {
    GUIButtonControl *button = 0;
    GUILabelControl *label = 0;
    GUIImageControl *thumbimage = 0;
    int cutoff = 0;
    bool has_thumbimage = false;

    if(height < (itemheight + itemspace))
      height = itemheight + itemspace;

    int visible_items = height/(itemheight + itemspace);

    int diff = pos + 1 - visible_items;
    if(diff <= 0)
    {
      for(int i = 0; i < visible_items; i++)
      {
        if(i<items.size())
        {
	  if(items[i]->get_thumbimage().length() > 0)
          {
	    thumbimage = new GUIImageControl(x + textoffset, y + i*(itemheight+itemspace), itemheight, itemheight, items[i]->get_thumbimage(), layer + 1);
	    thumbimage->render();
            delete thumbimage;
            thumbimage = 0;

	    has_thumbimage = true;
	  }

  	  if(items[i]->get_label2().length() > 0)
          {
            label = new GUILabelControl(x, y + i*(itemheight+itemspace), width, itemheight, (i<items.size())?items[i]->get_label2():"", layer + 1, font, ((i==pos) && focus)?rgbfocus:rgb, 0, "right");
            label->render();
            cutoff = label->get_realwidth() + 5;
            delete label;
            label = 0;
   	  }

          button = new GUIButtonControl(x, y + i*(itemheight+itemspace), width, itemheight, 
		  		      (i<items.size())?items[i]->get_label():"", layer, image_focus, image_nofocus, font, 
				       rgb, rgbfocus, textoffset + ((has_thumbimage)?(itemheight+6):0), alignment, cutoff);

          if(i == pos)
	    button->set_focus(focus);
          button->render();
          delete button;
	  button = 0;

	  cutoff = 0;
	  has_thumbimage = false;
        }
      }
    }
    else
    {
      for(int i = diff; i < (visible_items+diff); i++)
      {
 	if(items[i]->get_thumbimage().length() > 0)
        {
          thumbimage = new GUIImageControl(x + textoffset, y + (i-diff)*(itemheight+itemspace), itemheight, itemheight, items[i]->get_thumbimage(), layer + 1);
          thumbimage->render();
          delete thumbimage;
          thumbimage = 0;

	  has_thumbimage = true;
        }

  	if(items[i]->get_label2().length() > 0)
        {
	  label = new GUILabelControl(x, y + (i-diff)*(itemheight+itemspace), width, itemheight, items[i]->get_label2(), layer + 1, font, ((i==pos) && focus)?rgbfocus:rgb, 0, "right");
          label->render();
          cutoff = label->get_realwidth() + 5;
          delete label;
          label = 0;
	}

        button = new GUIButtonControl(x, y + (i-diff)*(itemheight+itemspace), width, itemheight, 
	 			      items[i]->get_label(), layer, image_focus, image_nofocus, font, rgb, 
				      rgbfocus, textoffset + ((has_thumbimage)?(itemheight+6):0), alignment, cutoff);
        if(i == pos)
	  button->set_focus(focus);
        button->render();
        delete button;
	button = 0;

	cutoff = 0;
	has_thumbimage = false;
      }
    }
  }
}  
