#include "common.hpp"

#include "gui/guitextboxcontrol.hpp"
#include "gui/guilabelcontrol.hpp"

pymms::gui::GUITextBoxControl::GUITextBoxControl(int x, int y, int w, int h, int l, 
                               std::string f, std::string rgb,
			       int itemh, int itemsp) : GUIControl(x, y, w, h, l)
{
  text = "";
  font = f;
  this->rgb = rgb;
  itemheight = (itemh>0)?itemh:30;
  itemspace = (itemsp>=0)?itemsp:0;
  font_size = pos = 0;
}

pymms::gui::GUITextBoxControl::~GUITextBoxControl()
{
  items.clear();
  pos = 0;
}

void pymms::gui::GUITextBoxControl::set_textfont(std::string f)
{
  ost::MutexLock lock(guicontrol_mutex);
  font = f;
  calc_font_size();
  split_text();
}

void pymms::gui::GUITextBoxControl::set_textrgb(std::string rgb)
{
  ost::MutexLock lock(guicontrol_mutex);
  this->rgb = rgb;
}

void pymms::gui::GUITextBoxControl::set_text(std::string t)
{
  ost::MutexLock lock(guicontrol_mutex);
  text = t;
  pos = 0;
  calc_font_size();
  split_text();
}

void pymms::gui::GUITextBoxControl::clear()
{
  ost::MutexLock lock(guicontrol_mutex);
  items.clear();
  pos = 0;
}

bool pymms::gui::GUITextBoxControl::on_action(std::string action)
{
  ost::MutexLock lock(guicontrol_mutex);
  bool retval = false;

  if(action == "right")
  {
    on_right();
    retval = true;
  }

  if(action == "left")
  {
    on_left();
    retval = true;
  }

  if(action == "prev" && focus)
  {
    int visible_items = height/(itemheight + itemspace);

    if(pos > visible_items)
      pos--;
    else
      pos = 0;

    retval = true;
  }

  if(action == "next" && focus)
  {
    int visible_items = height/(itemheight + itemspace);

    if( (pos == 0) && (items.size() > visible_items) )
      pos += visible_items;
    else if( (pos > 0) && (pos < (items.size() - 1)) )
      pos++;

    retval = true;
  }

  return retval;
}

void pymms::gui::GUITextBoxControl::set_textheight(int th)
{
  ost::MutexLock lock(guicontrol_mutex);
  itemheight = (th>0)?th:30;
  calc_font_size();
  split_text();
}

void pymms::gui::GUITextBoxControl::set_textspace(int ts)
{
  ost::MutexLock lock(guicontrol_mutex);
  itemspace = (ts>=0)?ts:0;
}

void pymms::gui::GUITextBoxControl::calc_font_size()
{
  font_size = itemheight;
  while(font_size > 0)
  {
    std::pair<int, int> size = string_format::calculate_string_size("test", font + "/" + conv::itos(font_size));
    if(size.second <= height)
      break;
    else
      font_size--;
  }
}

void pymms::gui::GUITextBoxControl::split_text()
{
  int linepos = 0;

  items.clear();
  while(linepos < text.size())
  {
    items.push_back(string_format::get_line(text, linepos, width, font + "/" + conv::itos(font_size)));
  }
}

void pymms::gui::GUITextBoxControl::render()
{
  ost::MutexLock lock(guicontrol_mutex);
  if(items.size())
  {
    GUILabelControl *label = 0;

    if(height < (itemheight + itemspace))
      height = itemheight + itemspace;

    int visible_items = height/(itemheight + itemspace);

    int diff = pos + 1 - visible_items;
    if(diff <= 0)
    {
      for(int i = 0; i < visible_items; i++)
      {
        label = new GUILabelControl(x, y + i*(itemheight+itemspace), width, itemheight, 
		  		      (i<items.size())?items[i]:"", layer, font, rgb);
        label->render();
        delete label;
	label = 0;
      }
    }
    else
    {
      for(int i = diff; i < (visible_items+diff); i++)
      {
        label = new GUILabelControl(x, y + (i-diff)*(itemheight+itemspace), width, itemheight, 
	 			     items[i], layer, font, rgb);
        label->render();
        delete label;
	label = 0;
      }
    }
  }
}  
