#include "gui/guiwindow.hpp"

#include <iostream>

pymms::gui::GUIWindow::GUIWindow() : guiwindow_mutex("guiwindow")
{
  config = S_Config::get_instance();
  control_id = 0;
  window_id = 0;
}

pymms::gui::GUIWindow::~GUIWindow()
{
}

void pymms::gui::GUIWindow::render()
{
  ost::MutexLock lock(guiwindow_mutex);
  if(controls.size())
  {
    for (std::vector<GUIControl*>::iterator i = controls.begin(),
       end = controls.end(); i != end; i++) {
      if((*i)->get_visible())
      {
        (*i)->render();
      }
    }
  }
}

bool pymms::gui::GUIWindow::add_control(GUIControl* c)
{
  ost::MutexLock lock(guiwindow_mutex);
  bool retval = false;

  if(!c->get_id())
  {
    c->set_id(++control_id);
    c->set_parent_id(window_id);
    controls.push_back(c);

    retval =  true;
  }    

  return retval;
}

bool pymms::gui::GUIWindow::on_action(std::string action)
{
  ost::MutexLock lock(guiwindow_mutex);
  bool retval = false;

  for (std::vector<GUIControl*>::iterator i = controls.begin(),
      end = controls.end(); i != end; i++) {
    if( (*i)->get_focus())
    {
      retval = (*i)->on_action(action);
      break;
    }
  }

  return retval;
}

bool pymms::gui::GUIWindow::set_focus(GUIControl *c)
{
  ost::MutexLock lock(guiwindow_mutex);
  bool retval = false;
  GUIControl *prevfocus = get_focus();

  for (std::vector<GUIControl*>::iterator i = controls.begin(),
       end = controls.end(); i != end; i++) {
    if((*i) == c)
    {
      if(prevfocus)
	prevfocus->set_focus(false);

      c->set_focus(true);
      retval =  true;
      break;
    }
  }

  return retval;
}

pymms::gui::GUIControl *pymms::gui::GUIWindow::get_focus()
{
  ost::MutexLock lock(guiwindow_mutex);
  GUIControl *retval = 0;

  for (std::vector<GUIControl*>::iterator i = controls.begin(),
      end = controls.end(); i != end; i++) {
    if((*i)->get_focus())
    {  
      retval =  *i;
      break;
    }
  }
  
  return retval;
}

bool pymms::gui::GUIWindow::remove_control(GUIControl *c)
{
  ost::MutexLock lock(guiwindow_mutex);
  bool retval = false;

  for (std::vector<GUIControl*>::iterator i = controls.begin(),
      end = controls.end(); i != end; i++) {
    if((*i) == c)
    {
      c->reset();

      controls.erase(i);
      retval =  true;
      break;
    }
  }

  return retval;
}

int pymms::gui::GUIWindow::get_width()
{
  ost::MutexLock lock(guiwindow_mutex);
  return config->p_v_res();
}

int pymms::gui::GUIWindow::get_height()
{
  ost::MutexLock lock(guiwindow_mutex);
  return config->p_h_res();
}

void pymms::gui::GUIWindow::set_id(int id)
{
  ost::MutexLock lock(guiwindow_mutex);
  window_id = id;

  if(controls.size())
  {
    for (std::vector<GUIControl*>::iterator i = controls.begin(),
       end = controls.end(); i != end; i++) {
      (*i)->set_parent_id(window_id);
    }
  }
}

int pymms::gui::GUIWindow::get_id()
{
  ost::MutexLock lock(guiwindow_mutex);
  return window_id;
}
