#include "gui/guiwindowmanager.hpp"
#include <iostream>

pymms::gui::GUIWindowManager::GUIWindowManager() : guiwindowmanager_mutex("guiwindowmanager") 
{
  renderer = S_Render::get_instance();
}

bool pymms::gui::GUIWindowManager::on_action(std::string action)
{
  if(windows.size())
    return  windows.front()->on_action(action);
}

void pymms::gui::GUIWindowManager::register_window(GUIWindow *window)
{
  guiwindowmanager_mutex.enterMutex();
  for (std::list<GUIWindow*>::iterator i = windows.begin(),
       end = windows.end(); i != end; i++) {
    if((*i) == window)
    {
      guiwindowmanager_mutex.leaveMutex();
      return;
    }
  }

  windows.push_back(window);
  guiwindowmanager_mutex.leaveMutex();
}

void pymms::gui::GUIWindowManager::unregister_window(GUIWindow *window)
{
  guiwindowmanager_mutex.enterMutex();
  for (std::list<GUIWindow*>::iterator i = windows.begin(),
       end = windows.end(); i != end; i++) {
    if((*i) == window)
    {
      windows.erase(i);
      guiwindowmanager_mutex.leaveMutex();

      render();
      return;
    }
  }

  guiwindowmanager_mutex.leaveMutex();
}

void pymms::gui::GUIWindowManager::set_active_window(GUIWindow *window)
{
  guiwindowmanager_mutex.enterMutex();
  if(windows.front() != window)
  {
    for (std::list<GUIWindow*>::iterator i = windows.begin(),
         end = windows.end(); i != end; i++) {
      if((*i) == window)
      {
        windows.erase(i);
        windows.push_front(window);
        break;
      }
    }
  }

  guiwindowmanager_mutex.leaveMutex();
  render();
}

void pymms::gui::GUIWindowManager::set_active_window(int id)
{
  guiwindowmanager_mutex.enterMutex();
  if(windows.front()->get_id() != id)
  {
    for (std::list<GUIWindow*>::iterator i = windows.begin(),
         end = windows.end(); i != end; i++) {
      if((*i)->get_id() == id)
      {
        GUIWindow *window = (*i);

        windows.erase(i);
        windows.push_front(window);
	guiwindowmanager_mutex.leaveMutex();

	render();
        return;
      }
    }
  }

  guiwindowmanager_mutex.leaveMutex();
}

pymms::gui::GUIWindow *pymms::gui::GUIWindowManager::get_active_window()
{
  ost::MutexLock lock(guiwindowmanager_mutex);
  if(windows.size())
    return windows.front();
  else
    return 0;
}

pymms::gui::GUIWindow *pymms::gui::GUIWindowManager::get_window(int id)
{
  ost::MutexLock lock(guiwindowmanager_mutex);
  for (std::list<GUIWindow*>::iterator i = windows.begin(),
       end = windows.end(); i != end; i++) {
    if((*i)->get_id() == id)
    {
      return (*i);
    }
  }

  return 0;
}

void pymms::gui::GUIWindowManager::render()
{
  guiwindowmanager_mutex.enterMutex();
  GUIWindow *window = ((windows.size()>0)?windows.front():0);

  if(window)
  {
    renderer->prepare_new_image();

    window->render();

    renderer->draw_and_release("guiwindow");
  }

  guiwindowmanager_mutex.leaveMutex();
}

void pymms::gui::GUIWindowManager::render(int id)
{
  guiwindowmanager_mutex.enterMutex();
  GUIWindow *window = ((windows.size()>0)?windows.front():0);
  
  if(window && (window->get_id() == id))
  {
    renderer->prepare_new_image();

    window->render();

    renderer->draw_and_release("guiwindow");
  }

  guiwindowmanager_mutex.leaveMutex();
}
