#include "Python.h"
#include <stdio.h>
#include <unistd.h>

#include "libfspp.hpp"
#include "config.hpp"
#include "guiwindowscriptlauncher.hpp"

#include "pymms/mmspython.hpp"
#include "gui/guiwindowmanager.hpp"

#include "python_config.hpp"

GUIWindowScriptLauncher::GUIWindowScriptLauncher(std::string startdir)
{
  window_id = 100;
  
  conf = S_Config::get_instance();

  if(file_exists(startdir))
  {
    add_control(new GUIImageControl(0, 0, get_width(), get_height(), "general/standard_bg.jpg", 0));
    add_control(new GUIImageControl(25, 10, 45, 45, "python/script.png", 3));
    add_control(new GUILabelControl(90, 10, 200, 45, "Scripts", 4, "Vera", "0x284b64"));
    add_control(new GUIRectangleControl(50, 85, get_width()-100, get_height()-180, 3, 100, "0x20325a"));
    list = new GUIListControl(70, 105, get_width()-140, get_height()-200, 4, "movie/marked.png", "", "Vera", "0xffffff", "0xffffff", 8, "left", 25);
    add_control(list);

    set_focus(list);

    dirs.push_front(startdir);
    read_dir(dirs.front());
  }
}

GUIWindowScriptLauncher::~GUIWindowScriptLauncher()
{
  if(controls.size())
  {
    for (std::vector<GUIControl*>::iterator i = controls.begin(),
          end = controls.end(); i != end; i++) {
      (*i)->free_resources();
      delete *i;
    }
  }    
}

Simplefile GUIWindowScriptLauncher::addsimplefile(const std::string& name, const MyPair& filetype, bool from_media)
{
  Simplefile r;
  r.path = name;
  std::string tempname = name.substr(0, name.size()-((filetype.first).size()+1));
  std::string::size_type pos = tempname.rfind("/");

  if (pos != std::string::npos)
    r.name = tempname.substr(pos+1);
  else
    r.name = tempname;

  if (conf->p_convert())
    r.name = string_format::convert(r.name);

  if (from_media)
    r.type = "media-file";
  else
    r.type = filetype.second;

  return r;
}

std::vector<Simplefile> GUIWindowScriptLauncher::rdir(const std::string& argv)
{
  std::vector<Simplefile> cur_files;

  for (file_iterator<file_t, default_order> i (argv); i != i.end (); i.advance(false))
  {
    std::string filename = filesystem::FExpand(i->getName());

    if (isDirectory (filename))
    {
      // dir
      Simplefile r;
      r.name = filename.substr(argv[argv.size()-1] == '/' ? argv.size() : argv.size() + 1);
      if (conf->p_convert())
        r.name = string_format::convert(r.name);
      //r.name += "/";
      r.path = filename;
      r.type = "dir";
      cur_files.push_back(r);
    } else {
        //std::list<std::pair<std::string, std::string> > ft;
        //std::pair<std::string, std::string> p("py", "py");
        //ft.push_back(p);
        // file
	PythonConfig *python_conf = S_PythonConfig::get_instance();
        const MyPair filetype = check_type(filename, python_conf->p_filetypes_s());
        if (filetype != emptyMyPair)
          cur_files.push_back(addsimplefile(filename, filetype));
    }
  }

  return cur_files;
}

void GUIWindowScriptLauncher::read_dir(std::string path)
{
  list->free_resources();

  std::vector<Simplefile> dir = rdir(path);
  for(int i = 0; i < dir.size(); i++)
  {
    SLItem *temp = new SLItem(dir[i]);
    if(dir[i].type == "dir")
      temp->set_thumbimage("python/folder.png");
    else
      temp->set_thumbimage("python/exec.png");

    list->add_item(temp);
  }
}

bool GUIWindowScriptLauncher::on_action(std::string a)
{
  bool retval = false;

  retval = GUIWindow::on_action(a);

  if(a == "back")
  {
    if(dirs.size() > 1)
    {
      dirs.pop_front();
      read_dir(dirs.front());
     
      retval = true;
    }
  }

  if(a == "action")
  {
    if(list->get_selected_item())
    {
      Simplefile file = ((SLItem*)list->get_selected_item())->get_fileinfo();
      if(file.type == "dir")
      {
        dirs.push_front(file.path);
        read_dir(dirs.front());
      }
      else
      {
        pymms::MMSPython *mmspython = pymms::S_MMSPython::get_instance();
        mmspython->eval_file(file.path);
      }
    }

    retval = true;
  }
 
  S_WindowManager::get_instance()->render();
  
  return retval;
}
