
#include "pymms/mmspythread.hpp"
#include "pymms/mmspython.hpp"
#include <iostream>

pymms::MMSPyThread::MMSPyThread(void *pExecuter, PyThreadState* mainThreadState, int id)
{
  this->pExecuter = pExecuter;
  this->id = id;
  // get the global lock
  PyEval_AcquireLock();
  // get a reference to the PyInterpreterState
  PyInterpreterState *mainInterpreterState = mainThreadState->interp;
  // create a thread state object for this thread
  threadState = PyThreadState_New(mainInterpreterState);

  // clear the thread state and free the lock
  PyThreadState_Swap(NULL);
  PyEval_ReleaseLock();
}

pymms::MMSPyThread::~MMSPyThread()
{
  terminate();
}

int pymms::MMSPyThread::eval_file(std::string src)
{
  type = 'F';
  source = src;
  start();
  return 0;
}

void pymms::MMSPyThread::run()
{
  std::string sourcedir, path;
  
  PyEval_AcquireLock();
  // swap in my thread state
  PyThreadState_Swap(threadState);

  sourcedir = source.substr(0, source.rfind("/"));
  //path = sourcedir + "/;";
  //path +=  getenv("PYTHONPATH"); 

  //PySys_SetPath((char*)path.c_str());
  chdir(sourcedir.c_str());  

  if (type == 'F') {
    // run script from file
    FILE *fp = fopen(source.c_str(), "r");
    if (fp)
    {
      if (PyRun_SimpleFile(fp, source.c_str()) == -1)
      {
        std::cerr << "Script error\n";
        if (PyErr_Occurred())   
	  PyErr_Print();
      }
      fclose(fp);
    }
    else
     std::cerr << source << " not found!\n";
  }

  // clear the thread state and release our hold on the global interpreter
  PyThreadState_Swap(NULL);
  PyEval_ReleaseLock();
}

void pymms::MMSPyThread::final()
{
  // grab the lock
  PyEval_AcquireLock();
  // swap my thread state out of the interpreter
  PyThreadState_Swap(NULL);

  // clear out any cruft from thread state object
  PyThreadState_Clear(threadState);
  // delete my thread state object
  PyThreadState_Delete(threadState);
  // release the lock
  PyEval_ReleaseLock();

  ((MMSPython*)pExecuter)->set_done(id);

  delete this;
}
