#include "guipywindow.hpp"
#include "gui/guiwindowmanager.hpp"

#include "themes.hpp"

GUIPYWindow::GUIPYWindow() : pywindow_mutex("pywindow_mutex")
{
  py_action = "";

  Themes *themes = S_Themes::get_instance();
  default_bg = new GUIImageControl(0, 0, config->p_v_res(), config->p_h_res(), themes->startmenu_background, 0);
  add_control(default_bg);
}

GUIPYWindow::~GUIPYWindow()
{
  remove_control(default_bg);
  default_bg->free_resources();
  delete default_bg;
}

bool GUIPYWindow::on_action(std::string action)
{
  if(pywindow_mutex.tryEnterMutex())
  {
    GUIWindow::on_action(action);

    py_action = action;
    pulse_action_event();

    pywindow_mutex.leaveMutex();
  }

  return true;
}

void GUIPYWindow::make_pending_call(PyObject *o)
{
  ost::MutexLock lock(pywindow_mutex);
  PyObject_CallMethod(o, "on_action", "O", PyString_FromString(py_action.c_str()));

  S_WindowManager::get_instance()->render();
}

void GUIPYWindow::wait_for_action_event()
{
  action_event.wait();
  action_event.reset();
}

void GUIPYWindow::pulse_action_event()
{
  action_event.signal();
}
