#include <Python.h>
#include "structmember.h"
#include "pycontrol.hpp"

using namespace pymms::gui;

#ifdef __cplusplus
extern "C" {
#endif

namespace pymms {

namespace python {

static PyObject* LabelControl_New(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
  LabelControl *self;
  int x = 0, y = 0, w = 0, h = 0, l = 5, tos = 0;
  char *text = "";
  char *font = "Vera";
  char *rgb = "0xffffff";
  char *alignment = "left";
  
  static char *kwlist[] = {"x", "y", "width", "height", 
                           "text", "font", "rgb", "textoffset", "alignment", NULL};
  
  self = (LabelControl*)type->tp_alloc(type, 0);
  if (!self) 
    return NULL;
    
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "llll|sssls", kwlist, &x, &y, 
                                   &w, &h, &text, &font, &rgb, &tos, &alignment))
    return NULL;
    
  self->guicontrol = new GUILabelControl(x, y, w, h, text, l, font, rgb, tos, alignment);		

  return (PyObject*)self;
}

static PyObject* LabelControl_set_label(LabelControl *self, PyObject *args, PyObject *kwds)
{
  char *text = NULL;
  char *font = NULL;
  char *rgb = NULL;

  static char *kwlist[] = {"text", "font", "rgb", NULL};
    
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "|sss", kwlist, &text, &font, &rgb))	
    return NULL;

  if(text)
    ((GUILabelControl*)self->guicontrol)->set_label(text);
  if(font)
    ((GUILabelControl*)self->guicontrol)->set_labelfont(font);
  if(rgb)
    ((GUILabelControl*)self->guicontrol)->set_labelrgb(rgb);

  Py_INCREF(Py_None);
  return Py_None;
}

static void LabelControl_Dealloc(LabelControl* self)
{
  delete self->guicontrol;
  self->ob_type->tp_free((PyObject*)self);
}

static PyMethodDef LabelControl_methods[] = {
  {"set_label", (PyCFunction)LabelControl_set_label, METH_VARARGS | METH_KEYWORDS, 0},
  {NULL, NULL, 0, NULL}
};

PyTypeObject LabelControl_Type = {
	PyObject_HEAD_INIT(NULL)
	0,                         /*ob_size*/
	"mmsv2gui.LabelControl",    /*tp_name*/
	sizeof(LabelControl),      /*tp_basicsize*/
	0,                         /*tp_itemsize*/
	(destructor)LabelControl_Dealloc,/*tp_dealloc*/
	0,                         /*tp_print*/
	0,                         /*tp_getattr*/
	0,                         /*tp_setattr*/
	0,                         /*tp_compare*/
	0,                         /*tp_repr*/
	0,                         /*tp_as_number*/
	0,                         /*tp_as_sequence*/
	0,                         /*tp_as_mapping*/
	0,                         /*tp_hash */
	0,                         /*tp_call*/
	0,                         /*tp_str*/
	0,                         /*tp_getattro*/
	0,                         /*tp_setattro*/
	0,                         /*tp_as_buffer*/
	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
	0,       /* tp_doc */
	0,		                     /* tp_traverse */
	0,		                     /* tp_clear */
	0,		                     /* tp_richcompare */
	0,		                     /* tp_weaklistoffset */
	0,		                     /* tp_iter */
	0,		                     /* tp_iternext */
	LabelControl_methods,      /* tp_methods */
	0,                         /* tp_members */
	0,                         /* tp_getset */
	&Control_Type,             /* tp_base */
	0,                         /* tp_dict */
	0,                         /* tp_descr_get */
	0,                         /* tp_descr_set */
	0,                         /* tp_dictoffset */
	0,                         /* tp_init */
	0,                         /* tp_alloc */
	LabelControl_New,          /* tp_new */
};

}

}

#ifdef __cplusplus
}
#endif
