#include <Python.h>
#include "structmember.h"
#include "pycontrol.hpp"

using namespace pymms::gui;

#ifdef __cplusplus
extern "C" {
#endif

namespace pymms {

namespace python {

static PyObject* RectangleControl_New(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
  RectangleControl *self;
  int x = 0, y = 0, w = 0, h = 0, l = 2, alpha = 100;
  char *rgb = "0xffffff";
  
  static char *kwlist[] = {"x", "y", "width", "height", 
                           "alpha", "rgb", NULL};
  
  self = (RectangleControl*)type->tp_alloc(type, 0);
  if (!self) 
    return NULL;
    
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "llll|ls", kwlist, &x, &y, 
                                   &w, &h, &alpha, &rgb))
    return NULL;
    
  self->guicontrol = new GUIRectangleControl(x, y, w, h, l, alpha, rgb);		

  return (PyObject*)self;
}

static PyObject* RectangleControl_set_color(RectangleControl *self, PyObject *args, PyObject *kwds)
{
  char *rgb = NULL;
  int alpha = 0;

  static char *kwlist[] = {"alpha", "rgb", NULL};
    
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "|ls", kwlist, &alpha, &rgb))	
    return NULL;

  if(rgb)
    ((GUIRectangleControl*)self->guicontrol)->set_rgb(rgb);
  if(alpha)
    ((GUIRectangleControl*)self->guicontrol)->set_alpha(alpha);

  Py_INCREF(Py_None);
  return Py_None;
}

static void RectangleControl_Dealloc(RectangleControl* self)
{
  delete self->guicontrol;
  self->ob_type->tp_free((PyObject*)self);
}

static PyMethodDef RectangleControl_methods[] = {
  {"set_color", (PyCFunction)RectangleControl_set_color, METH_VARARGS | METH_KEYWORDS, 0},
  {NULL, NULL, 0, NULL}
};

PyTypeObject RectangleControl_Type = {
	PyObject_HEAD_INIT(NULL)
	0,                         /*ob_size*/
	"mmsv2gui.RectangleControl",    /*tp_name*/
	sizeof(RectangleControl),      /*tp_basicsize*/
	0,                         /*tp_itemsize*/
	(destructor)RectangleControl_Dealloc,/*tp_dealloc*/
	0,                         /*tp_print*/
	0,                         /*tp_getattr*/
	0,                         /*tp_setattr*/
	0,                         /*tp_compare*/
	0,                         /*tp_repr*/
	0,                         /*tp_as_number*/
	0,                         /*tp_as_sequence*/
	0,                         /*tp_as_mapping*/
	0,                         /*tp_hash */
	0,                         /*tp_call*/
	0,                         /*tp_str*/
	0,                         /*tp_getattro*/
	0,                         /*tp_setattro*/
	0,                         /*tp_as_buffer*/
	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
	0,       /* tp_doc */
	0,		                     /* tp_traverse */
	0,		                     /* tp_clear */
	0,		                     /* tp_richcompare */
	0,		                     /* tp_weaklistoffset */
	0,		                     /* tp_iter */
	0,		                     /* tp_iternext */
	RectangleControl_methods,      /* tp_methods */
	0,                         /* tp_members */
	0,                         /* tp_getset */
	&Control_Type,             /* tp_base */
	0,                         /* tp_dict */
	0,                         /* tp_descr_get */
	0,                         /* tp_descr_set */
	0,                         /* tp_dictoffset */
	0,                         /* tp_init */
	0,                         /* tp_alloc */
	RectangleControl_New,          /* tp_new */
};

}

}

#ifdef __cplusplus
}
#endif
