#include "keyboard.hpp"

#include "global.hpp"

#ifdef use_sdl
#include "common.hpp"
#include "renderer.hpp"
#include "sdlout.hpp"
#endif

using std::list;
using std::string;

static const char* convert_table[] = {
 ""     ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,"TAB","" ,"" ,"" ,"ENTER","" ,"" ,
 ""     ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,""   ,"" ,"" ,"" ,""     ,"" ,"" ,
 "SPACE","" ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,""   ,"" ,"" ,"" ,""     ,"" ,"" ,
 "0"    ,"1","2","3","4","5","6","7","8","9"  ,":",";","<","="    ,">","?",
 "@"    ,"A","B","C","D","E","F","G","H","I"  ,"J","K","L","M"    ,"N","O",
 "P"    ,"Q","R","S","T","U","V","W","X","Y"  ,"Z","[","" ,"]"    ,"^","_",
 ""     ,"a","b","c","d","e","f","g","h","i"  ,"j","k","l","m"    ,"n","o",
 "p"    ,"q","r","s","t","u","v","w","x","y"  ,"z","{","" ,"}"    ,"~","" ,
 ""     ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,""   ,"" ,"" ,"" ,""     ,"" ,"" ,
 ""     ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,""   ,"" ,"" ,"" ,""     ,"" ,"" ,
 ""     ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,""   ,"" ,"" ,"" ,""     ,"" ,"" ,
 ""     ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,""   ,"" ,"" ,"" ,""     ,"" ,"" ,
 ""     ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,""   ,"" ,"" ,"" ,""     ,"" ,"" ,
 ""     ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,""   ,"" ,"" ,"" ,""     ,"" ,"" ,
 ""     ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,""   ,"" ,"" ,"" ,""     ,"" ,"" ,
 ""     ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,"" ,""   ,"" ,"" ,"" ,""     ,"" ,"" ,

 "BACKSPACE","DEL" ,"INS" ,"HOME","END","PGUP","PGDOWN","ESC",""  ,""  ,""   ,""   ,""   ,""   ,""   ,"",
 "RIGHT"    ,"LEFT","DOWN","UP"  ,""   ,""    ,""      ,""   ,""  ,""  ,""   ,""   ,""   ,""   ,""   ,"",
 ""         ,""    ,""    ,""    ,""   ,""    ,""      ,""   ,""  ,""  ,""   ,""   ,""   ,""   ,""   ,"",
 ""         ,""    ,""    ,""    ,""   ,""    ,""      ,""   ,""  ,""  ,""   ,""   ,""   ,""   ,""   ,"",
 ""         ,"F1"  ,"F2"  ,"F3"  ,"F4" ,"F5"  ,"F6"    ,"F7" ,"F8","F9","F10","F11","F12","F13","F14","F15"};

Keyboard::Keyboard()
  : halfdelay_time(100), tty_mode(0)
{}

Keyboard::~Keyboard()
{ 
  terminate(); /* thread */ 
  normal(); 
}

bool Keyboard::init()
{
#ifdef use_sdl
  SDL_EnableKeyRepeat( 500, 30 );
#endif
  cbreak();

  return true;
}

string convert_int_to_string(int i)
{
  if (i >= 0 && i < 337) {
    string value = convert_table[i];
    return value;
  } else {
    char kbuf[10] = {0};
    snprintf(kbuf, sizeof(kbuf), "#%d", i);
    string keycode = string_format::convert(kbuf);
    return keycode;
 }
  return "";
}

#ifdef use_sdl
string Keyboard::getSDLkey()
{
  SDLdev *dev = reinterpret_cast<SDLdev*>(S_Render::get_instance()->device);

  if (dev) {
    dev->get_input();
    dev->get_input_event.wait();
    return dev->input_key;
  } else {
    std::cerr << "FIXME: this should not happen" << std::endl;
    return "";
  }
}
#endif

void Keyboard::run()
{
  InputMaster *input_master = S_InputMaster::get_instance();

  while (true) {
    
    S_Global::get_instance()->stop_bit_input_check(input);

    input_done_.signal();
    input_ready.wait();
    input_ready.reset();

  nogood:    

    string input_from_device;
#ifdef use_sdl
    if(SDL_WasInit(SDL_INIT_VIDEO)) {
      input_from_device = getSDLkey();
    } else {
#endif
      input_from_device = convert_int_to_string(getch2(halfdelay_time));
#ifdef use_sdl
    } // SDL_WasInit(...)
#endif  

    if (accept_all) {

      Input i;

      if (!input_master->find_input(input_from_device, i))
	i.key = input_from_device;

      input = i;

    } else {
      if (!input_master->find_input(input_from_device, input))
	goto nogood;
    }
  }
}

// one-char-at-a-time
void Keyboard::cbreak()
{
#ifdef use_sdl
  if(!SDL_WasInit(SDL_INIT_VIDEO)) {
#endif

  if (tty_mode == 0)
    {
      tcgetattr(0, &orig_tty);
      tty_mode = 1;
      new_tty = orig_tty;
    }

  new_tty.c_lflag &= ~(ICANON | ECHO);
  new_tty.c_cc[VMIN] = 1;
  new_tty.c_cc[VTIME] = 0;
  tcsetattr(0, TCSANOW, &new_tty);

#ifdef use_sdl
  } else /* SDL_WasInit(...) */
    SDL_EnableUNICODE(1);
#endif
}

// normal keyboard mode
void Keyboard::normal()
{
#ifdef use_sdl
  if(!SDL_WasInit(SDL_INIT_VIDEO)) {
#endif

  if (tty_mode == 1)
    {
      tcsetattr(0, TCSANOW, &orig_tty);
      new_tty = orig_tty;
    }

#ifdef use_sdl
  } else /* SDL_WasInit(...) */
    SDL_EnableUNICODE(0);
#endif
}

list<string> Keyboard::not_accepted_keys()
{
  list<string> unaccepted;
  unaccepted.push_back("DEL");
  unaccepted.push_back("INS");
  unaccepted.push_back("TAB");
  unaccepted.push_back("HOME");
  unaccepted.push_back("END");
  unaccepted.push_back("PGUP");
  unaccepted.push_back("PGDOWN");
  unaccepted.push_back("ESC");
  unaccepted.push_back("RIGHT");
  unaccepted.push_back("LEFT");
  unaccepted.push_back("DOWN");
  unaccepted.push_back("UP");
  unaccepted.push_back("ENTER");
  unaccepted.push_back("RSHIFT");
  unaccepted.push_back("LSHIFT");
  unaccepted.push_back("RCTRL");
  unaccepted.push_back("LCTRL");
  unaccepted.push_back("RALT");
  unaccepted.push_back("LALT");
  unaccepted.push_back("F1");
  unaccepted.push_back("F2");
  unaccepted.push_back("F3");
  unaccepted.push_back("F4");
  unaccepted.push_back("F5");
  unaccepted.push_back("F6");
  unaccepted.push_back("F7");
  unaccepted.push_back("F8");
  unaccepted.push_back("F9");
  unaccepted.push_back("F10");
  unaccepted.push_back("F11");
  unaccepted.push_back("F12");
  return unaccepted;
}
