#include "lirc.hpp"

#include "config.h"

#include "lirc_config.hpp"

#include "gettext.hpp"
#include "config.hpp"
#include "common.hpp"
#include "print.hpp"

#include "global.hpp"

#include <stdio.h>

#include <signal.h>
#include <fcntl.h>
#include <unistd.h>

#include <iostream>

#include "lirc/lirc_client.h"

// lirc
struct IR {
  char *buf;
  unsigned int code;
  unsigned int serial;
  char key[64];
  char remote[64];
  char rxkey;
};

Lirc::Lirc()
  : running(false), dump_fast(0), next_time(0), 
    this_time(0), flush(false)
{
  Config *conf = S_Config::get_instance();
  LircConfig *lirc_conf = S_LircConfig::get_instance();
  lirc_conf->parse_configuration_file(conf->p_homedir());

  timeinterv = lirc_conf->p_lirc_delay();

  running = true;
}

bool Lirc::init()
{
  if ((irsd = lirc_init("mms",1)) == -1) {
    DebugPrint perror(gettext("Could not connect to lirc daemon"), 
		      Print::DEBUGGING, DebugPrint::CRITICAL, "LIRC");
    return false;
  }

  int flags = fcntl(irsd, F_GETFL);
  flags |= O_NONBLOCK;
  fcntl(irsd, F_GETFL, &flags);
  running = true;
  return true;
}

Lirc::~Lirc()
{
  terminate(); /* thread */
  lirc_deinit();
}

void Lirc::suspend()
{
  running = false;
  lirc_deinit();
}

void Lirc::wake_up()
{
  init();

  Input i; 
  input = i;


  struct timeval tv;
  gettimeofday(&tv,NULL);
  next_time = (unsigned long long) tv.tv_sec*1000+tv.tv_usec/1000 + 200;
  flush=true;
  running = true;
}

void ir_parsebuf(IR * irs) {
  sscanf(irs->buf,"%x%x%s%s",&(irs->code),&irs->serial,irs->key,irs->remote);
}

void Lirc::run()
{
  InputMaster *input_master = S_InputMaster::get_instance();

  struct timeval tv;
  IR irdata;

  while (true) {

    S_Global::get_instance()->stop_bit_input_check(input);

    dump_fast = 2;
    
    input_done_.signal();
    input_ready.wait();
    input_ready.reset();
    
    char buffer[64];

  nogood:

    size_t len = read(irsd, buffer, 64);

    if (dump_fast > 0) {
      gettimeofday(&tv,NULL);
      this_time = (unsigned long long) tv.tv_sec*1000+tv.tv_usec/1000;
      if (next_time > this_time) {
        if (!flush)
	  --dump_fast;
        goto nogood;
      } else {
	next_time = timeinterv+this_time;
        flush=false;
      }
    }
    
    if (len < 0)
      continue;

    if (!running)
      continue;
    
    for(size_t i = 0; i<len; ++i) {
      if (buffer[i] == '\n') {
	buffer[i] = '\0';
	break;
      }
    }
    
    irdata.buf = buffer;
    
    ir_parsebuf(&irdata);

    Input temp;
    bool found_input = input_master->find_input(irdata.key, temp);
    
    if (irdata.serial > 0 && !temp.multiple)
      goto nogood;
  
    if (accept_all) {

      Input i;

      if (found_input)
	if (temp.mode == "search" || temp.mode == "general")
	  i = temp;

      i.key = irdata.key;
      input = i;

    } else {
      if (found_input)
	input = temp;
      else
	goto nogood;
    }

    DebugPrint perror("got command " + input.command + " ( " + 
		      conv::itos(irdata.serial) + " ) ", 
		      Print::INFO, DebugPrint::ALL, "LIRC");
  }
}

