#include "config.h"

#include "plugins.hpp"

#include "libfspp.hpp"

#include <signal.h>
#include <stdio.h>
#include <dlfcn.h>
#include <sys/stat.h>

#include <fstream>
#include <iostream>

using std::vector;
using std::string;

void Plugins::delete_plugins() 
{
  for (vector<FeaturePlugin*>::iterator iter=fp_data.begin(), iend=fp_data.end(); iter!=iend; ++iter) {
    //    void *handle = (*iter)->handle;
    (*iter)->destructor((*iter));
    //    dlclose(handle);
  }
  for (vector<AudioPlayerPlugin*>::iterator iter=audio_players.begin(), iend=audio_players.end(); 
       iter!=iend; ++iter) {
    void *handle = (*iter)->handle;
    (*iter)->destructor((*iter));
    dlclose(handle);
  }
  for (vector<MoviePlayerPlugin*>::iterator iter=movie_players.begin(), iend=movie_players.end(); 
       iter!=iend; ++iter) {
    void *handle = (*iter)->handle;
    (*iter)->destructor((*iter));
    dlclose(handle);
  }
  for (vector<PrinterPlugin*>::iterator iter=printers.begin(), iend=printers.end(); iter!=iend; ++iter) {
    void *handle = (*iter)->handle;
    (*iter)->destructor((*iter));
    dlclose(handle);
  }
  for (vector<InputPlugin*>::iterator iter=ip_data.begin(), iend=ip_data.end(); iter!=iend; ++iter) {
    void *handle = (*iter)->handle;
    (*iter)->destructor((*iter));
    dlclose(handle);
  }
  for (vector<OutputPlugin*>::iterator iter=op_data.begin(), iend=op_data.end(); iter!=iend; ++iter) {
    //    void *handle = (*iter)->handle;
    if ((*iter)->loaded()) {
      (*iter)->device->deinit();
    }
    (*iter)->destructor((*iter));
    // FIXME: probably something to do with sdl_quit
    //    dlclose(handle);
  }
}

Plugin *Plugins::add_plugin(const string& filename)
{
#ifdef mms_debug
  cerr << "Plugins::add_plugin() " << filename << endl;
#endif

  string base_filename = basename(filename.c_str());
#ifdef mms_debug
  cerr << "Plugins name is " << base_filename << endl;
#endif

  // search in all types if already loaded plugin

  // input plugin ?
  for (std::vector<InputPlugin*>::const_iterator l = ip_data.begin(); l != ip_data.end(); ++l)
    if (base_filename == basename((*l)->filename.c_str()))
      return NULL;
#ifdef mms_debug
  cerr << "Not an already loaded input" << endl;
#endif

  // output plugin ?
  for (std::vector<OutputPlugin*>::const_iterator l = op_data.begin(); l != op_data.end(); ++l)
    if (base_filename == basename((*l)->filename.c_str()))
      return NULL;

#ifdef mms_debug
  cerr << "Not an already loaded output plugin" << endl;
#endif

  // feature plugin ?
  for (std::vector<FeaturePlugin*>::const_iterator l = fp_data.begin(); l != fp_data.end(); ++l)
    if (base_filename == basename((*l)->filename.c_str()))
      return NULL;

#ifdef mms_debug
  cerr << "Not an already loaded feature plugin" << endl;
#endif

  // audio player plugin ?
  for (std::vector<AudioPlayerPlugin*>::const_iterator l = audio_players.begin(); 
       l != audio_players.end(); ++l)
    if (base_filename == basename((*l)->filename.c_str()))
      return NULL;

#ifdef mms_debug
  cerr << "Not an already loaded audio player plugin" << endl;
#endif

  // movie player plugin ?
  for (std::vector<MoviePlayerPlugin*>::const_iterator l = movie_players.begin(); 
       l != movie_players.end(); ++l)
    if (base_filename == basename((*l)->filename.c_str()))
      return NULL;

#ifdef mms_debug
  cerr << "Not an already loaded movie player plugin" << endl;
#endif

  // printer plugin ?
  for (std::vector<PrinterPlugin*>::const_iterator l = printers.begin(); 
       l != printers.end(); ++l)
    if (base_filename == basename((*l)->filename.c_str()))
      return NULL;

#ifdef mms_debug
  cerr << "Not an already loaded printer plugin" << endl;
#endif

  Plugin *p = load(filename);
  return p;
}

Plugin* Plugins::load(const string& filename)
{
  void *h;
  void *(*constructor) (void);
  void *(*destructor) (Plugin*);

  string base_filename = basename(filename.c_str());

  if ((h = dlopen(filename.c_str(), RTLD_NOW | RTLD_GLOBAL))) {

#ifdef mms_debug
    cerr << "Plugin loaded" << endl;
#endif

    // FIXME oldstyle cast - ISO C++ forbid it directly - howto do this now ??
    if ((constructor = (void *(*)())dlsym(h, "construct")) && ((destructor = (void *(*)(Plugin*))dlsym(h, "destroy"))))
      {
	Plugin *p = static_cast<Plugin *>(constructor());
	if (p) {
	  p->handle = h;
	  p->destructor = destructor;
	  p->filename = filename;
	  switch (p->type()) {
	  case IN_PLUGIN:
	  case OUT_PLUGIN:
	  case FEATURE_PLUGIN:
	  case AUDIO_PLAYER_PLUGIN:
	  case MOVIE_PLAYER_PLUGIN:
	  case PRINTER_PLUGIN:
	    return p;
	    break;
	  default:
	    cerr << "Unknown plugin type, removed: " << base_filename << endl;
	    delete p;
	    break;
	  }
	} else
	  cerr << "Plugin fail to pre-init: " << base_filename << endl;
      } else
	cerr << "Error: Does not look like a mmsv2 plugin !" << endl;
  } else
    cerr << dlerror() << endl;

  dlclose(h);

  return NULL;
}

bool prioritize_plugins(const FeaturePlugin *fp1, const FeaturePlugin *fp2)
{
  return fp1->priority() < fp2->priority();
}

void Plugins::scan()
{
  string dir = MMS_PLUGINS_DIR;

#ifdef mms_debug
  cerr << "Plugins scanning: " << dir << endl;
#endif

  for (file_iterator<file_t, default_order> i (dir); i != i.end (); i.advance(false)) {

    string filename = i->getName();

#ifdef debug_mms
    cerr <<"Plugins Scan:: trying "<< filename << endl;
#endif

    int p;
    struct stat statbuf;

    if (!stat(filename.c_str(), &statbuf) && S_ISREG(statbuf.st_mode))
      if ((p=filename.rfind('.'))>0 && (filename.substr(p) == ".so")) {
        Plugin *plugin = add_plugin(filename);
        if (plugin) {
	  switch (plugin->type()) {
	  case IN_PLUGIN:{
	    InputPlugin *ip = static_cast<InputPlugin *>(plugin);
	    cerr << "Input plugin: " << ip->plugin_name() << " loaded" << endl;
	    ip_data.push_back(ip);
	    break;
	  }
	  case OUT_PLUGIN:{
	    OutputPlugin *op = static_cast<OutputPlugin *>(plugin);
	    cerr << "Output plugin: " << op->plugin_name() << " loaded" << endl;
	    op_data.push_back(op);
	    break;
	  }
	  case FEATURE_PLUGIN:{
	    FeaturePlugin *fp = static_cast<FeaturePlugin *>(plugin);
	    cerr << "Feature plugin: " << fp->plugin_name() << " loaded" << endl;
	    fp_data.push_back(fp);
	    break;
	  }
	  case AUDIO_PLAYER_PLUGIN:{
	    AudioPlayerPlugin *ap = static_cast<AudioPlayerPlugin *>(plugin);
	    cerr << "Audio player plugin: " << ap->plugin_name() << " loaded" << endl;
	    audio_players.push_back(ap);
	    break;
	  }
	  case MOVIE_PLAYER_PLUGIN:{
	    MoviePlayerPlugin *vp = static_cast<MoviePlayerPlugin *>(plugin);
	    cerr << "Movie player plugin: " << vp->plugin_name() << " loaded" << endl;
	    movie_players.push_back(vp);
	    break;
	  }
	  case PRINTER_PLUGIN:{
	    PrinterPlugin *pp = static_cast<PrinterPlugin *>(plugin);
	    cerr << "Printer plugin: " << pp->plugin_name() << " loaded" << endl;
	    printers.push_back(pp);
	    break;
	  }
	  default: cerr << "Unknown plugin type " << filename << endl;
	  }
	} else
	  cerr << "ERROR loading Plugin:" << filename << endl;
      }
  }

  // sort feature plugins by priority
  std::sort(fp_data.begin(), fp_data.end(), prioritize_plugins);
}

Plugin* Plugins::find(const string& name)
{
  for (std::vector<FeaturePlugin*>::const_iterator l = fp_data.begin(), lend=fp_data.end(); l != lend; ++l)
    if (name == (*l)->plugin_name())
      return *l;
  for (std::vector<OutputPlugin*>::const_iterator l = op_data.begin(), lend = op_data.end(); l != lend; ++l)
    if (name == (*l)->plugin_name())
      return *l;
  for (std::vector<InputPlugin*>::const_iterator l = ip_data.begin(), lend = ip_data.end(); l != lend; ++l)
    if (name == (*l)->plugin_name())
      return *l;
  for (std::vector<AudioPlayerPlugin*>::const_iterator l = audio_players.begin(), lend = audio_players.end(); l != lend; ++l)
    if (name == (*l)->plugin_name())
      return *l;
  for (std::vector<MoviePlayerPlugin*>::const_iterator l = movie_players.begin(), lend = movie_players.end(); l != lend; ++l)
    if (name == (*l)->plugin_name())
      return *l;
  for (std::vector<PrinterPlugin*>::const_iterator l = printers.begin(), lend = printers.end(); l != lend; ++l)
    if (name == (*l)->plugin_name())
      return *l;
  return 0;
}
