#ifndef SEARCH_HPP
#define SEARCH_HPP

#include "module.hpp"
#include "functor.hpp"

#include "singleton.hpp"

#include <string>
#include <vector>

class SearchBaseGenList
{
public:
  virtual std::vector<std::pair<std::string, int> > call(const std::string&) = 0;
  virtual ~SearchBaseGenList() {};
};

template <typename TClass>
class SearchGenList : public SearchBaseGenList
{
 private:
  std::vector<std::pair<std::string, int> > (TClass::*fpt)(const std::string&);
  TClass* pt2Object;

 public:
  
  SearchGenList(TClass* _pt2Object, std::vector<std::pair<std::string, int> >(TClass::*_fpt)(const std::string&))
  { pt2Object = _pt2Object; fpt=_fpt; };

  std::vector<std::pair<std::string, int> > call(const std::string& s)
  { return (*pt2Object.*fpt)(s); };
};

class EnterBaseFunctor
{
public:
  virtual void call(int) = 0;
  virtual ~EnterBaseFunctor() {};
};

template <typename TClass>
class EnterFunctor : public EnterBaseFunctor
{
 private:
  void (TClass::*fpt)(int);
  TClass* pt2Object;
  
 public:
  EnterFunctor(TClass* _pt2Object, void(TClass::*_fpt)(int))
  { pt2Object = _pt2Object;  fpt=_fpt; };

  void call(int el)
  { (*pt2Object.*fpt)(el);};
};


class SearchModule
{
public:
  SearchBaseGenList *list_gen;
  EnterBaseFunctor *enter_func;
  std::string name;

  SearchModule(const std::string& n, SearchBaseGenList* sgl, EnterBaseFunctor* e_f);
};

class result
{
public:
  bool operator==(const result& r) const
  {
    return (module == r.module && name == r.name && id == r.id);
  }
  
  std::string module;
  std::string name;
  int id;
};

class Search : public Module
{
private:
  std::vector<SearchModule> modules;

  std::vector<result> results;

  std::string cur_module;

  std::string search_word;
  std::string search_module;
  int pos;

  ExtraMenu em;

  void set_module(const std::string& mod);

  void print(const std::string& search_word);
  void print_element(const result& r, const result& position, int& y);

public: 
  Search();

  void register_module(const SearchModule& s);
  
  Options* get_opts() 
  { return 0; }

  void startup_updater()
  {}

  std::string mainloop();
};

typedef Singleton<Search> S_Search;

#endif
