#ifndef UPDATER_HPP
#define UPDATER_HPP

#include <cc++/thread.h>

#include "plugins/output/image.hpp"

#include "functor.hpp"
#include "singleton.hpp"

#include <time.h>

#include <list>
#include <string>
#include <cassert>

class TimeElement
{
public:
  TimeElement(const std::string& n, TimeFi* t, TimeFv* call_back)
    : name(n), active(true)
  {
    time_f = t;
    call_back_f = call_back;
  }
  
  TimeFi *time_f;
  TimeFv *call_back_f;
  std::string name;
  struct timeval next_time;
  bool active;
};

class TriggerElement
{
public:
  TriggerElement(const std::string& n, const std::list<std::string>& trig, TimeFv* t, TimeFv* c)
    : name(n), trigger(trig)
  {
    assert(t != 0);
    // clean_up can be empty if it's time controlled instead

    trigger_func = t;
    clean_up = c;
  }
  
  TimeFv *trigger_func;
  TimeFv *clean_up;
  std::string name;
  std::list<std::string> trigger;
};

class Updater : public ost::Thread
{
public:
  Updater();
  ~Updater() { terminate(); }

  void activate();
  void deactivate();

  class Timer 
  {
  public:
    void add(TimeElement te);
    void del(std::string name);

    // for individual elements
    void activate(const std::string& name);
    void deactivate(const std::string& name);
    void run_and_deactivate(const std::string& name);
    bool status(const std::string& name);

    struct timeval current_time;
    
    std::list<TimeElement> elements;

    ost::Mutex elements_mut;

    struct timeval set_timeout(const struct timeval& t, int wait);
  };

  class Trigger
  {
  public:
    void add(const TriggerElement& te);
    // cleanup
    void del(const std::string& name);
    // while running
    void trigger_input(const std::string& trigger);

  private:
    ost::Mutex elements_mut;

    std::list<TriggerElement> elements;
  };

  Timer timer;
  Trigger trigger;

  Overlay progressbar;
  int status_progressbar;
  int total_progressbar;
  std::string header;

private:
  struct timeval diff_timeval(const struct timeval& t1, const struct timeval& t2);

  void run();
  
  virtual void set_nicelevel() = 0;

  bool running;
};

class BackgroundUpdater : public Updater
{
private:
  void set_nicelevel();
};

class ScreenUpdater : public Updater
{
private:
  void set_nicelevel();
};

typedef Singleton<ScreenUpdater> S_ScreenUpdater;
typedef Singleton<BackgroundUpdater> S_BackgroundUpdater;

#endif
